@@ -23,9 +23,9 @@
 
 #include <unistd.h>
 #include <dlfcn.h>
-#include <iostream>
 #include <stdlib.h>
-#include <vector>
+
+using namespace std;
 
 #include ASPELL_INCLUDE
 
@@ -33,7 +33,7 @@
 #include "execmd.h"
 #include "rclaspell.h"
 #include "debuglog.h"
-
+#include "unacpp.h"
 #include "ptmutex.h"
 
 // Just a place where we keep the Aspell library entry points together
@@ -258,15 +258,20 @@ public:
     {}
     void newData() {
 	while (m_db.termWalkNext(m_tit, *m_input)) {
-	    // Prefixed terms are also somewhere else without the suffix,
-	    // skip them
-	    if (m_input->empty() || 
-		('A' <= m_input->at(0) && m_input->at(0) <= 'Z'))
-		continue;
-	    if (!Rcl::Db::isSpellingCandidate(*m_input))
+	    LOGDEB2(("Aspell::buildDict: term: [%s]\n", m_input->c_str()));
+	    if (!Rcl::Db::isSpellingCandidate(*m_input)) {
+		LOGDEB2(("Aspell::buildDict: SKIP\n"));
 		continue;
+	    }
+	    if (!o_index_stripchars) {
+		string lower;
+		if (!unacmaybefold(*m_input, lower, "UTF-8", UNACOP_FOLD))
+		    continue;
+		m_input->swap(lower);
+	    }
 	    // Got a non-empty sort-of appropriate term, let's send it to
 	    // aspell
+	    LOGDEB2(("Apell::buildDict: SEND\n"));
 	    m_input->append("\n");
 	    return;
 	}
@@ -284,28 +289,53 @@ bool Aspell::buildDict(Rcl::Db &db, string &reason)
     // We create the dictionary by executing the aspell command:
     // aspell --lang=[lang] create master [dictApath]
     ExecCmd aspell;
-    list<string> args;
+    vector<string> args;
     args.push_back(string("--lang=")+ m_lang);
     args.push_back("--encoding=utf-8");
     args.push_back("create");
     args.push_back("master");
     args.push_back(dicPath());
-    aspell.setStderr("/dev/null");
+
+    // Have to disable stderr, as numerous messages about bad strings are
+    // printed. We'd like to keep errors about missing databases though, so
+    // make it configurable for diags
+    bool keepStderr = false;
+    m_config->getConfParam("aspellKeepStderr", &keepStderr);
+    if (!keepStderr)
+	aspell.setStderr("/dev/null");
 
     Rcl::TermIter *tit = db.termWalkOpen();
     if (tit == 0) {
 	reason = "termWalkOpen failed\n";
 	return false;
     }
-
     string termbuf;
     AspExecPv pv(&termbuf, tit, db);
     aspell.setProvide(&pv);
-
+    
     if (aspell.doexec(m_data->m_exec, args, &termbuf)) {
-	reason = string("aspell dictionary creation command failed.\n"
-			"One possible reason might be missing language "
-			"data files for lang = ") + m_lang;
+	ExecCmd cmd;
+	args.clear();
+	args.push_back("dicts");
+	string dicts;
+	bool hasdict = false;
+	if (cmd.doexec(m_data->m_exec, args, 0, &dicts)) {
+	    vector<string> vdicts;
+	    stringToTokens(dicts, vdicts, "\n\r\t ");
+	    if (find(vdicts.begin(), vdicts.end(), m_lang) != vdicts.end()) {
+		hasdict = true;
+	    }
+	}
+	if (hasdict)
+	    reason = string(
+		"\naspell dictionary creation command failed. Reason unknown.\n"
+		"Try to set aspellKeepStderr = 1 in recoll.conf, and execute \n"
+		"the indexing command in a terminal to see the aspell "
+		"diagnostic output.\n");
+	else
+	    reason = string("aspell dictionary creation command failed.\n"
+			    "One possible reason might be missing language "
+			    "data files for lang = ") + m_lang;
 	return false;
     }
     db.termWalkClose(tit);
@@ -340,17 +370,27 @@ bool Aspell::make_speller(string& reason)
     return true;
 }
 
-bool Aspell::check(Rcl::Db &db, const string &term, string& reason)
+bool Aspell::check(const string &iterm, string& reason)
 {
-    LOGDEB2(("Aspell::check [%s]\n", term.c_str()));
+    LOGDEB2(("Aspell::check [%s]\n", iterm.c_str()));
+    string mterm(iterm);
 
     if (!ok() || !make_speller(reason))
 	return false;
-    if (term.empty())
+    if (iterm.empty())
         return true; //??
 
+    if (!o_index_stripchars) {
+	string lower;
+	if (!unacmaybefold(mterm, lower, "UTF-8", UNACOP_FOLD)) {
+	    LOGERR(("Aspell::check : cant lowercase input\n"));
+	    return false;
+	}
+	mterm.swap(lower);
+    }
+
     int ret = aapi.aspell_speller_check(m_data->m_speller, 
-                                        term.c_str(), term.length());
+                                        mterm.c_str(), mterm.length());
     reason.clear();
     switch (ret) {
     case 0: return false;
@@ -363,19 +403,29 @@ bool Aspell::check(Rcl::Db &db, const string &term, string& reason)
     }
 }
 
-bool Aspell::suggest(Rcl::Db &db, const string &term, 
+bool Aspell::suggest(Rcl::Db &db, const string &_term, 
                      list<string>& suggestions, string& reason)
 {
     if (!ok() || !make_speller(reason))
 	return false;
-    if (term.empty())
+    string mterm(_term);
+    if (mterm.empty())
         return true; //??
 
+    if (!o_index_stripchars) {
+	string lower;
+	if (!unacmaybefold(mterm, lower, "UTF-8", UNACOP_FOLD)) {
+	    LOGERR(("Aspell::check : cant lowercase input\n"));
+	    return false;
+	}
+	mterm.swap(lower);
+    }
+
     AspellCanHaveError *ret;
 
     const AspellWordList *wl = 
 	aapi.aspell_speller_suggest(m_data->m_speller, 
-                                    term.c_str(), term.length());
+                                    mterm.c_str(), mterm.length());
     if (wl == 0) {
 	reason = aapi.aspell_speller_error_message(m_data->m_speller);
 	return false;
@@ -383,14 +433,13 @@ bool Aspell::suggest(Rcl::Db &db, const string &term,
     AspellStringEnumeration *els = aapi.aspell_word_list_elements(wl);
     const char *word;
     while ((word = aapi.aspell_string_enumeration_next(els)) != 0) {
-	// stemDiffers checks that the word exists (we don't want
-	// aspell computed stuff, only exact terms from the dictionary),
-	// and that it stems differently to the base word (else it's not 
-	// useful to expand the search). Or is it ? 
-        // ******** This should depend if
-	// stemming is turned on or not for querying  *******
-	string sw(word);
-	if (db.termExists(sw) && db.stemDiffers("english", sw, term))
+	// Check that the word exists in the index (we don't want
+	// aspell computed stuff, only exact terms from the
+	// dictionary).  We used to also check that it stems
+	// differently from the base word but this is complicated
+	// (stemming on/off + language), so we now leave this to the
+	// caller.
+	if (db.termExists(word))
 	    suggestions.push_back(word);
     }
     aapi.delete_aspell_string_enumeration(els);
@@ -423,12 +472,12 @@ using namespace std;
 
 static char *thisprog;
 RclConfig *rclconfig;
-Rcl::Db rcldb;
 
 static char usage [] =
 " -b : build dictionary\n"
 " -s <term>: suggestions for term\n"
-"\n\n"
+" -c <term>: check term\n"
+"\n"
 ;
 static void
 Usage(void)
@@ -441,6 +490,7 @@ static int     op_flags;
 #define OPT_MOINS 0x1
 #define OPT_s	  0x2 
 #define OPT_b	  0x4 
+#define OPT_c     0x8
 
 int main(int argc, char **argv)
 {
@@ -457,6 +507,10 @@ int main(int argc, char **argv)
 	while (**argv)
 	    switch (*(*argv)++) {
 	    case 'b':	op_flags |= OPT_b; break;
+	    case 'c':	op_flags |= OPT_c; if (argc < 2)  Usage();
+		word = *(++argv);
+		argc--; 
+		goto b1;
 	    case 's':	op_flags |= OPT_s; if (argc < 2)  Usage();
 		word = *(++argv);
 		argc--; 
@@ -482,7 +536,9 @@ int main(int argc, char **argv)
 	exit(1);
     }
 
-    if (!rcldb.open(dbdir, Rcl::Db::DbRO, 0)) {
+    Rcl::Db rcldb(rclconfig);
+
+    if (!rcldb.open(Rcl::Db::DbRO, 0)) {
 	fprintf(stderr, "Could not open database in %s\n", dbdir.c_str());
 	exit(1);
     }
@@ -498,6 +554,18 @@ int main(int argc, char **argv)
 	    cerr << "buildDict failed: " << reason << endl;
 	    exit(1);
 	}
+    } else if (op_flags & OPT_c) {
+	bool ret = aspell.check(word, reason);
+	if (!ret && reason.size()) {
+	    cerr << "Aspell error: " << reason << endl;
+	    return 1;
+	}
+	cout << word;
+	if (ret) {
+	    cout << " is in dictionary" << endl;
+	} else {
+	    cout << " not in dictionary" << endl;
+	}
     } else {
 	list<string> suggs;
 	if (!aspell.suggest(rcldb, word, suggs, reason)) {
