@@ -15,15 +15,19 @@
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 #ifndef TEST_RCLCONFIG
+#include "autoconfig.h"
+
 #include <unistd.h>
 #include <fcntl.h>
 #include <stdio.h>
 #include <errno.h>
 #include <langinfo.h>
 #include <limits.h>
+#include <sys/param.h>
 
-#include <set>
 #include <algorithm>
+#include <list>
+using std::list;
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -32,8 +36,10 @@
 #endif
 
 #include <iostream>
+#include <sstream>
 #include <cstdlib>
 #include <cstring>
+using namespace std;
 
 #include "cstr.h"
 #include "pathut.h"
@@ -44,17 +50,18 @@
 #include "textsplit.h"
 #include "readfile.h"
 #include "fstreewalk.h"
+#include "cpuconf.h"
 
-#ifndef NO_NAMESPACES
-using namespace std;
-#endif /* NO_NAMESPACES */
+typedef pair<int,int> RclPII;
 
-#ifndef MIN
-#define MIN(A,B) (((A)<(B)) ? (A) : (B))
-#endif
-#ifndef MAX
-#define MAX(A,B) (((A)>(B)) ? (A) : (B))
-#endif
+// Static, logically const, RclConfig members are initialized once from the
+// first object build during process initialization.
+
+// We default to a case- and diacritics-less index for now
+bool o_index_stripchars = true;
+
+string RclConfig::o_localecharset; 
+string RclConfig::o_origcwd; 
 
 bool ParamStale::needrecompute()
 {
@@ -77,6 +84,7 @@ bool ParamStale::needrecompute()
     }
     return false;
 }
+
 void ParamStale::init(RclConfig *rconf, ConfNull *cnf, const string& nm)
 {
     parent = rconf;
@@ -93,6 +101,7 @@ void RclConfig::zeroMe() {
     mimeconf = 0; 
     mimeview = 0; 
     m_fields = 0;
+    m_ptrans = 0;
     m_stopsuffixes = 0;
     m_maxsufflen = 0;
     m_stpsuffstate.init(this, 0, "recoll_noindex");
@@ -100,7 +109,7 @@ void RclConfig::zeroMe() {
     m_rmtstate.init(this, 0, "indexedmimetypes");
 }
 
-bool RclConfig::isDefaultConfig()
+bool RclConfig::isDefaultConfig() const
 {
     string defaultconf = path_cat(path_canon(path_home()), ".recoll/");
     string specifiedconf = path_canon(m_confdir);
@@ -111,6 +120,16 @@ bool RclConfig::isDefaultConfig()
 RclConfig::RclConfig(const string *argcnf)
 {
     zeroMe();
+
+    if (o_origcwd.empty()) {
+	char buf[MAXPATHLEN];
+	if (getcwd(buf, MAXPATHLEN)) {
+	    o_origcwd = string(buf);
+	} else {
+	    fprintf(stderr, "recollxx: can't retrieve current working directory: relative path translations will fail\n");
+	}
+    }
+
     // Compute our data dir name, typically /usr/local/share/recoll
     const char *cdatadir = getenv("RECOLL_DATADIR");
     if (cdatadir == 0) {
@@ -135,7 +154,7 @@ RclConfig::RclConfig(const string *argcnf)
     } else {
 	const char *cp = getenv("RECOLL_CONFDIR");
 	if (cp) {
-	    m_confdir = cp;
+	    m_confdir = path_canon(cp);
 	} else {
 	    autoconfdir = true;
 	    m_confdir = path_cat(path_home(), ".recoll/");
@@ -159,6 +178,34 @@ RclConfig::RclConfig(const string *argcnf)
 	    return;
     }
 
+    // This can't change once computed inside a process. It would be
+    // nicer to move this to a static class initializer to avoid
+    // possible threading issues but this doesn't work (tried) as
+    // things would not be ready. In practise we make sure that this
+    // is called from the main thread at once, by constructing a config
+    // from recollinit
+    if (o_localecharset.empty()) {
+	const char *cp;
+	cp = nl_langinfo(CODESET);
+	// We don't keep US-ASCII. It's better to use a superset
+	// Ie: me have a C locale and some french file names, and I
+	// can't imagine a version of iconv that couldn't translate
+	// from iso8859?
+	// The 646 thing is for solaris. 
+	if (cp && *cp && strcmp(cp, "US-ASCII") 
+#ifdef sun
+	    && strcmp(cp, "646")
+#endif
+	    ) {
+	    o_localecharset = string(cp);
+	} else {
+	    // Use cp1252 instead of iso-8859-1, it's a superset.
+	    o_localecharset = string(cstr_cp1252);
+	}
+	LOGDEB1(("RclConfig::getDefCharset: localecharset [%s]\n",
+		 o_localecharset.c_str()));
+    }
+
     m_cdirs.push_back(m_confdir);
     m_cdirs.push_back(path_cat(m_datadir, "examples"));
     string cnferrloc = m_confdir + " or " + path_cat(m_datadir, "examples");
@@ -177,7 +224,9 @@ RclConfig::RclConfig(const string *argcnf)
 	m_reason = string("No/bad mimeconf in: ") + cnferrloc;
 	return;
     }
-    mimeview = new ConfStack<ConfSimple>("mimeview", m_cdirs, true);
+    mimeview = new ConfStack<ConfSimple>("mimeview", m_cdirs, false);
+    if (mimeview == 0)
+	mimeview = new ConfStack<ConfSimple>("mimeview", m_cdirs, true);
     if (mimeview == 0 || !mimeview->ok()) {
 	m_reason = string("No/bad mimeview in: ") + cnferrloc;
 	return;
@@ -185,6 +234,12 @@ RclConfig::RclConfig(const string *argcnf)
     if (!readFieldsConfig(cnferrloc))
 	return;
 
+    // Default is no threading
+    m_thrConf = create_vector<RclPII>
+	(RclPII(-1, 0))(RclPII(-1, 0))(RclPII(-1, 0));
+
+    m_ptrans = new ConfSimple(path_cat(m_confdir, "ptrans").c_str());
+
     m_ok = true;
     setKeyDir(cstr_null);
 
@@ -239,6 +294,12 @@ bool RclConfig::updateMainConfig()
 	FsTreeWalker::setNoFnmPathname();
     }
 
+    static int m_index_stripchars_init = 0;
+    if (!m_index_stripchars_init) {
+	getConfParam("indexStripChars", &o_index_stripchars);
+	m_index_stripchars_init = 1;
+    }
+
     return true;
 }
 
@@ -264,11 +325,11 @@ void RclConfig::setKeyDir(const string &dir)
     if (m_conf == 0)
 	return;
 
-    if (!m_conf->get("defaultcharset", defcharset, m_keydir))
-	defcharset.erase();
+    if (!m_conf->get("defaultcharset", m_defcharset, m_keydir))
+	m_defcharset.erase();
 }
 
-bool RclConfig::getConfParam(const string &name, int *ivp)
+bool RclConfig::getConfParam(const string &name, int *ivp) const
 {
     string value;
     if (!getConfParam(name, value))
@@ -282,7 +343,7 @@ bool RclConfig::getConfParam(const string &name, int *ivp)
     return true;
 }
 
-bool RclConfig::getConfParam(const string &name, bool *bvp)
+bool RclConfig::getConfParam(const string &name, bool *bvp) const
 {
     if (!bvp) 
 	return false;
@@ -295,17 +356,7 @@ bool RclConfig::getConfParam(const string &name, bool *bvp)
     return true;
 }
 
-bool RclConfig::getConfParam(const string &name, vector<string> *svvp)
-{
-    if (!svvp) 
-	return false;
-    svvp->clear();
-    string s;
-    if (!getConfParam(name, s))
-	return false;
-    return stringToStrings(s, *svvp);
-}
-bool RclConfig::getConfParam(const string &name, list<string> *svvp)
+bool RclConfig::getConfParam(const string &name, vector<string> *svvp) const
 {
     if (!svvp) 
 	return false;
@@ -316,15 +367,116 @@ bool RclConfig::getConfParam(const string &name, list<string> *svvp)
     return stringToStrings(s, *svvp);
 }
 
-list<string> RclConfig::getTopdirs()
+bool RclConfig::getConfParam(const string &name, vector<int> *vip) const
 {
-    list<string> tdl;
+    if (!vip) 
+	return false;
+    vip->clear();
+    vector<string> vs;
+    if (!getConfParam(name, &vs))
+	return false;
+    vip->reserve(vs.size());
+    for (unsigned int i = 0; i < vs.size(); i++) {
+	char *ep;
+	vip->push_back(strtol(vs[i].c_str(), &ep, 0));
+	if (ep == vs[i].c_str()) {
+	    LOGDEB(("RclConfig::getConfParam: bad int value in [%s]\n",
+		    name.c_str()));
+	    return false;
+	}
+    }
+    return true;
+}
+
+void RclConfig::initThrConf()
+{
+    // Default is no threading
+    m_thrConf = create_vector<RclPII>
+	(RclPII(-1, 0))(RclPII(-1, 0))(RclPII(-1, 0));
+
+    vector<int> vq;
+    vector<int> vt;
+    if (!getConfParam("thrQSizes", &vq)) {
+	LOGINFO(("RclConfig::initThrConf: no thread info (queues)\n"));
+	goto out;
+    }
+
+    // If the first queue size is 0, autoconf is requested.
+    if (vq.size() > 0 && vq[0] == 0) {
+	LOGDEB(("RclConfig::initThrConf: autoconf requested\n"));
+	CpuConf cpus;
+	if (!getCpuConf(cpus) || cpus.ncpus < 1) {
+	    LOGERR(("RclConfig::initThrConf: could not retrieve cpu conf\n"));
+	    cpus.ncpus = 1;
+	}
+	// Arbitrarily set threads config based on number of CPUS. This also
+	// depends on the IO setup actually, so we're bound to be wrong...
+	if (cpus.ncpus == 1) {
+	    // Somewhat counter-intuitively (because of possible IO//)
+	    // it seems that the best config here is no threading
+	} else if (cpus.ncpus < 4) {
+	    // Untested so let's guess...
+	    m_thrConf = create_vector<RclPII>
+		(RclPII(2, 2))(RclPII(2, 2))(RclPII(2, 1));
+	} else if (cpus.ncpus < 6) {
+	    m_thrConf = create_vector<RclPII>
+		(RclPII(2, 4))(RclPII(2, 2))(RclPII(2, 1));
+	} else {
+	    m_thrConf = create_vector<RclPII>
+		(RclPII(2, 5))(RclPII(2, 3))(RclPII(2, 1));
+	}
+	goto out;
+    } else if (vq.size() > 0 && vq[0] < 0) {
+	// threads disabled by config
+	goto out;
+    }
+
+    if (!getConfParam("thrTCounts", &vt) ) {
+	LOGINFO(("RclConfig::initThrConf: no thread info (threads)\n"));
+	goto out;
+    }
+
+    if (vq.size() != 3 || vt.size() != 3) {
+	LOGINFO(("RclConfig::initThrConf: bad thread info vector sizes\n"));
+	goto out;
+    }
+
+    // Normal case: record info from config
+    m_thrConf.clear();
+    for (unsigned int i = 0; i < 3; i++) {
+	m_thrConf.push_back(RclPII(vq[i], vt[i]));
+    }
+
+out:
+    ostringstream sconf;
+    for (unsigned int i = 0; i < 3; i++) {
+	sconf << "(" << m_thrConf[i].first << ", " << m_thrConf[i].second <<
+	    ") ";
+    }
+
+    LOGDEB(("RclConfig::initThrConf: chosen config (ql,nt): %s\n", 
+	    sconf.str().c_str()));
+}
+
+pair<int,int> RclConfig::getThrConf(ThrStage who) const
+{
+    if (m_thrConf.size() != 3) {
+	LOGERR(("RclConfig::getThrConf: bad data in rclconfig\n"));
+	return pair<int,int>(-1,-1);
+    }
+    return m_thrConf[who];
+}
+
+vector<string> RclConfig::getTopdirs() const
+{
+    vector<string> tdl;
     if (!getConfParam("topdirs", &tdl)) {
-	LOGERR(("RclConfig::getTopdirs: no top directories in config or bad list format\n"));
+	LOGERR(("RclConfig::getTopdirs: no top directories in config or "
+		"bad list format\n"));
 	return tdl;
     }
 
-    for (list<string>::iterator it = tdl.begin(); it != tdl.end(); it++) {
+    for (vector<string>::iterator it = tdl.begin(); it != tdl.end(); it++) {
 	*it = path_tildexpand(*it);
 	*it = path_canon(*it);
     }
@@ -336,75 +488,19 @@ list<string> RclConfig::getTopdirs()
 //  If defcharset was set (from the config or a previous call, this
 //   is done in setKeydir), use it.
 //  Else, try to guess it from the locale
-//  Use iso8859-1 as ultimate default
+//  Use cp1252 (as a superset of iso8859-1) as ultimate default
 //
 // For filenames, same thing except that we do not use the config file value
 // (only the locale).
-const string& RclConfig::getDefCharset(bool filename) 
+const string& RclConfig::getDefCharset(bool filename) const
 {
-    // This can't change once computed inside a process. It would be
-    // nicer to move this to a static class initializer to avoid
-    // possible threading issues but this doesn't work (tried) as
-    // things would not be ready. In practise we make sure that this
-    // is called from the main thread at once, by calling
-    // getDefCharset from recollinit
-    static string localecharset; 
-    if (localecharset.empty()) {
-	const char *cp;
-	cp = nl_langinfo(CODESET);
-	// We don't keep US-ASCII. It's better to use a superset
-	// Ie: me have a C locale and some french file names, and I
-	// can't imagine a version of iconv that couldn't translate
-	// from iso8859?
-	// The 646 thing is for solaris. 
-	if (cp && *cp && strcmp(cp, "US-ASCII") 
-#ifdef sun
-	    && strcmp(cp, "646")
-#endif
-	    ) {
-	    localecharset = string(cp);
-	} else {
-	    // Note: it seems that all versions of iconv will take
-	    // iso-8859. Some won't take iso8859
-	    localecharset = string(cstr_iso_8859_1);
-	}
-	LOGDEB1(("RclConfig::getDefCharset: localecharset [%s]\n",
-		localecharset.c_str()));
-    }
-
-    if (defcharset.empty()) {
-	defcharset = localecharset;
-    }
-
     if (filename) {
-	return localecharset;
+	return o_localecharset;
     } else {
-	return defcharset;
+	return m_defcharset.empty() ? o_localecharset : m_defcharset;
     }
 }
 
-bool RclConfig::addLocalFields(map<string, string> *tgt)
-{
-    LOGDEB0(("RclConfig::addLocalFields: keydir [%s]\n", m_keydir.c_str()));
-    string sfields;
-    if (tgt == 0 || ! getConfParam("localfields", sfields))
-        return false;
-    // Substitute ':' with '\n' inside the string. There is no way to escape ':'
-    for (string::size_type i = 0; i < sfields.size(); i++)
-        if (sfields[i] == ':')
-            sfields[i] = '\n';
-    // Parse the result with a confsimple and add the results to the metadata
-    ConfSimple conf(sfields, 1, true);
-    list<string> nmlst = conf.getNames(cstr_null);
-    for (list<string>::const_iterator it = nmlst.begin();
-         it != nmlst.end(); it++) {
-        conf.get(*it, (*tgt)[*it]);
-        LOGDEB(("RclConfig::addLocalFields: [%s] => [%s]\n",
-		(*it).c_str(), (*tgt)[*it].c_str()));
-    }
-    return true;
-}
-
 // Get all known document mime values. We get them from the mimeconf
 // 'index' submap.
 // It's quite possible that there are other mime types in the index
@@ -414,15 +510,9 @@ bool RclConfig::addLocalFields(map<string, string> *tgt)
 //
 // This unfortunately means that searches by file names and mime type
 // filtering don't work well together.
-list<string> RclConfig::getAllMimeTypes()
+vector<string> RclConfig::getAllMimeTypes() const
 {
-    list<string> lst;
-    if (mimeconf == 0)
-	return lst;
-    lst = mimeconf->getNames("index");
-    lst.sort();
-    lst.unique();
-    return lst;
+    return mimeconf ? mimeconf->getNames("index") : vector<string>();
 }
 
 // Things for suffix comparison. We define a string class and string 
@@ -501,19 +591,19 @@ bool RclConfig::inStopSuffixes(const string& fni)
     }
 }
 
-string RclConfig::getMimeTypeFromSuffix(const string& suff)
+string RclConfig::getMimeTypeFromSuffix(const string& suff) const
 {
     string mtype;
     mimemap->get(suff, mtype, m_keydir);
     return mtype;
 }
 
-string RclConfig::getSuffixFromMimeType(const string &mt)
+string RclConfig::getSuffixFromMimeType(const string &mt) const
 {
     string suffix;
-    list<string>sfs = mimemap->getNames(cstr_null);
+    vector<string>sfs = mimemap->getNames(cstr_null);
     string mt1;
-    for (list<string>::const_iterator it = sfs.begin(); 
+    for (vector<string>::const_iterator it = sfs.begin(); 
 	 it != sfs.end(); it++) {
 	if (mimemap->get(*it, mt1, cstr_null))
 	    if (!stringicmp(mt, mt1))
@@ -523,7 +613,7 @@ string RclConfig::getSuffixFromMimeType(const string &mt)
 }
 
 /** Get list of file categories from mimeconf */
-bool RclConfig::getMimeCategories(list<string>& cats) 
+bool RclConfig::getMimeCategories(vector<string>& cats) const
 {
     if (!mimeconf)
 	return false;
@@ -531,11 +621,11 @@ bool RclConfig::getMimeCategories(list<string>& cats)
     return true;
 }
 
-bool RclConfig::isMimeCategory(string& cat) 
+bool RclConfig::isMimeCategory(string& cat) const
 {
-    list<string>cats;
+    vector<string>cats;
     getMimeCategories(cats);
-    for (list<string>::iterator it = cats.begin(); it != cats.end(); it++) {
+    for (vector<string>::iterator it = cats.begin(); it != cats.end(); it++) {
 	if (!stringicmp(*it,cat))
 	    return true;
     }
@@ -543,7 +633,7 @@ bool RclConfig::isMimeCategory(string& cat)
 }
 
 /** Get list of mime types for category from mimeconf */
-bool RclConfig::getMimeCatTypes(const string& cat, list<string>& tps)
+bool RclConfig::getMimeCatTypes(const string& cat, vector<string>& tps) const
 {
     tps.clear();
     if (!mimeconf)
@@ -576,7 +666,7 @@ string RclConfig::getMimeHandlerDef(const string &mtype, bool filtertypes)
     return hs;
 }
 
-bool RclConfig::getGuiFilterNames(list<string>& cats) 
+bool RclConfig::getGuiFilterNames(vector<string>& cats) const
 {
     if (!mimeconf)
 	return false;
@@ -584,7 +674,7 @@ bool RclConfig::getGuiFilterNames(list<string>& cats)
     return true;
 }
 
-bool RclConfig::getGuiFilter(const string& catfiltername, string& frag)
+bool RclConfig::getGuiFilter(const string& catfiltername, string& frag) const
 {
     frag.clear();
     if (!mimeconf)
@@ -609,21 +699,25 @@ bool RclConfig::valueSplitAttributes(const string& whole, string& value,
     // Handle additional attributes. We substitute the semi-colons
     // with newlines and use a ConfSimple
     if (!attrstr.empty()) {
-        for (string::size_type i = 0; i < attrstr.size(); i++)
+        for (string::size_type i = 0; i < attrstr.size(); i++) {
             if (attrstr[i] == ';')
                 attrstr[i] = '\n';
-        attrs = ConfSimple(attrstr);
+	}
+        attrs.reparse(attrstr);
+    } else {
+	attrs.clear();
     }
+    
     return true;
 }
 
-
-string RclConfig::getMissingHelperDesc()
+bool RclConfig::getMissingHelperDesc(string& out) const
 {
     string fmiss = path_cat(getConfDir(), "missing");
-    string out;
-    file_to_string(fmiss, out);
-    return out;
+    out.clear();
+    if (!file_to_string(fmiss, out))
+	return false;
+    return true;
 }
 
 void RclConfig::storeMissingHelperDesc(const string &s)
@@ -652,8 +746,9 @@ bool RclConfig::readFieldsConfig(const string& cnferrloc)
     // Build a direct map avoiding all indirections for field to
     // prefix translation
     // Add direct prefixes from the [prefixes] section
-    list<string>tps = m_fields->getNames("prefixes");
-    for (list<string>::const_iterator it = tps.begin(); it != tps.end();it++) {
+    vector<string>tps = m_fields->getNames("prefixes");
+    for (vector<string>::const_iterator it = tps.begin(); 
+	 it != tps.end(); it++) {
 	string val;
 	m_fields->get(*it, val, "prefixes");
 	ConfSimple attrs;
@@ -677,7 +772,7 @@ bool RclConfig::readFieldsConfig(const string& cnferrloc)
     // Having the aliases in the prefix map avoids an additional indirection
     // at index time.
     tps = m_fields->getNames("aliases");
-    for (list<string>::const_iterator it = tps.begin(); it != tps.end();it++) {
+    for (vector<string>::const_iterator it = tps.begin(); it != tps.end();it++) {
 	string canonic = stringtolower(*it); // canonic name
 	FieldTraits ft;
 	map<string, FieldTraits>::const_iterator pit = 
@@ -687,9 +782,9 @@ bool RclConfig::readFieldsConfig(const string& cnferrloc)
 	}
 	string aliases;
 	m_fields->get(canonic, aliases, "aliases");
-	list<string> l;
+	vector<string> l;
 	stringToStrings(aliases, l);
-	for (list<string>::const_iterator ait = l.begin();
+	for (vector<string>::const_iterator ait = l.begin();
 	     ait != l.end(); ait++) {
 	    if (pit != m_fldtotraits.end())
 		m_fldtotraits[stringtolower(*ait)] = ft;
@@ -706,9 +801,9 @@ bool RclConfig::readFieldsConfig(const string& cnferrloc)
     }
 #endif
 
-    list<string> sl = m_fields->getNames("stored");
+    vector<string> sl = m_fields->getNames("stored");
     if (!sl.empty()) {
-	for (list<string>::const_iterator it = sl.begin(); 
+	for (vector<string>::const_iterator it = sl.begin(); 
 	     it != sl.end(); it++) {
 	    string fld = fieldCanon(stringtolower(*it));
 	    m_storedFields.insert(fld);
@@ -716,8 +811,8 @@ bool RclConfig::readFieldsConfig(const string& cnferrloc)
     }
 
     // Extended file attribute to field translations
-    list<string>xattrs = m_fields->getNames("xattrtofields");
-    for (list<string>::const_iterator it = xattrs.begin(); 
+    vector<string>xattrs = m_fields->getNames("xattrtofields");
+    for (vector<string>::const_iterator it = xattrs.begin(); 
 	 it != xattrs.end(); it++) {
 	string val;
 	m_fields->get(*it, val, "xattrtofields");
@@ -729,34 +824,35 @@ bool RclConfig::readFieldsConfig(const string& cnferrloc)
 
 // Return specifics for field name:
 bool RclConfig::getFieldTraits(const string& _fld, const FieldTraits **ftpp)
+    const
 {
     string fld = fieldCanon(_fld);
     map<string, FieldTraits>::const_iterator pit = m_fldtotraits.find(fld);
     if (pit != m_fldtotraits.end()) {
 	*ftpp = &pit->second;
 	LOGDEB1(("RclConfig::getFieldTraits: [%s]->[%s]\n", 
-		 _fld.c_str(), ft.pfx.c_str()));
+		 _fld.c_str(), pit->second.pfx.c_str()));
 	return true;
     } else {
-	LOGDEB1(("RclConfig::readFieldsConfig: no prefix for field [%s]\n",
+	LOGDEB1(("RclConfig::getFieldTraits: no prefix for field [%s]\n",
 		 fld.c_str()));
 	*ftpp = 0;
 	return false;
     }
 }
 
-set<string> RclConfig::getIndexedFields()
+set<string> RclConfig::getIndexedFields() const
 {
     set<string> flds;
     if (m_fields == 0)
 	return flds;
 
-    list<string> sl = m_fields->getNames("prefixes");
+    vector<string> sl = m_fields->getNames("prefixes");
     flds.insert(sl.begin(), sl.end());
     return flds;
 }
 
-string RclConfig::fieldCanon(const string& f)
+string RclConfig::fieldCanon(const string& f) const
 {
     string fld = stringtolower(f);
     map<string, string>::const_iterator it = m_aliastocanon.find(fld);
@@ -769,42 +865,91 @@ string RclConfig::fieldCanon(const string& f)
     return fld;
 }
 
-list<string> RclConfig::getFieldSectNames(const string &sk, const char* patrn)
+vector<string> RclConfig::getFieldSectNames(const string &sk, const char* patrn)
+    const
 {
     if (m_fields == 0)
-        return list<string>();
+        return vector<string>();
     return m_fields->getNames(sk, patrn);
 }
 
 bool RclConfig::getFieldConfParam(const string &name, const string &sk, 
-                                  string &value)
+                                  string &value) const
 {
     if (m_fields == 0)
         return false;
     return m_fields->get(name, value, sk);
 }
 
+string RclConfig::getMimeViewerAllEx() const
+{
+    string hs;
+    if (mimeview == 0)
+	return hs;
+    mimeview->get("xallexcepts", hs, "");
+    return hs;
+}
 
-string RclConfig::getMimeViewerDef(const string &mtype, const string& apptag)
+bool RclConfig::setMimeViewerAllEx(const string& allex)
+{
+    if (mimeview == 0)
+        return false;
+    if (!mimeview->set("xallexcepts", allex, "")) {
+	m_reason = string("RclConfig:: cant set value. Readonly?");
+	return false;
+    }
+
+    return true;
+}
+
+string RclConfig::getMimeViewerDef(const string &mtype, const string& apptag,
+				   bool useall) const
 {
     LOGDEB2(("RclConfig::getMimeViewerDef: mtype [%s] apptag [%s]\n",
 	     mtype.c_str(), apptag.c_str()));
     string hs;
     if (mimeview == 0)
 	return hs;
+
+    if (useall) {
+	// Check for exception
+	string excepts = getMimeViewerAllEx();
+	vector<string> vex;
+	stringToTokens(excepts, vex);
+	bool isexcept = false;
+	for (vector<string>::iterator it = vex.begin();
+	     it != vex.end(); it++) {
+	    vector<string> mita;
+	    stringToTokens(*it, mita, "|");
+	    if ((mita.size() == 1 && apptag.empty() && mita[0] == mtype) ||
+		(mita.size() == 2 && mita[1] == apptag && mita[0] == mtype)) {
+		// Exception to x-all
+		isexcept = true;
+		break;
+	    }
+	}
+
+	if (isexcept == false) {
+	    mimeview->get("application/x-all", hs, "view");
+	    return hs;
+	}
+	// Fallthrough to normal case.
+    }
+
     if (apptag.empty() || !mimeview->get(mtype + string("|") + apptag,
                                          hs, "view"))
         mimeview->get(mtype, hs, "view");
     return hs;
 }
 
-bool RclConfig::getMimeViewerDefs(vector<pair<string, string> >& defs)
+bool RclConfig::getMimeViewerDefs(vector<pair<string, string> >& defs) const
 {
     if (mimeview == 0)
 	return false;
-    list<string>tps = mimeview->getNames("view");
-    for (list<string>::const_iterator it = tps.begin(); it != tps.end();it++) {
-	defs.push_back(pair<string, string>(*it, getMimeViewerDef(*it, "")));
+    vector<string>tps = mimeview->getNames("view");
+    for (vector<string>::const_iterator it = tps.begin(); 
+	 it != tps.end();it++) {
+	defs.push_back(pair<string, string>(*it, getMimeViewerDef(*it, "", 0)));
     }
     return true;
 }
@@ -813,30 +958,20 @@ bool RclConfig::setMimeViewerDef(const string& mt, const string& def)
 {
     if (mimeview == 0)
         return false;
-    string pconfname = path_cat(getConfDir(), "mimeview");
-    // Make sure this exists 
-    close(open(pconfname.c_str(), O_CREAT|O_WRONLY, 0600));
-    ConfTree tree(pconfname.c_str());
-    if (!tree.set(mt, def, "view")) {
-	m_reason = string("RclConfig::setMimeViewerDef: cant set value in ")
-	    + pconfname;
-	return false;
-    }
+    bool status;
+    if (!def.empty()) 
+	status = mimeview->set(mt, def, "view");
+    else 
+	status = mimeview->erase(mt, "view");
 
-    list<string> cdirs;
-    cdirs.push_back(m_confdir);
-    cdirs.push_back(path_cat(m_datadir, "examples"));
-
-    delete mimeview;
-    mimeview = new ConfStack<ConfSimple>("mimeview", cdirs, true);
-    if (mimeview == 0 || !mimeview->ok()) {
-	m_reason = string("No/bad mimeview in: ") + m_confdir;
+    if (!status) {
+	m_reason = string("RclConfig:: cant set value. Readonly?");
 	return false;
     }
     return true;
 }
 
-bool RclConfig::mimeViewerNeedsUncomp(const string &mimetype)
+bool RclConfig::mimeViewerNeedsUncomp(const string &mimetype) const
 {
     string s;
     vector<string> v;
@@ -847,36 +982,34 @@ bool RclConfig::mimeViewerNeedsUncomp(const string &mimetype)
     return true;
 }
 
-/**
- * Return icon name and path
- */
-string RclConfig::getMimeIconName(const string &mtype, string *path)
+string RclConfig::getMimeIconPath(const string &mtype, const string &apptag)
+    const
 {
     string iconname;
-    mimeconf->get(mtype, iconname, "icons");
+    if (!apptag.empty())
+	mimeconf->get(mtype + string("|") + apptag, iconname, "icons");
+    if (iconname.empty())
+        mimeconf->get(mtype, iconname, "icons");
     if (iconname.empty())
 	iconname = "document";
 
-    if (path) {
-	string iconsdir;
-
+    string iconpath;
 #if defined (__FreeBSD__) && __FreeBSD_version < 500000
-	// gcc 2.95 dies if we call getConfParam here ??
-	if (m_conf) m_conf->get(string("iconsdir"), iconsdir, m_keydir);
+    // gcc 2.95 dies if we call getConfParam here ??
+    if (m_conf) m_conf->get(string("iconsdir"), iconpath, m_keydir);
 #else
-	getConfParam("iconsdir", iconsdir);
+    getConfParam("iconsdir", iconpath);
 #endif
-	if (iconsdir.empty()) {
-	    iconsdir = path_cat(m_datadir, "images");
-	} else {
-	    iconsdir = path_tildexpand(iconsdir);
-	}
-	*path = path_cat(iconsdir, iconname) + ".png";
+
+    if (iconpath.empty()) {
+	iconpath = path_cat(m_datadir, "images");
+    } else {
+	iconpath = path_tildexpand(iconpath);
     }
-    return iconname;
+    return path_cat(iconpath, iconname) + ".png";
 }
 
-string RclConfig::getDbDir()
+string RclConfig::getDbDir() const
 {
     string dbdir;
     if (!getConfParam("dbdir", dbdir)) {
@@ -893,7 +1026,42 @@ string RclConfig::getDbDir()
     return path_canon(dbdir);
 }
 
-bool RclConfig::sourceChanged()
+void RclConfig::urlrewrite(const string& dbdir, string& url) const
+{
+    LOGDEB2(("RclConfig::urlrewrite: dbdir [%s] url [%s]\n",
+	    dbdir.c_str(), url.c_str()));
+
+    // Do path translations exist for this index ?
+    if (m_ptrans == 0 || !m_ptrans->hasSubKey(dbdir)) {
+	LOGDEB2(("RclConfig::urlrewrite: no paths translations (m_ptrans %p)\n",
+		m_ptrans));
+	return;
+    }
+
+    string path = fileurltolocalpath(url);
+    if (path.empty()) {
+	LOGDEB2(("RclConfig::urlrewrite: not file url\n"));
+	return;
+    }
+
+    // For each translation check if the prefix matches the input path,
+    // replace and return the result if it does.
+    vector<string> opaths = m_ptrans->getNames(dbdir);
+    for (vector<string>::const_iterator it = opaths.begin(); 
+	 it != opaths.end(); it++) {
+	if (it->size() <= path.size() && !path.compare(0, it->size(), *it)) {
+	    string npath;
+	    // This call always succeeds because the key comes from getNames()
+	    if (m_ptrans->get(*it, npath, dbdir)) { 
+		path = path.replace(0, it->size(), npath);
+		url = "file://" + path;
+	    }
+	    break;
+	}
+    }
+}
+
+bool RclConfig::sourceChanged() const
 {
     if (m_conf && m_conf->sourceChanged())
 	return true;
@@ -905,21 +1073,23 @@ bool RclConfig::sourceChanged()
 	return true;
     if (m_fields && m_fields->sourceChanged())
 	return true;
+    if (m_ptrans && m_ptrans->sourceChanged())
+	return true;
     return false;
 }
 
-string RclConfig::getStopfile()
+string RclConfig::getStopfile() const
 {
     return path_cat(getConfDir(), "stoplist.txt");
 }
-string RclConfig::getPidfile()
+string RclConfig::getPidfile() const
 {
     return path_cat(getConfDir(), "index.pid");
 }
 
 // The index status file is fast changing, so it's possible to put it outside
 // of the config directory (for ssds, not sure this is really useful).
-string RclConfig::getIdxStatusFile()
+string RclConfig::getIdxStatusFile() const
 {
     string path;
     if (!getConfParam("idxstatusfile", path)) {
@@ -934,7 +1104,16 @@ string RclConfig::getIdxStatusFile()
     }
 }
 
-list<string>& RclConfig::getSkippedNames()
+string RclConfig::getWebQueueDir() const
+{
+    string webqueuedir;
+    if (!getConfParam("webqueuedir", webqueuedir))
+	webqueuedir = "~/.recollweb/ToIndex/";
+    webqueuedir = path_tildexpand(webqueuedir);
+    return webqueuedir;
+}
+
+vector<string>& RclConfig::getSkippedNames()
 {
     if (m_skpnstate.needrecompute()) {
 	stringToStrings(m_skpnstate.savedvalue, m_skpnlist);
@@ -942,9 +1121,9 @@ list<string>& RclConfig::getSkippedNames()
     return m_skpnlist;
 }
 
-list<string> RclConfig::getSkippedPaths()
+vector<string> RclConfig::getSkippedPaths() const
 {
-    list<string> skpl;
+    vector<string> skpl;
     getConfParam("skippedPaths", &skpl);
 
     // Always add the dbdir and confdir to the skipped paths. This is 
@@ -952,37 +1131,46 @@ list<string> RclConfig::getSkippedPaths()
     // don't do this.
     skpl.push_back(getDbDir());
     skpl.push_back(getConfDir());
-    for (list<string>::iterator it = skpl.begin(); it != skpl.end(); it++) {
+    // And the web queue dir
+    skpl.push_back(getWebQueueDir());
+    for (vector<string>::iterator it = skpl.begin(); it != skpl.end(); it++) {
 	*it = path_tildexpand(*it);
 	*it = path_canon(*it);
     }
-    skpl.sort();
-    skpl.unique();
+    sort(skpl.begin(), skpl.end());
+    vector<string>::iterator uit = unique(skpl.begin(), skpl.end());
+    skpl.resize(uit - skpl.begin());
     return skpl;
 }
 
-list<string> RclConfig::getDaemSkippedPaths()
+vector<string> RclConfig::getDaemSkippedPaths() const
 {
-    list<string> skpl = getSkippedPaths();
-
-    list<string> dskpl;
+    vector<string> dskpl;
     getConfParam("daemSkippedPaths", &dskpl);
 
-    for (list<string>::iterator it = dskpl.begin(); it != dskpl.end(); it++) {
+    for (vector<string>::iterator it = dskpl.begin(); it != dskpl.end(); it++) {
 	*it = path_tildexpand(*it);
 	*it = path_canon(*it);
     }
-    dskpl.sort();
 
-    skpl.merge(dskpl);
-    skpl.unique();
+    vector<string> skpl1 = getSkippedPaths();
+    vector<string> skpl;
+    if (dskpl.empty()) {
+	skpl = skpl1;
+    } else {
+	sort(dskpl.begin(), dskpl.end());
+	merge(dskpl.begin(), dskpl.end(), skpl1.begin(), skpl1.end(), 
+	      skpl.begin());
+	vector<string>::iterator uit = unique(skpl.begin(), skpl.end());
+	skpl.resize(uit - skpl.begin());
+    }
     return skpl;
 }
 
 
 // Look up an executable filter.  We look in $RECOLL_FILTERSDIR,
 // filtersdir in config file, then let the system use the PATH
-string RclConfig::findFilter(const string &icmd)
+string RclConfig::findFilter(const string &icmd) const
 {
     // If the path is absolute, this is it
     if (icmd[0] == '/')
@@ -1024,20 +1212,20 @@ string RclConfig::findFilter(const string &icmd)
 /** 
  * Return decompression command line for given mime type
  */
-bool RclConfig::getUncompressor(const string &mtype, list<string>& cmd)
+bool RclConfig::getUncompressor(const string &mtype, vector<string>& cmd) const
 {
     string hs;
 
     mimeconf->get(mtype, hs, cstr_null);
     if (hs.empty())
 	return false;
-    list<string> tokens;
+    vector<string> tokens;
     stringToStrings(hs, tokens);
     if (tokens.empty()) {
 	LOGERR(("getUncompressor: empty spec for mtype %s\n", mtype.c_str()));
 	return false;
     }
-    list<string>::iterator it = tokens.begin();
+    vector<string>::iterator it = tokens.begin();
     if (tokens.size() < 2)
 	return false;
     if (stringlowercmp("uncompress", *it++)) 
@@ -1059,6 +1247,12 @@ static const char blurb0[] =
 "# values is identical.\n"
     ;
 
+// Use uni2ascii -a K to generate these from the utf-8 strings
+// Swedish and Danish. 
+static const char swedish_ex[] = "unac_except_trans = \303\244\303\244 \303\204\303\244 \303\266\303\266 \303\226\303\266 \303\274\303\274 \303\234\303\274 \303\237ss \305\223oe \305\222oe \303\246ae \303\206ae \357\254\201fi \357\254\202fl \303\245\303\245 \303\205\303\245";
+// German:
+static const char german_ex[] = "unac_except_trans = \303\244\303\244 \303\204\303\244 \303\266\303\266 \303\226\303\266 \303\274\303\274 \303\234\303\274 \303\237ss \305\223oe \305\222oe \303\246ae \303\206ae \357\254\201fi \357\254\202fl";
+
 // Create initial user config by creating commented empty files
 static const char *configfiles[] = {"recoll.conf", "mimemap", "mimeconf", 
 				    "mimeview"};
@@ -1079,12 +1273,22 @@ bool RclConfig::initUserConfig()
 	    strerror(errno);
 	return false;
     }
+    string lang = localelang();
     for (int i = 0; i < ncffiles; i++) {
 	string dst = path_cat(m_confdir, string(configfiles[i])); 
 	if (access(dst.c_str(), 0) < 0) {
 	    FILE *fp = fopen(dst.c_str(), "w");
 	    if (fp) {
 		fprintf(fp, "%s\n", blurb);
+		if (!strcmp(configfiles[i], "recoll.conf")) {
+		    // Add improved unac_except_trans for some languages
+		    if (lang == "se" || lang == "dk" || lang == "no" || 
+			lang == "fi") {
+			fprintf(fp, "%s\n", swedish_ex);
+		    } else if (lang == "de") {
+			fprintf(fp, "%s\n", german_ex);
+		    }
+		}
 		fclose(fp);
 	    } else {
 		m_reason += string("fopen ") + dst + ": " + strerror(errno);
@@ -1102,6 +1306,7 @@ void RclConfig::freeAll()
     delete mimeconf; 
     delete mimeview; 
     delete m_fields;
+    delete m_ptrans;
     delete STOPSUFFIXES;
     // just in case
     zeroMe();
@@ -1117,7 +1322,6 @@ void RclConfig::initFrom(const RclConfig& r)
     m_datadir = r.m_datadir;
     m_keydir = r.m_keydir;
     m_cdirs = r.m_cdirs;
-    // We should use reference-counted objects instead!
     if (r.m_conf)
 	m_conf = new ConfStack<ConfTree>(*(r.m_conf));
     if (r.mimemap)
@@ -1128,6 +1332,8 @@ void RclConfig::initFrom(const RclConfig& r)
 	mimeview = new ConfStack<ConfSimple>(*(r.mimeview));
     if (r.m_fields)
 	m_fields = new ConfStack<ConfSimple>(*(r.m_fields));
+    if (r.m_ptrans)
+	m_ptrans = new ConfSimple(*(r.m_ptrans));
     m_fldtotraits = r.m_fldtotraits;
     m_aliastocanon = r.m_aliastocanon;
     m_storedFields = r.m_storedFields;
@@ -1135,11 +1341,12 @@ void RclConfig::initFrom(const RclConfig& r)
     if (r.m_stopsuffixes)
 	m_stopsuffixes = new SuffixStore(*((SuffixStore*)r.m_stopsuffixes));
     m_maxsufflen = r.m_maxsufflen;
-    defcharset = r.defcharset;
+    m_defcharset = r.m_defcharset;
 
     m_stpsuffstate.init(this, mimemap, r.m_stpsuffstate.paramname);
     m_skpnstate.init(this, m_conf, r.m_skpnstate.paramname);
     m_rmtstate.init(this, m_conf, r.m_rmtstate.paramname);
+    m_thrConf = r.m_thrConf;
 }
 
 #else // -> Test
@@ -1148,7 +1355,7 @@ void RclConfig::initFrom(const RclConfig& r)
 #include <signal.h>
 
 #include <iostream>
-#include <list>
+#include <vector>
 #include <string>
 
 using namespace std;
@@ -1250,22 +1457,26 @@ int main(int argc, char **argv)
         }
 	cout << endl;	
     } else if (op_flags & OPT_c) {
-        // Check that all known mime types have an icon and belong to
-        // some category
-        list<string> catnames;
+	// Checking the configuration consistency
+	
+	// Find and display category names
+        vector<string> catnames;
         config->getMimeCategories(catnames);
         cout << "Categories: ";
-        set<string> allmtsfromcats;
-        for (list<string>::const_iterator it = catnames.begin(); 
+        for (vector<string>::const_iterator it = catnames.begin(); 
              it != catnames.end(); it++) {
             cout << *it << " ";
         }
         cout << endl;
-        for (list<string>::const_iterator it = catnames.begin(); 
+
+	// Compute union of all types from each category. Check that there
+	// are no duplicates while we are at it.
+        set<string> allmtsfromcats;
+        for (vector<string>::const_iterator it = catnames.begin(); 
              it != catnames.end(); it++) {
-            list<string> cts;
+            vector<string> cts;
             config->getMimeCatTypes(*it, cts);
-            for (list<string>::const_iterator it1 = cts.begin(); 
+            for (vector<string>::const_iterator it1 = cts.begin(); 
                  it1 != cts.end(); it1++) {
                 // Already in map -> duplicate
                 if (allmtsfromcats.find(*it1) != allmtsfromcats.end()) {
@@ -1275,19 +1486,45 @@ int main(int argc, char **argv)
             }
         }
 
-        list<string> mtypes = config->getAllMimeTypes();
-        for (list<string>::const_iterator it = mtypes.begin();
+	// Retrieve complete list of mime types 
+        vector<string> mtypes = config->getAllMimeTypes();
+
+	// And check that each mime type is found in exactly one category
+        for (vector<string>::const_iterator it = mtypes.begin();
              it != mtypes.end(); it++) {
             if (allmtsfromcats.find(*it) == allmtsfromcats.end()) {
                 cout << "Not found in catgs: [" << *it << "]" << endl;
             }
         }
+
+	// List mime types not in mimeview
+        for (vector<string>::const_iterator it = mtypes.begin();
+             it != mtypes.end(); it++) {
+	    if (config->getMimeViewerDef(*it, "", false).empty()) {
+		cout << "No viewer: [" << *it << "]" << endl;
+	    }
+        }
+
+	// Check that each mime type has an indexer
+        for (vector<string>::const_iterator it = mtypes.begin();
+             it != mtypes.end(); it++) {
+	    if (config->getMimeHandlerDef(*it, false).empty()) {
+		cout << "No filter: [" << *it << "]" << endl;
+	    }
+        }
+
+	// Check that each mime type has a defined icon
+        for (vector<string>::const_iterator it = mtypes.begin();
+             it != mtypes.end(); it++) {
+	    if (config->getMimeIconPath(*it, "") == "document") {
+		cout << "No or generic icon: [" << *it << "]" << endl;
+	    }
+        }
+
     } else {
         config->setKeyDir(cstr_null);
-	list<string> names = config->getConfNames();
-	names.sort();
-	names.unique();
-	for (list<string>::iterator it = names.begin(); 
+	vector<string> names = config->getConfNames();
+	for (vector<string>::iterator it = names.begin(); 
 	     it != names.end();it++) {
 	    string value;
 	    config->getConfParam(*it, value);
