@@ -19,6 +19,7 @@
 #include <unistd.h>
 #include <sys/types.h>
 #include <string.h>
+#include <errno.h>
 
 #include "cstr.h"
 #include "pathut.h"
@@ -127,8 +128,8 @@ public:
             confstr += line + "\n";
         }
         ConfSimple fields(confstr, 1);
-        list<string> names = fields.getNames(cstr_null);
-        for (list<string>::iterator it = names.begin();
+        vector<string> names = fields.getNames(cstr_null);
+        for (vector<string>::iterator it = names.begin();
              it != names.end(); it++) {
             string value;
             fields.get(*it, value, cstr_null);
@@ -179,9 +180,7 @@ BeagleQueueIndexer::BeagleQueueIndexer(RclConfig *cnf, Rcl::Db *db,
     : m_config(cnf), m_db(db), m_cache(0), m_updater(updfunc), 
       m_nocacheindex(false)
 {
-
-    if (!m_config->getConfParam("beaglequeuedir", m_queuedir))
-        m_queuedir = path_tildexpand("~/.beagle/ToIndex/");
+    m_queuedir = m_config->getWebQueueDir();
     path_catslash(m_queuedir);
     m_cache = new BeagleQueueCache(cnf);
 }
@@ -204,8 +203,10 @@ bool BeagleQueueIndexer::indexFromCache(const string& udi)
     string data;
     string hittype;
 
-    if (!m_cache || !m_cache->getFromCache(udi, dotdoc, data, &hittype))
+    if (!m_cache || !m_cache->getFromCache(udi, dotdoc, data, &hittype)) {
+	LOGERR(("BeagleQueueIndexer::indexFromCache: cache failed\n"));
         return false;
+    }
 
     if (hittype.empty()) {
         LOGERR(("BeagleIndexer::index: cc entry has no hit type\n"));
@@ -224,8 +225,8 @@ bool BeagleQueueIndexer::indexFromCache(const string& udi)
         return true;
     } else {
         Rcl::Doc doc;
-        FileInterner interner(data, m_config, m_tmpdir, 
-                              FileInterner::FIF_doUseInputMimetype,
+        FileInterner interner(data, m_config, 
+			      FileInterner::FIF_doUseInputMimetype,
                               dotdoc.mimetype);
         FileInterner::Status fis;
         try {
@@ -266,6 +267,11 @@ bool BeagleQueueIndexer::index()
         return false;
     LOGDEB(("BeagleQueueIndexer::processqueue: [%s]\n", m_queuedir.c_str()));
     m_config->setKeyDir(m_queuedir);
+    if (!makepath(m_queuedir)) {
+	LOGERR(("BeagleQueueIndexer:: can't create queuedir [%s] errno %d\n", 
+		m_queuedir.c_str(), errno));
+	return false;
+    }
     if (!m_cache || !m_cache->cc()) {
         LOGERR(("BeagleQueueIndexer: cache initialization failed\n"));
         return false;
@@ -307,7 +313,7 @@ bool BeagleQueueIndexer::index()
     // Finally index the queue
     FsTreeWalker walker(FsTreeWalker::FtwNoRecurse);
     walker.addSkippedName(".*");
-    FsTreeWalker::Status status =walker.walk(m_queuedir, *this);
+    FsTreeWalker::Status status = walker.walk(m_queuedir, *this);
     LOGDEB(("BeagleQueueIndexer::processqueue: done: status %d\n", status));
     return true;
 }
@@ -428,7 +434,7 @@ BeagleQueueIndexer::processone(const string &path,
         // to use beagle-generated fields like beagle:inurl
         doc.meta = dotdoc.meta;
 
-        FileInterner interner(path, stp, m_config, m_tmpdir, 
+        FileInterner interner(path, stp, m_config,
                               FileInterner::FIF_doUseInputMimetype,
                               &dotdoc.mimetype);
         FileInterner::Status fis;
