@@ -18,13 +18,11 @@
 #define _fsindexer_h_included_
 
 #include <list>
-#ifndef NO_NAMESPACES
-using std::list;
-#endif
 
 #include "indexer.h"
 #include "fstreewalk.h"
 #ifdef IDX_THREADS
+#include "ptmutex.h"
 #include "workqueue.h"
 #endif // IDX_THREADS
 
@@ -32,18 +30,8 @@ class DbIxStatusUpdater;
 class FIMissingStore;
 struct stat;
 
-#ifdef IDX_THREADS
-class IndexingTask {
-public:
-    IndexingTask(const string& u, const string& p, const Rcl::Doc& d)
-	:udi(u), parent_udi(p), doc(d)
-    {}
-    string udi;
-    string parent_udi;
-    Rcl::Doc doc;
-};
-extern void *FsIndexerIndexWorker(void*);
-#endif // IDX_THREADS
+class DbUpdTask;
+class InternfileTask;
 
 /** Index selected parts of the file system
  
@@ -69,17 +57,17 @@ class FsIndexer : public FsTreeWalkerCB {
     /** 
      * Top level file system tree index method for updating a given database.
      *
-     * We create the temporary directory, open the database,
+     * We open the database,
      * then call a file system walk for each top-level directory.
      */
-    bool index();
+    bool index(bool quickshallow = 0);
 
     /** Index a list of files. No db cleaning or stemdb updating */
-    bool indexFiles(list<string> &files, ConfIndexer::IxFlag f = 
+    bool indexFiles(std::list<std::string> &files, ConfIndexer::IxFlag f = 
 		    ConfIndexer::IxFNone);
 
     /** Purge a list of files. */
-    bool purgeFiles(list<string> &files);
+    bool purgeFiles(std::list<std::string> &files);
 
     /**  Tree walker callback method */
     FsTreeWalker::Status 
@@ -88,32 +76,93 @@ class FsIndexer : public FsTreeWalkerCB {
     /** Make signature for file up to date checks */
     static void makesig(const struct stat *stp, string& out);
 
+    /* Hold the description for an external metadata-gathering command */
+    struct MDReaper {
+	string fieldname;
+	vector<string> cmdv;
+    };
+
  private:
+
+    class PurgeCandidateRecorder {
+    public:
+	PurgeCandidateRecorder() 
+	    : dorecord(false) {}
+	void setRecord(bool onoff) 
+	{
+	    dorecord = onoff;
+	}
+	void record(const string& udi)
+	{
+	    // This test does not need to be protected: the value is set at
+	    // init and never changed.
+	    if (!dorecord)
+		return;
+#ifdef IDX_THREADS
+	    PTMutexLocker locker(mutex);
+#endif
+	    udis.push_back(udi);
+	}
+	const vector<string>& getCandidates() 
+	{
+	    return udis;
+	}
+    private:
+#ifdef IDX_THREADS
+	PTMutexInit mutex;
+#endif
+	bool dorecord;
+	std::vector<std::string> udis;
+    };
+
     FsTreeWalker m_walker;
     RclConfig   *m_config;
     Rcl::Db     *m_db;
-    TempDir     m_tmpdir;
     string       m_reason;
     DbIxStatusUpdater *m_updater;
-    list<string> m_tdl;
+    // Top/start directories list
+    std::vector<std::string> m_tdl;
+    // Store for missing filters and associated mime types
     FIMissingStore *m_missing;
+    // Recorder for files that may need subdoc purging.
+    PurgeCandidateRecorder m_purgeCandidates;
 
     // The configuration can set attribute fields to be inherited by
     // all files in a file system area. Ie: set "rclaptg = thunderbird"
     // inside ~/.thunderbird. The boolean is set at init to avoid
     // further wasteful processing if no local fields are set.
     bool         m_havelocalfields;
-    map<string, string> m_localfields;
+    string       m_slocalfields;
+    map<string, string>  m_localfields;
+
+    // Same idea with the metadata-gathering external commands,
+    // (e.g. used to reap tagging info: "tmsu tags %f")
+    bool m_havemdreapers;
+    string m_smdreapers;
+    vector<MDReaper> m_mdreapers;
 
 #ifdef IDX_THREADS
-    friend void *FsIndexerIndexWorker(void*);
-    WorkQueue<IndexingTask*> m_wqueue;
+    friend void *FsIndexerDbUpdWorker(void*);
+    friend void *FsIndexerInternfileWorker(void*);
+    int m_loglevel;
+    WorkQueue<InternfileTask*> m_iwqueue;
+    WorkQueue<DbUpdTask*> m_dwqueue;
+    bool m_haveInternQ;
+    bool m_haveSplitQ;
+    RclConfig   *m_stableconfig;
 #endif // IDX_THREADS
 
     bool init();
     void localfieldsfromconf();
-    void setlocalfields(Rcl::Doc& doc);
+    void mdreapersfromconf();
+    void setlocalfields(const map<string, string>& flds, Rcl::Doc& doc);
+    void reapmetadata(const vector<MDReaper>& reapers, const string &fn,
+		      Rcl::Doc& doc);
     string getDbDir() {return m_config->getDbDir();}
+    FsTreeWalker::Status 
+    processonefile(RclConfig *config, const string &fn, 
+		   const struct stat *, const map<string,string>& localfields,
+		   const vector<MDReaper>& mdreapers);
 };
 
 #endif /* _fsindexer_h_included_ */
