@@ -32,6 +32,9 @@ using std::vector;
 #include "rclconfig.h"
 #include "rcldb.h"
 #include "rcldoc.h"
+#ifdef IDX_THREADS
+#include "ptmutex.h"
+#endif
 
 class FsIndexer;
 class BeagleQueueIndexer;
@@ -60,12 +63,18 @@ class DbIxStatus {
  * stop as soon as possible without corrupting state */
 class DbIxStatusUpdater {
  public:
+#ifdef IDX_THREADS
+    PTMutexInit m_mutex;
+#endif
     DbIxStatus status;
     virtual ~DbIxStatusUpdater(){}
 
     // Convenience: change phase/fn and update
     virtual bool update(DbIxStatus::Phase phase, const string& fn)
     {
+#ifdef IDX_THREADS
+	PTMutexLocker lock(m_mutex);
+#endif
         status.phase = phase;
         status.fn = fn;
         return update();
@@ -107,7 +116,7 @@ class ConfIndexer {
     bool createAspellDict();
 
     /** List possible stemmer names */
-    static list<string> getStemmerNames();
+    static vector<string> getStemmerNames();
 
     /** Index a list of files. No db cleaning or stemdb updating */
     bool indexFiles(list<string> &files, IxFlag f = IxFNone);
@@ -119,6 +128,8 @@ class ConfIndexer {
     /** Purge a list of files. */
     bool purgeFiles(list<string> &files);
 
+    /** Set in place reset mode */
+    void setInPlaceReset() {m_db.setInPlaceReset();}
  private:
     RclConfig *m_config;
     Rcl::Db    m_db;
@@ -127,6 +138,13 @@ class ConfIndexer {
     BeagleQueueIndexer *m_beagler; 
     DbIxStatusUpdater  *m_updater;
     string              m_reason;
+
+    // The first time we index, we do things a bit differently to
+    // avoid user frustration (make at least some results available
+    // fast by using several passes, the first ones to index common
+    // interesting locations).
+    bool runFirstIndexing();
+    bool firstFsIndexingSequence();
 };
 
 #endif /* _INDEXER_H_INCLUDED_ */
