@@ -35,6 +35,9 @@
 #include <cstdio>
 #include <cstdlib>
 #include <list>
+#include <vector>
+using std::list;
+using std::vector;
 
 #include "debuglog.h"
 #include "rclmon.h"
@@ -501,7 +504,7 @@ bool startMonitor(RclConfig *conf, int opts)
 		// purge.
 		deleted.push_back(ev.m_path);
 		if (ev.evflags() & RclMonEvent::RCLEVT_ISDIR) {
-		    list<string> paths;
+		    vector<string> paths;
 		    if (subtreelist(conf, ev.m_path, paths)) {
 			deleted.insert(deleted.end(),
 				       paths.begin(), paths.end());
@@ -556,6 +559,10 @@ bool startMonitor(RclConfig *conf, int opts)
 	// Check for a config change
 	if (!(opts & RCLMON_NOCONFCHECK) && o_reexec && conf->sourceChanged()) {
 	    LOGDEB(("Rclmonprc: config changed, reexecuting myself\n"));
+	    // We never want to have a -n option after a config
+	    // change. -n was added by the reexec after the initial
+	    // pass even if it was not given on the command line
+	    o_reexec->removeArg("-n");
 	    o_reexec->reexec();
 	}
 	// Lock queue before waiting again
