@@ -43,10 +43,9 @@ using namespace std;
 #include "rclconfig.h"
 #include "mh_html.h"
 #include "fileudi.h"
-#include "beaglequeuecache.h"
 #include "cancelcheck.h"
 #include "copyfile.h"
-#include "ptmutex.h"
+#include "fetcher.h"
 
 #ifdef RCL_USE_XATTR
 #include "pxattr.h"
@@ -81,39 +80,52 @@ static string colon_restore(const string& in)
 #ifdef RCL_USE_XATTR
 void FileInterner::reapXAttrs(const string& path)
 {
+    LOGDEB2(("FileInterner::reapXAttrs: [%s]\n", path.c_str()));
+    
+    // Retrieve xattrs names from files and mapping table from config
     vector<string> xnames;
     if (!pxattr::list(path, &xnames)) {
 	LOGERR(("FileInterner::reapXattrs: pxattr::list: errno %d\n", errno));
 	return;
     }
     const map<string, string>& xtof = m_cfg->getXattrToField();
+
+    // Record the xattrs: names found in the config are either skipped
+    // or mapped depending if the translation is empty. Other names
+    // are recorded as-is
     for (vector<string>::const_iterator it = xnames.begin();
 	 it != xnames.end(); it++) {
-	map<string, string>::const_iterator mit;
-	if ((mit = xtof.find(*it)) != xtof.end()) {
-	    string value;
-	    if (!pxattr::get(path, *it, &value, pxattr::PXATTR_NOFOLLOW)) {
-		LOGERR(("FileInterner::reapXattrs: pxattr::get failed"
-			"for %s, errno %d\n", (*it).c_str(), errno));
+	string key = *it;
+	map<string, string>::const_iterator mit = xtof.find(*it);
+	if (mit != xtof.end()) {
+	    if (mit->second.empty()) {
 		continue;
+	    } else {
+		key = mit->second;
 	    }
-	    // Encode should we ?
-	    m_XAttrsFields[mit->second] = value;
 	}
+	string value;
+	if (!pxattr::get(path, *it, &value, pxattr::PXATTR_NOFOLLOW)) {
+	    LOGERR(("FileInterner::reapXattrs: pxattr::get failed"
+		    "for %s, errno %d\n", (*it).c_str(), errno));
+	    continue;
+	}
+	// Encode should we ?
+	m_XAttrsFields[key] = value;
+	LOGDEB2(("FileInterner::reapXAttrs: [%s] -> [%s]\n", 
+		 key.c_str(), value.c_str()));
     }
 }
 #endif // RCL_USE_XATTR
 
 // This is used when the user wants to retrieve a search result doc's parent
 // (ie message having a given attachment)
-bool FileInterner::getEnclosing(const string &url, const string &ipath,
-				string &eurl, string &eipath, string& udi)
+bool FileInterner::getEnclosingUDI(const Rcl::Doc &doc, string& udi)
 {
-    eurl = url;
-    eipath = ipath;
+    LOGDEB(("FileInterner::getEnclosingUDI(): url [%s] ipath [%s]\n", 
+	    doc.url.c_str(), doc.ipath.c_str()));
+    string eipath = doc.ipath;
     string::size_type colon;
-    LOGDEB(("FileInterner::getEnclosing(): url [%s] ipath [%s]\n", 
-	    url.c_str(), eipath.c_str()));
     if (eipath.empty())
 	return false;
     if ((colon =  eipath.find_last_of(cstr_isep)) != string::npos) {
@@ -121,63 +133,18 @@ bool FileInterner::getEnclosing(const string &url, const string &ipath,
     } else {
 	eipath.erase();
     }
-    make_udi(url_gpath(eurl), eipath, udi);
-
-    LOGDEB(("FileInterner::getEnclosing() after: [%s]\n", eipath.c_str()));
+    
+    make_udi(url_gpath(doc.idxurl.empty() ? doc.url : doc.idxurl), eipath, udi);
     return true;
 }
 
-// Uncompress input file into a temporary one, by executing the appropriate
-// script.
-static bool uncompressfile(RclConfig *conf, const string& ifn, 
-			   const list<string>& cmdv, TempDir& tdir, 
-			   string& tfile)
+string FileInterner::getLastIpathElt(const string& ipath)
 {
-    // Make sure tmp dir is empty. we guarantee this to filters
-    if (!tdir.ok() || !tdir.wipe()) {
-	LOGERR(("uncompressfile: can't clear temp dir %s\n", tdir.dirname()));
-	return false;
-    }
-    string cmd = cmdv.front();
-
-    // Substitute file name and temp dir in command elements
-    list<string>::const_iterator it = cmdv.begin();
-    ++it;
-    list<string> args;
-    map<char, string> subs;
-    subs['f'] = ifn;
-    subs['t'] = tdir.dirname();
-    for (; it != cmdv.end(); it++) {
-	string ns;
-	pcSubst(*it, ns, subs);
-	args.push_back(ns);
-    }
-
-    // Execute command and retrieve output file name, check that it exists
-    ExecCmd ex;
-    int status = ex.doexec(cmd, args, 0, &tfile);
-    if (status || tfile.empty()) {
-	LOGERR(("uncompressfile: doexec: failed for [%s] status 0x%x\n", 
-		ifn.c_str(), status));
-	if (!tdir.wipe()) {
-	    LOGERR(("uncompressfile: wipedir failed\n"));
-	}
-	return false;
-    }
-    if (tfile[tfile.length() - 1] == '\n')
-	tfile.erase(tfile.length() - 1, 1);
-    return true;
-}
-
-// Delete temporary uncompressed file
-void FileInterner::tmpcleanup()
-{
-    if (m_tfile.empty())
-	return;
-    if (unlink(m_tfile.c_str()) < 0) {
-	LOGERR(("FileInterner::tmpcleanup: unlink(%s) errno %d\n", 
-		m_tfile.c_str(), errno));
-	return;
+    string::size_type sep;
+    if ((sep =  ipath.find_last_of(cstr_isep)) != string::npos) {
+	return ipath.substr(sep + 1);
+    } else {
+	return ipath;
     }
 }
 
@@ -188,23 +155,31 @@ void FileInterner::tmpcleanup()
 // Empty handler on return says that we're in error, this will be
 // processed by the first call to internfile().
 // Split into "constructor calls init()" to allow use from other constructor
-FileInterner::FileInterner(const string &f, const struct stat *stp,
-			   RclConfig *cnf, 
-			   TempDir& td, int flags, const string *imime)
-    : m_tdir(td), m_ok(false), m_missingdatap(0)
+FileInterner::FileInterner(const string &fn, const struct stat *stp,
+			   RclConfig *cnf, int flags, const string *imime)
+    : m_ok(false), m_missingdatap(0), m_uncomp((flags & FIF_forPreview) != 0)
 {
+    LOGDEB0(("FileInterner::FileInterner(fn=%s)\n", fn.c_str()));
+    if (fn.empty()) {
+	LOGERR(("FileInterner::FileInterner: empty file name!\n"));
+	return;
+    }
     initcommon(cnf, flags);
-    init(f, stp, cnf, flags, imime);
+    init(fn, stp, cnf, flags, imime);
 }
 
 void FileInterner::init(const string &f, const struct stat *stp, RclConfig *cnf,
                         int flags, const string *imime)
 {
+    if (f.empty()) {
+	LOGERR(("FileInterner::init: empty file name!\n"));
+	return;
+    }
     m_fn = f;
 
-    // This is used by filters which manage some kind of cache.
-    // Indexing by udi makes things easier (because they sometimes get a temp 
-    // as input
+    // Compute udi for the input file. This is used by filters which
+    // manage some kind of cache.  Indexing by udi makes things easier
+    // because they sometimes get a temp as actual input.
     string udi;
     make_udi(f, cstr_null, udi);
 
@@ -217,7 +192,9 @@ void FileInterner::init(const string &f, const struct stat *stp, RclConfig *cnf,
     // In general, even when the input mime type is set (when
     // previewing), we can't use it: it's the type for the actual
     // document, but this can be part of a compound document, and
-    // we're dealing with the top level file here.
+    // we're dealing with the top level file here, or this could be a
+    // compressed file. The flag tells us we really can use it
+    // (e.g. the beagle indexer sets it).
     if (flags & FIF_doUseInputMimetype) {
         if (!imime) {
             LOGERR(("FileInterner:: told to use null imime\n"));
@@ -225,11 +202,10 @@ void FileInterner::init(const string &f, const struct stat *stp, RclConfig *cnf,
         }
         l_mime = *imime;
     } else {
-        LOGDEB(("FileInterner:: [%s] mime [%s] preview %d\n", 
+        LOGDEB(("FileInterner::init fn [%s] mime [%s] preview %d\n", 
                 f.c_str(), imime?imime->c_str() : "(null)", m_forPreview));
 
-        // We need to run mime type identification in any case to check
-        // for a compressed file.
+        // Run mime type identification in any case (see comment above).
         l_mime = mimetype(m_fn, stp, m_cfg, usfci);
 
         // If identification fails, try to use the input parameter. This
@@ -245,17 +221,17 @@ void FileInterner::init(const string &f, const struct stat *stp, RclConfig *cnf,
 	// Has mime: check for a compressed file. If so, create a
 	// temporary uncompressed file, and rerun the mime type
 	// identification, then do the rest with the temp file.
-	list<string>ucmd;
+	vector<string>ucmd;
 	if (m_cfg->getUncompressor(l_mime, ucmd)) {
 	    // Check for compressed size limit
 	    int maxkbs = -1;
 	    if (!m_cfg->getConfParam("compressedfilemaxkbs", &maxkbs) ||
 		maxkbs < 0 || !stp || int(stp->st_size / 1024) < maxkbs) {
-		if (!uncompressfile(m_cfg, m_fn, ucmd, m_tdir, m_tfile)) {
+		if (!m_uncomp.uncompressfile(m_fn, ucmd, m_tfile)) {
 		    return;
 		}
-		LOGDEB1(("FileInterner:: after ucomp: m_tdir %s, tfile %s\n", 
-			 m_tdir.dirname(), m_tfile.c_str()));
+		LOGDEB1(("FileInterner:: after ucomp: tfile %s\n", 
+			 m_tfile.c_str()));
 		m_fn = m_tfile;
 		// Stat the uncompressed file, mainly to get the size
 		struct stat ucstat;
@@ -284,7 +260,7 @@ void FileInterner::init(const string &f, const struct stat *stp, RclConfig *cnf,
 
     // Look for appropriate handler (might still return empty)
     m_mimetype = l_mime;
-    Dijon::Filter *df = getMimeHandler(l_mime, m_cfg, !m_forPreview);
+    RecollFilter *df = getMimeHandler(l_mime, m_cfg, !m_forPreview);
 
     if (!df or df->is_unknown()) {
 	// No real handler for this type, for now :( 
@@ -305,7 +281,8 @@ void FileInterner::init(const string &f, const struct stat *stp, RclConfig *cnf,
 #endif //RCL_USE_XATTR
 
     df->set_docsize(docsize);
-    if (!df->set_document_file(m_fn)) {
+    if (!df->set_document_file(l_mime, m_fn)) {
+	delete df;
 	LOGERR(("FileInterner:: error converting %s\n", m_fn.c_str()));
 	return;
     }
@@ -317,9 +294,10 @@ void FileInterner::init(const string &f, const struct stat *stp, RclConfig *cnf,
 
 // Setup from memory data (ie: out of the web cache). imime needs to be set.
 FileInterner::FileInterner(const string &data, RclConfig *cnf, 
-                           TempDir& td, int flags, const string& imime)
-    : m_tdir(td), m_ok(false), m_missingdatap(0)
+                           int flags, const string& imime)
+    : m_ok(false), m_missingdatap(0), m_uncomp((flags & FIF_forPreview) != 0)
 {
+    LOGDEB0(("FileInterner::FileInterner(data)\n"));
     initcommon(cnf, flags);
     init(data, cnf, flags, imime);
 }
@@ -334,7 +312,7 @@ void FileInterner::init(const string &data, RclConfig *cnf,
     m_mimetype = imime;
 
     // Look for appropriate handler (might still return empty)
-    Dijon::Filter *df = getMimeHandler(m_mimetype, m_cfg, !m_forPreview);
+    RecollFilter *df = getMimeHandler(m_mimetype, m_cfg, !m_forPreview);
 
     if (!df) {
 	// No handler for this type, for now :( if indexallfilenames
@@ -345,22 +323,21 @@ void FileInterner::init(const string &data, RclConfig *cnf,
     df->set_property(Dijon::Filter::OPERATING_MODE, 
 			    m_forPreview ? "view" : "index");
 
-    bool setres = false;
+    bool result = false;
     df->set_docsize(data.length());
     if (df->is_data_input_ok(Dijon::Filter::DOCUMENT_STRING)) {
-	setres = df->set_document_string(data);
+	result = df->set_document_string(m_mimetype, data);
     } else if (df->is_data_input_ok(Dijon::Filter::DOCUMENT_DATA)) {
-	setres = df->set_document_data(data.c_str(), data.length());
+	result = df->set_document_data(m_mimetype, data.c_str(), data.length());
     } else if (df->is_data_input_ok(Dijon::Filter::DOCUMENT_FILE_NAME)) {
-	string filename;
-	if (dataToTempFile(data, m_mimetype, filename)) {
-	    if (!(setres=df->set_document_file(filename))) {
-		m_tmpflgs[0] = false;
-		m_tempfiles.pop_back();
-	    }
+	TempFile temp = dataToTempFile(data, m_mimetype);
+	if (temp.isNotNull() && 
+	    (result = df->set_document_file(m_mimetype, temp->filename()))) {
+	    m_tmpflgs[m_handlers.size()] = true;
+	    m_tempfiles.push_back(temp);
 	}
     }
-    if (!setres) {
+    if (!result) {
 	LOGINFO(("FileInterner:: set_doc failed inside for mtype %s\n", 
                  m_mimetype.c_str()));
 	delete df;
@@ -381,118 +358,52 @@ void FileInterner::initcommon(RclConfig *cnf, int flags)
     m_targetMType = cstr_textplain;
 }
 
-// We use a single beagle cache object to access beagle data. We protect it 
-// against multiple thread access.
-static PTMutexInit o_beagler_mutex;
-
-FileInterner::FileInterner(const Rcl::Doc& idoc, RclConfig *cnf, 
-                           TempDir& td, int flags)
-    : m_tdir(td), m_ok(false), m_missingdatap(0)
+FileInterner::FileInterner(const Rcl::Doc& idoc, RclConfig *cnf, int flags)
+    : m_ok(false), m_missingdatap(0), m_uncomp(((flags & FIF_forPreview) != 0))
 {
-    LOGDEB(("FileInterner::FileInterner(idoc)\n"));
+    LOGDEB0(("FileInterner::FileInterner(idoc)\n"));
     initcommon(cnf, flags);
 
-    // We do insist on having an url...
-    if (idoc.url.empty()) {
-        LOGERR(("FileInterner::FileInterner:: no url!\n"));
+    DocFetcher *fetcher = docFetcherMake(idoc);
+    if (fetcher == 0) {
+        LOGERR(("FileInterner:: no backend\n"));
         return;
     }
-
-    // This stuff will be moved to some kind of generic function:
-    //   get(idoc, ofn, odata, ometa) 
-    // and use some kind of backstore object factory next time we add a 
-    // backend (if ever). 
-    string backend;
-    idoc.getmeta(Rcl::Doc::keybcknd, &backend);
-    
-    if (backend.empty() || !backend.compare("FS")) {
-        // Filesystem document. Intern from file.
-        // The url has to be like file://
-        if (idoc.url.find(cstr_fileu) != 0) {
-            LOGERR(("FileInterner: FS backend and non fs url: [%s]\n",
-                    idoc.url.c_str()));
-            return;
-        }
-        string fn = idoc.url.substr(7, string::npos);
-        struct stat st;
-        if (stat(fn.c_str(), &st) < 0) {
-            LOGERR(("FileInterner:: cannot access document file: [%s]\n",
-                    fn.c_str()));
-            return;
-        }
-        init(fn, &st, cnf, flags, &idoc.mimetype);
-    } else if (!backend.compare("BGL")) {
-        string udi;
-        if (!idoc.getmeta(Rcl::Doc::keyudi, &udi) || udi.empty()) {
-            LOGERR(("FileInterner:: no udi in idoc\n"));
-            return;
-        }
-
-        string data;
-        Rcl::Doc dotdoc;
-	{
-	    PTMutexLocker locker(o_beagler_mutex);
-	    // Retrieve from our webcache (beagle data). The beagler
-	    // object is created at the first call of this routine and
-	    // deleted when the program exits.
-	    static BeagleQueueCache o_beagler(cnf);
-	    if (!o_beagler.getFromCache(udi, dotdoc, data)) {
-		LOGINFO(("FileInterner:: failed fetch from Beagle cache for [%s]\n",
-			 udi.c_str()));
-		return;
-	    }
-	}
-        if (dotdoc.mimetype.compare(idoc.mimetype)) {
-            LOGINFO(("FileInterner:: udi [%s], mimetp mismatch: in: [%s], bgl "
-                     "[%s]\n", idoc.mimetype.c_str(), dotdoc.mimetype.c_str()));
-        }
-        init(data, cnf, flags, dotdoc.mimetype);
-    } else {
-        LOGERR(("FileInterner:: unknown backend: [%s]\n", backend.c_str()));
-        return;
+    DocFetcher::RawDoc rawdoc;
+    if (!fetcher->fetch(cnf, idoc, rawdoc)) {
+	LOGERR(("FileInterner:: fetcher failed\n"));
+	return;
     }
+    switch (rawdoc.kind) {
+    case DocFetcher::RawDoc::RDK_FILENAME:
+        init(rawdoc.data, &rawdoc.st, cnf, flags, &idoc.mimetype);
+	break;
+    case DocFetcher::RawDoc::RDK_DATA:
+        init(rawdoc.data, cnf, flags, idoc.mimetype);
+	break;
+    default:
+	LOGERR(("FileInterner::FileInterner(idoc): bad rawdoc kind ??\n"));
+    }
+    return;
 }
 
-#include "fsindexer.h"
-bool FileInterner::makesig(const Rcl::Doc& idoc, string& sig)
+bool FileInterner::makesig(RclConfig *cnf, const Rcl::Doc& idoc, string& sig)
 {
-    if (idoc.url.empty()) {
-        LOGERR(("FileInterner::makesig:: no url!\n"));
+    DocFetcher *fetcher = docFetcherMake(idoc);
+    if (fetcher == 0) {
+        LOGERR(("FileInterner::makesig no backend for doc\n"));
         return false;
     }
-    string backend;
-    idoc.getmeta(Rcl::Doc::keybcknd, &backend);
-    
-    if (backend.empty() || !backend.compare("FS")) {
-        if (idoc.url.find(cstr_fileu) != 0) {
-            LOGERR(("FileInterner: FS backend and non fs url: [%s]\n",
-                    idoc.url.c_str()));
-            return false;
-        }
-        string fn = idoc.url.substr(7, string::npos);
-        struct stat st;
-        if (stat(fn.c_str(), &st) < 0) {
-            LOGERR(("FileInterner:: cannot access document file: [%s]\n",
-                    fn.c_str()));
-            return false;
-        }
-	FsIndexer::makesig(&st, sig);
-	return true;
-    } else if (!backend.compare("BGL")) {
-	// Bgl sigs are empty
-	sig.clear();
-	return true;
-    } else {
-        LOGERR(("FileInterner:: unknown backend: [%s]\n", backend.c_str()));
-        return false;
-    }
-    return false;
+
+    bool ret = fetcher->makesig(cnf, idoc, sig);
+
+    delete fetcher;
+    return ret;
 }
 
 FileInterner::~FileInterner()
 {
-    tmpcleanup();
-    for (vector<Dijon::Filter*>::iterator it = m_handlers.begin();
+    for (vector<RecollFilter*>::iterator it = m_handlers.begin();
 	 it != m_handlers.end(); it++) {
         returnMimeHandler(*it);
     }
@@ -502,103 +413,86 @@ FileInterner::~FileInterner()
 // Create a temporary file for a block of data (ie: attachment) found
 // while walking the internal document tree, with a type for which the
 // handler needs an actual file (ie : external script).
-bool FileInterner::dataToTempFile(const string& dt, const string& mt, 
-				  string& fn)
+TempFile FileInterner::dataToTempFile(const string& dt, const string& mt)
 {
-    // Find appropriate suffix for mime type
+    // Create temp file with appropriate suffix for mime type
     TempFile temp(new TempFileInternal(m_cfg->getSuffixFromMimeType(mt)));
-    if (temp->ok()) {
-        // We are called before the handler is actually on the stack, so the
-        // index is m_handlers.size(). m_tmpflgs is a static array, so this is
-        // no problem
-	m_tmpflgs[m_handlers.size()] = true;
-	m_tempfiles.push_back(temp);
-    } else {
+    if (!temp->ok()) {
 	LOGERR(("FileInterner::dataToTempFile: cant create tempfile: %s\n",
 		temp->getreason().c_str()));
-	return false;
+	return TempFile();
     }
 
     int fd = open(temp->filename(), O_WRONLY);
     if (fd < 0) {
 	LOGERR(("FileInterner::dataToTempFile: open(%s) failed errno %d\n",
 		temp->filename(), errno));
-	return false;
+	return TempFile();
     }
     if (write(fd, dt.c_str(), dt.length()) != (int)dt.length()) {
 	close(fd);
 	LOGERR(("FileInterner::dataToTempFile: write to %s failed errno %d\n",
 		temp->filename(), errno));
-	return false;
+	return TempFile();
     }
     close(fd);
-    fn = temp->filename();
-    return true;
+    return temp;
 }
 
 // See if the error string is formatted as a missing helper message,
-// accumulate helper name if it is
+// accumulate helper name if it is. The format of the message is:
+// RECFILTERROR HELPERNOTFOUND program1 [program2 ...]
 void FileInterner::checkExternalMissing(const string& msg, const string& mt)
 {
+    LOGDEB2(("checkExternalMissing: [%s]\n", msg.c_str()));
     if (m_missingdatap && msg.find("RECFILTERROR") == 0) {
-	list<string> lerr;
-	stringToStrings(msg, lerr);
-	if (lerr.size() > 2) {
-	    list<string>::iterator it = lerr.begin();
-	    lerr.erase(it++);
+	vector<string> verr;
+	stringToStrings(msg, verr);
+	if (verr.size() > 2) {
+	    vector<string>::iterator it = verr.begin();
+	    it++;
 	    if (*it == "HELPERNOTFOUND") {
-		lerr.erase(it++);
-		string s;
-		stringsToString(lerr, s);
-		m_missingdatap->m_missingExternal.insert(s);
-		m_missingdatap->m_typesForMissing[s].insert(mt);
+		it++;
+		for (; it != verr.end(); it++) {
+		    m_missingdatap->addMissing(*it, mt);
+		}
 	    }
 	}		    
     }
 }
 
-void FileInterner::getMissingExternal(FIMissingStore *st, string& out) 
+void FIMissingStore::getMissingExternal(string& out) 
 {
-    if (st)
-	stringsToString(st->m_missingExternal, out);
+    for (map<string, set<string> >::const_iterator it = 
+	     m_typesForMissing.begin(); it != m_typesForMissing.end(); it++) {
+	out += string(" ") + it->first;
+    }
+    trimstring(out);
 }
 
-void FileInterner::getMissingDescription(FIMissingStore *st, string& out)
+void FIMissingStore::getMissingDescription(string& out)
 {
-    if (st == 0)
-	return;
-
     out.erase();
 
-    for (set<string>::const_iterator it = 
-	     st->m_missingExternal.begin();
-	 it != st->m_missingExternal.end(); it++) {
-	out += *it;
-	map<string, set<string> >::const_iterator it2;
-	it2 = st->m_typesForMissing.find(*it);
-	if (it2 != st->m_typesForMissing.end()) {
-	    out += " (";
-	    set<string>::const_iterator it3;
-	    for (it3 = it2->second.begin(); 
-		 it3 != it2->second.end(); it3++) {
-		out += *it3;
-		out += string(" ");
-	    }
-	    trimstring(out);
-	    out += ")";
-	}	
+    for (map<string, set<string> >::const_iterator it = 
+	     m_typesForMissing.begin(); it != m_typesForMissing.end(); it++) {
+	out += it->first + " (";
+	set<string>::const_iterator it3;
+	for (it3 = it->second.begin(); 
+		 it3 != it->second.end(); it3++) {
+	    out += *it3 + " ";
+	}
+	trimstring(out);
+	out += ")";
 	out += "\n";
     }
 }
 
-void FileInterner::getMissingFromDescription(FIMissingStore *st, const string& in)
+FIMissingStore::FIMissingStore(const string& in)
 {
-    if (st == 0)
-	return;
-
     // The "missing" file is text. Each line defines a missing filter
-    // and the list of mime types actually encountered that needed it (see method
-    // getMissingDescription())
+    // and the list of mime types actually encountered that needed it
+    // (see method getMissingDescription())
 
     vector<string> lines;
     stringToTokens(in, lines, "\n");
@@ -627,10 +521,9 @@ void FileInterner::getMissingFromDescription(FIMissingStore *st, const string& i
 	if (filter.empty())
 	    continue;
 
-	st->m_missingExternal.insert(filter);
 	for (vector<string>::const_iterator itt = mtypes.begin(); 
 	     itt != mtypes.end(); itt++) {
-	    st->m_typesForMissing[filter].insert(*itt);
+	    m_typesForMissing[filter].insert(*itt);
 	}
     }
 }
@@ -652,7 +545,7 @@ static inline bool getKeyValue(const map<string, string>& docdata,
 
 bool FileInterner::dijontorcl(Rcl::Doc& doc)
 {
-    Dijon::Filter *df = m_handlers.back();
+    RecollFilter *df = m_handlers.back();
     if (df == 0) {
 	//??
 	LOGERR(("FileInterner::dijontorcl: null top handler ??\n"));
@@ -677,13 +570,20 @@ bool FileInterner::dijontorcl(Rcl::Doc& doc)
 	    }
 	} else if (it->first == cstr_dj_keymd) {
 	    doc.dmtime = it->second;
+	} else if (it->first == cstr_dj_keyanc) {
+	    doc.haschildren = true;
 	} else if (it->first == cstr_dj_keyorigcharset) {
 	    doc.origcharset = it->second;
+	} else if (it->first == cstr_dj_keyfn) {
+	    // Only if not set during the stack walk
+	    const string *fnp = 0;
+	    if (!doc.peekmeta(Rcl::Doc::keyfn, &fnp) || fnp->empty())
+		doc.meta[Rcl::Doc::keyfn] = it->second;
 	} else if (it->first == cstr_dj_keymt || 
 		   it->first == cstr_dj_keycharset) {
 	    // don't need/want these.
 	} else {
-	    doc.meta[it->first] = it->second;
+	    doc.addmeta(m_cfg->fieldCanon(it->first), it->second);
 	}
     }
     if (doc.meta[Rcl::Doc::keyabs].empty() && 
@@ -716,10 +616,12 @@ void FileInterner::collectIpathAndMT(Rcl::Doc& doc) const
 
 #ifdef RCL_USE_XATTR
     // Set fields from extended file attributes.
-    // These can be overriden by values from inside the file
+    // These can be later augmented by values from inside the file
     for (map<string,string>::const_iterator it = m_XAttrsFields.begin(); 
 	 it != m_XAttrsFields.end(); it++) {
-	doc.meta[it->first] = it->second;
+	LOGDEB1(("Internfile:: setting [%s] from xattrs value [%s]\n",
+		 m_cfg->fieldCanon(it->first).c_str(), it->second.c_str()));
+	doc.meta[m_cfg->fieldCanon(it->first)] = it->second;
     }
 #endif //RCL_USE_XATTR
 
@@ -727,7 +629,7 @@ void FileInterner::collectIpathAndMT(Rcl::Doc& doc) const
     doc.mimetype = m_mimetype;
 
     string ipathel;
-    for (vector<Dijon::Filter*>::const_iterator hit = m_handlers.begin();
+    for (vector<RecollFilter*>::const_iterator hit = m_handlers.begin();
 	 hit != m_handlers.end(); hit++) {
 	const map<string, string>& docdata = (*hit)->get_meta_data();
 	if (getKeyValue(docdata, cstr_dj_keyipath, ipathel)) {
@@ -735,7 +637,7 @@ void FileInterner::collectIpathAndMT(Rcl::Doc& doc) const
 		// We have a non-empty ipath
 		hasipath = true;
 		getKeyValue(docdata, cstr_dj_keymt, doc.mimetype);
-		getKeyValue(docdata, cstr_dj_keyfn, doc.utf8fn);
+		getKeyValue(docdata, cstr_dj_keyfn, doc.meta[Rcl::Doc::keyfn]);
 	    } else {
 		if (doc.fbytes.empty())
 		    getKeyValue(docdata, cstr_dj_keydocsize, doc.fbytes);
@@ -768,7 +670,7 @@ void FileInterner::popHandler()
 {
     if (m_handlers.empty())
 	return;
-    int i = m_handlers.size()-1;
+    int i = m_handlers.size() - 1;
     if (m_tmpflgs[i]) {
 	m_tempfiles.pop_back();
 	m_tmpflgs[i] = false;
@@ -788,7 +690,8 @@ int FileInterner::addHandler()
     getKeyValue(docdata, cstr_dj_keycharset, charset);
     getKeyValue(docdata, cstr_dj_keymt, mimetype);
 
-    LOGDEB(("FileInterner::addHandler: next_doc is %s\n", mimetype.c_str()));
+    LOGDEB(("FileInterner::addHandler: next_doc is %s target [%s]\n", 
+	    mimetype.c_str(), m_targetMType.c_str()));
 
     // If we find a document of the target type (text/plain in
     // general), we're done decoding. If we hit text/plain, we're done
@@ -808,7 +711,7 @@ int FileInterner::addHandler()
 	return ADD_CONTINUE;
     }
 
-    Dijon::Filter *newflt = getMimeHandler(mimetype, m_cfg);
+    RecollFilter *newflt = getMimeHandler(mimetype, m_cfg);
     if (!newflt) {
 	// If we can't find a handler, this doc can't be handled
 	// but there can be other ones so we go on
@@ -834,22 +737,20 @@ int FileInterner::addHandler()
     bool setres = false;
     newflt->set_docsize(txt->length());
     if (newflt->is_data_input_ok(Dijon::Filter::DOCUMENT_STRING)) {
-	setres = newflt->set_document_string(*txt);
+	setres = newflt->set_document_string(mimetype, *txt);
     } else if (newflt->is_data_input_ok(Dijon::Filter::DOCUMENT_DATA)) {
-	setres = newflt->set_document_data(txt->c_str(), txt->length());
+	setres = newflt->set_document_data(mimetype,txt->c_str(),txt->length());
     } else if (newflt->is_data_input_ok(Dijon::Filter::DOCUMENT_FILE_NAME)) {
-	string filename;
-	if (dataToTempFile(*txt, mimetype, filename)) {
-	    if (!(setres = newflt->set_document_file(filename))) {
-		m_tmpflgs[m_handlers.size()] = false;
-		m_tempfiles.pop_back();
-	    } else {
-		// Hack here, but really helps perfs: if we happen to
-		// create a temp file for, ie, an image attachment,
-		// keep it around for preview reuse
-		if (!mimetype.compare(0, 6, "image/")) {
-		    m_imgtmp = m_tempfiles.back();
-		}
+	TempFile temp = dataToTempFile(*txt, mimetype);
+	if (temp.isNotNull() && 
+	    (setres = newflt->set_document_file(mimetype, temp->filename()))) {
+	    m_tmpflgs[m_handlers.size()] = true;
+	    m_tempfiles.push_back(temp);
+	    // Hack here, but really helps perfs: if we happen to
+	    // create a temp file for, ie, an image attachment, keep
+	    // it around for preview to use it through get_imgtmp()
+	    if (!mimetype.compare(0, 6, "image/")) {
+		m_imgtmp = m_tempfiles.back();
 	    }
 	}
     }
@@ -999,15 +900,16 @@ FileInterner::Status FileInterner::internfile(Rcl::Doc& doc, const string& ipath
 	return FIError;
     }
 
-    // If indexing compute ipath and significant mimetype.  ipath is
-    // returned through doc.ipath. We also retrieve some metadata
-    // fields from the ancesters (like date or author). This is useful
-    // for email attachments. The values will be replaced by those
-    // internal to the document (by dijontorcl()) if any, so the order
-    // of calls is important.
-    if (!m_forPreview) {
-	collectIpathAndMT(doc);
-    } else {
+    // Compute ipath and significant mimetype.  ipath is returned
+    // through doc.ipath. We also retrieve some metadata fields from
+    // the ancesters (like date or author). This is useful for email
+    // attachments. The values will be replaced by those internal to
+    // the document (by dijontorcl()) if any, so the order of calls is
+    // important. We used to only do this when indexing, but the aux
+    // fields like filename and author may be interesting when
+    // previewing too
+    collectIpathAndMT(doc);
+    if (m_forPreview) {
 	doc.mimetype = m_reachedMType;
     }
     // Keep this AFTER collectIpathAndMT
@@ -1046,23 +948,43 @@ static string urltolocalpath(string url)
 // - The internfile temporary directory gets destroyed by its destructor
 // - The output temporary file which is held in a reference-counted
 //   object and will be deleted when done with.
+// This DOES NOT work with a non-internal file (because at least one conversion 
+// is always performed).
 bool FileInterner::idocToFile(TempFile& otemp, const string& tofile,
 			      RclConfig *cnf, const Rcl::Doc& idoc)
 {
     LOGDEB(("FileInterner::idocToFile\n"));
-    idoc.dump();
+    // idoc.dump();
 
-    TempDir tmpdir;
+    if (idoc.ipath.empty()) {
+	LOGDEB(("FileInterner::idocToFile: not a sub-document !\n"));
+	// We could do a copy here but it's much more complicated than
+	// it seems because the source is not necessarily a simple
+	// depending on the backend. Until we fix the Internfile
+	// constructor to not do the first conversion, it's much saner
+	// to just return an error
+	return false;
+    }
 
     // We set FIF_forPreview for consistency with the previous version
     // which determined this by looking at mtype!=null. Probably
     // doesn't change anything in this case.
-    FileInterner interner(idoc, cnf, tmpdir, FIF_forPreview);
+    FileInterner interner(idoc, cnf, FIF_forPreview);
     interner.setTargetMType(idoc.mimetype);
+    return interner.interntofile(otemp, tofile, idoc.ipath, idoc.mimetype);
+}
+
+bool FileInterner::interntofile(TempFile& otemp, const string& tofile,
+				const string& ipath, const string& mimetype)
+{
+    if (!ok()) {
+	LOGERR(("FileInterner::interntofile: constructor failed\n"));
+	return false;
+    }
     Rcl::Doc doc;
-    Status ret = interner.internfile(doc, idoc.ipath);
+    Status ret = internfile(doc, ipath);
     if (ret == FileInterner::FIError) {
-	LOGERR(("FileInterner::idocToFile: internfile() failed\n"));
+	LOGERR(("FileInterner::interntofile: internfile() failed\n"));
 	return false;
     }
 
@@ -1072,39 +994,41 @@ bool FileInterner::idocToFile(TempFile& otemp, const string& tofile,
     // performed. A common case would be an "Open" on an html file
     // (we'd end up with text/plain content). As the html version is
     // saved in this case, use it.  
-    if (!stringlowercmp("text/html", idoc.mimetype) && 
-        !interner.get_html().empty()) {
-        doc.text = interner.get_html();
+    if (!stringlowercmp("text/html", mimetype) && !get_html().empty()) {
+        doc.text = get_html();
         doc.mimetype = "text/html";
     }
 
     string filename;
     TempFile temp;
     if (tofile.empty()) {
-	TempFile temp1(new TempFileInternal(cnf->getSuffixFromMimeType(idoc.mimetype)));
+	TempFile temp1(new TempFileInternal(
+			   m_cfg->getSuffixFromMimeType(mimetype)));
 	temp = temp1;
 	if (!temp->ok()) {
-	    LOGERR(("FileInterner::idocToFile: cant create temporary file"));
+	    LOGERR(("FileInterner::interntofile: can't create temp file\n"));
 	    return false;
 	}
 	filename = temp->filename();
     } else {
 	filename = tofile;
     }
+
     int fd = open(filename.c_str(), O_WRONLY|O_CREAT, 0600);
     if (fd < 0) {
-	LOGERR(("FileInterner::idocToFile: open(%s) failed errno %d\n",
+	LOGERR(("FileInterner::interntofile: open(%s) failed errno %d\n",
 		filename.c_str(), errno));
 	return false;
     }
     const string& dt = doc.text;
     if (write(fd, dt.c_str(), dt.length()) != (int)dt.length()) {
 	close(fd);
-	LOGERR(("FileInterner::idocToFile: write to %s failed errno %d\n",
+	LOGERR(("FileInterner::interntofile: write to %s failed errno %d\n",
 		filename.c_str(), errno));
 	return false;
     }
     close(fd);
+
     if (tofile.empty())
 	otemp = temp;
     return true;
@@ -1125,13 +1049,14 @@ bool FileInterner::isCompressed(const string& fn, RclConfig *cnf)
         return false;
     }
 
-    list<string>ucmd;
+    vector<string> ucmd;
     if (cnf->getUncompressor(l_mime, ucmd)) {
         return true;
     }
     return false;
 }
 
+// Static.
 bool FileInterner::maybeUncompressToTemp(TempFile& temp, const string& fn, 
                                          RclConfig *cnf, const Rcl::Doc& doc)
 {
@@ -1149,7 +1074,7 @@ bool FileInterner::maybeUncompressToTemp(TempFile& temp, const string& fn,
         return false;
     }
 
-    list<string>ucmd;
+    vector<string>ucmd;
     if (!cnf->getUncompressor(l_mime, ucmd)) {
         return true;
     }
@@ -1161,16 +1086,16 @@ bool FileInterner::maybeUncompressToTemp(TempFile& temp, const string& fn,
                  fn.c_str(), maxkbs));
         return false;
     }
-    TempDir tmpdir;
     temp = 
       TempFile(new TempFileInternal(cnf->getSuffixFromMimeType(doc.mimetype)));
-    if (!tmpdir.ok() || !temp->ok()) {
-        LOGERR(("FileInterner: cant create temporary file/dir"));
+    if (!temp->ok()) {
+        LOGERR(("FileInterner: cant create temporary file"));
         return false;
     }
 
+    Uncomp uncomp;
     string uncomped;
-    if (!uncompressfile(cnf, fn, ucmd, tmpdir, uncomped)) {
+    if (!uncomp.uncompressfile(fn, ucmd, uncomped)) {
         return false;
     }
 
@@ -1263,7 +1188,7 @@ int main(int argc, char **argv)
 	perror("stat");
 	exit(1);
     }
-    FileInterner interner(fn, &st, config, "/tmp", 0);
+    FileInterner interner(fn, &st, config, 0);
     Rcl::Doc doc;
     FileInterner::Status status = interner.internfile(doc, ipath);
     switch (status) {
