@@ -14,6 +14,14 @@
  *   Free Software Foundation, Inc.,
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
+#include "autoconfig.h"
+
+#include <sys/types.h>
+#include <sys/wait.h>
+
+#include <list>
+using namespace std;
+
 #include "cstr.h"
 #include "execmd.h"
 #include "mh_exec.h"
@@ -24,13 +32,6 @@
 #include "md5.h"
 #include "rclconfig.h"
 
-#include <sys/types.h>
-#include <sys/wait.h>
-
-#ifndef NO_NAMESPACES
-using namespace std;
-#endif /* NO_NAMESPACES */
-
 // This is called periodically by ExeCmd when it is waiting for data,
 // or when it does receive some. We may choose to interrupt the
 // command.
@@ -73,6 +74,10 @@ bool MimeHandlerExec::next_document()
 	LOGDEB(("MimeHandlerExec::next_document(): helper known missing\n"));
 	return false;
     }
+
+    int filtermaxseconds = 900;
+    m_config->getConfParam("filtermaxseconds", &filtermaxseconds);
+
     if (params.empty()) {
 	// Hu ho
 	LOGERR(("MimeHandlerExec::mkDoc: empty params\n"));
@@ -80,15 +85,11 @@ bool MimeHandlerExec::next_document()
 	return false;
     }
 
-    int filtermaxseconds = 900;
-    m_config->getConfParam("filtermaxseconds", &filtermaxseconds);
-
     // Command name
     string cmd = params.front();
     
-    // Build parameter list: delete cmd name and add the file name
-    list<string>::iterator it = params.begin();
-    list<string>myparams(++it, params.end());
+    // Build parameter vector: delete cmd name and add the file name
+    vector<string>myparams(params.begin() + 1, params.end());
     myparams.push_back(m_fn);
     if (!m_ipath.empty())
 	myparams.push_back(m_ipath);
@@ -99,6 +100,7 @@ bool MimeHandlerExec::next_document()
     ExecCmd mexec;
     MEAdv adv(filtermaxseconds);
     mexec.setAdvise(&adv);
+    mexec.putenv("RECOLL_CONFDIR", m_config->getConfDir());
     mexec.putenv(m_forPreview ? "RECOLL_FILTER_FORPREVIEW=yes" :
 		"RECOLL_FILTER_FORPREVIEW=no");
 
@@ -143,31 +145,47 @@ bool MimeHandlerExec::next_document()
     return true;
 }
 
-void MimeHandlerExec::finaldetails()
+void MimeHandlerExec::handle_cs(const string& mt, const string& icharset)
 {
-    m_metaData[cstr_dj_keyorigcharset] = m_dfltInputCharset;
+    string charset(icharset);
 
-    // cfgFilterOutputCharset comes from the mimeconf filter definition line
-    string& charset = m_metaData[cstr_dj_keycharset];
-    charset = cfgFilterOutputCharset.empty() ? "UTF-8" : cfgFilterOutputCharset;
-    if (!stringlowercmp("default", charset)) {
-	charset = m_dfltInputCharset;
+    // cfgFilterOutputCharset comes from the mimeconf filter
+    // definition line and defaults to UTF-8 if empty. If the value is
+    // "default", we use the default input charset value defined in
+    // recoll.conf (which may vary depending on directory)
+    if (charset.empty()) {
+	charset = cfgFilterOutputCharset.empty() ? cstr_utf8 : 
+	    cfgFilterOutputCharset;
+	if (!stringlowercmp("default", charset)) {
+	    charset = m_dfltInputCharset;
+	}
     }
-
-    string& mt = m_metaData[cstr_dj_keymt];
-    mt = cfgFilterOutputMtype.empty() ? "text/html" : 
-	cfgFilterOutputMtype;
+    m_metaData[cstr_dj_keyorigcharset] = charset;
 
     // If this is text/plain transcode_to/check utf-8
     if (!mt.compare(cstr_textplain)) {
-	(void)txtdcode("mh_exec");
-    }
-
-    string md5, xmd5, reason;
-    if (MD5File(m_fn, md5, &reason)) {
-	m_metaData[cstr_dj_keymd5] = MD5HexPrint(md5, xmd5);
+	(void)txtdcode("mh_exec/m");
     } else {
-	LOGERR(("MimeHandlerExec: cant compute md5 for [%s]: %s\n", 
-		m_fn.c_str(), reason.c_str()));
+	m_metaData[cstr_dj_keycharset] = charset;
     }
 }
+
+void MimeHandlerExec::finaldetails()
+{
+    // The default output mime type is html, but it may be defined
+    // otherwise in the filter definition.
+    m_metaData[cstr_dj_keymt] = cfgFilterOutputMtype.empty() ? "text/html" : 
+	cfgFilterOutputMtype;
+
+    if (!m_forPreview) {
+	string md5, xmd5, reason;
+	if (MD5File(m_fn, md5, &reason)) {
+	    m_metaData[cstr_dj_keymd5] = MD5HexPrint(md5, xmd5);
+	} else {
+	    LOGERR(("MimeHandlerExec: cant compute md5 for [%s]: %s\n", 
+		    m_fn.c_str(), reason.c_str()));
+	}
+    }
+
+    handle_cs(m_metaData[cstr_dj_keymt]);
+}
