@@ -46,16 +46,16 @@ using namespace std;
 static const int maxdepth = 20;
 static const string cstr_mail_charset("charset");
 
-MimeHandlerMail::MimeHandlerMail(RclConfig *cnf, const string &mt) 
-    : RecollFilter(cnf, mt), m_bincdoc(0), m_fd(-1), m_stream(0), m_idx(-1)
+MimeHandlerMail::MimeHandlerMail(RclConfig *cnf, const string &id) 
+    : RecollFilter(cnf, id), m_bincdoc(0), m_fd(-1), m_stream(0), m_idx(-1)
 {
 
     // Look for additional headers to be processed as per config:
-    list<string> hdrnames = 
+    vector<string> hdrnames = 
         m_config->getFieldSectNames("mail");
     if (hdrnames.empty())
         return;
-    for (list<string>::const_iterator it = hdrnames.begin();
+    for (vector<string>::const_iterator it = hdrnames.begin();
          it != hdrnames.end(); it++) {
         (void)m_config->getFieldConfParam(*it, "mail", m_addProcdHdrs[*it]);
     }
@@ -85,25 +85,26 @@ void MimeHandlerMail::clear()
     RecollFilter::clear();
 }
 
-bool MimeHandlerMail::set_document_file(const string &fn)
+bool MimeHandlerMail::set_document_file(const string& mt, const string &fn)
 {
     LOGDEB(("MimeHandlerMail::set_document_file(%s)\n", fn.c_str()));
-    RecollFilter::set_document_file(fn);
+    RecollFilter::set_document_file(mt, fn);
     if (m_fd >= 0) {
 	close(m_fd);
 	m_fd = -1;
     }
 
-    // Yes, we read the file twice. It would be possible in theory to add
-    // the md5 computation to the mime analysis, but ...
-    string md5, xmd5, reason;
-    if (MD5File(fn, md5, &reason)) {
-	m_metaData[cstr_dj_keymd5] = MD5HexPrint(md5, xmd5);
-    } else {
-	LOGERR(("MimeHandlerMail: cant compute md5 for [%s]: %s\n", fn.c_str(),
-		reason.c_str()));
+    if (!m_forPreview) {
+	// Yes, we read the file twice. It would be possible in theory
+	// to add the md5 computation to the mime analysis, but ...
+	string md5, xmd5, reason;
+	if (MD5File(fn, md5, &reason)) {
+	    m_metaData[cstr_dj_keymd5] = MD5HexPrint(md5, xmd5);
+	} else {
+	    LOGERR(("MimeHandlerMail: cant md5 [%s]: %s\n", fn.c_str(),
+		    reason.c_str()));
+	}
     }
-
     m_fd = open(fn.c_str(), 0);
     if (m_fd < 0) {
 	LOGERR(("MimeHandlerMail::set_document_file: open(%s) errno %d\n",
@@ -122,15 +123,19 @@ bool MimeHandlerMail::set_document_file(const string &fn)
     return true;
 }
 
-bool MimeHandlerMail::set_document_string(const string &msgtxt)
+bool MimeHandlerMail::set_document_string(const string& mt, 
+					  const string &msgtxt)
 {
     LOGDEB1(("MimeHandlerMail::set_document_string\n"));
     LOGDEB2(("Message text: [%s]\n", msgtxt.c_str()));
+    RecollFilter::set_document_string(mt, msgtxt);
     delete m_stream;
 
-    string md5, xmd5;
-    MD5String(msgtxt, md5);
-    m_metaData[cstr_dj_keymd5] = MD5HexPrint(md5, xmd5);
+    if (!m_forPreview) {
+	string md5, xmd5;
+	MD5String(msgtxt, md5);
+	m_metaData[cstr_dj_keymd5] = MD5HexPrint(md5, xmd5);
+    }
 
     if ((m_stream = new stringstream(msgtxt)) == 0 || !m_stream->good()) {
 	LOGERR(("MimeHandlerMail::set_document_string: stream create error."
@@ -180,12 +185,15 @@ bool MimeHandlerMail::next_document()
     if (m_idx == -1) {
 	m_metaData[cstr_dj_keymt] = cstr_textplain;
 	res = processMsg(m_bincdoc, 0);
-	LOGDEB1(("MimeHandlerMail::next_document: mimetype %s\n",
-		m_metaData[cstr_dj_keymt].c_str()));
+	LOGDEB1(("MimeHandlerMail::next_document: mt %s, att cnt %d\n",
+		 m_metaData[cstr_dj_keymt].c_str(), m_attachments.size()));
         const string& txt = m_metaData[cstr_dj_keycontent];
         if (m_startoftext < txt.size())
             m_metaData[cstr_dj_keyabstract] = 
                 truncate_to_word(txt.substr(m_startoftext), 250);
+	if (m_attachments.size() > 0) {
+	    m_metaData[cstr_dj_keyanc] = "t";
+	}
     } else {
         m_metaData[cstr_dj_keyabstract].clear();
 	res = processAttach();
@@ -242,39 +250,27 @@ bool MimeHandlerMail::processAttach()
     MHMailAttach *att = m_attachments[m_idx];
 
     m_metaData[cstr_dj_keymt] = att->m_contentType;
+    m_metaData[cstr_dj_keyorigcharset] = att->m_charset;
     m_metaData[cstr_dj_keycharset] = att->m_charset;
     m_metaData[cstr_dj_keyfn] = att->m_filename;
-    // Change the title to something helpul
     m_metaData[cstr_dj_keytitle] = att->m_filename + "  (" + m_subject + ")";
     LOGDEB1(("  processAttach:ct [%s] cs [%s] fn [%s]\n", 
 	    att->m_contentType.c_str(),
 	    att->m_charset.c_str(),
 	    att->m_filename.c_str()));
 
+    // Erase current content and replace
     m_metaData[cstr_dj_keycontent] = string();
     string& body = m_metaData[cstr_dj_keycontent];
     att->m_part->getBody(body, 0, att->m_part->bodylength);
-    string decoded;
-    const string *bdp;
-    if (!decodeBody(att->m_contentTransferEncoding, body, decoded, &bdp)) {
-	return false;
-    }
-    if (bdp != &body)
-	body = decoded;
-
-    // Special case for text/plain content. Internfile should deal
-    // with this but it expects text/plain to be utf-8 already, so we
-    // handle the transcoding if needed
-    if (m_metaData[cstr_dj_keymt] == cstr_textplain) {
-	string utf8;
-	if (!transcode(body, utf8, m_metaData[cstr_dj_keycharset], "UTF-8")) {
-	    LOGERR(("  processAttach: transcode to utf-8 failed "
-		    "for charset [%s]\n", m_metaData[cstr_dj_keycharset].c_str()));
- 	    // can't transcode at all -> data is garbage just erase it
- 	    body.clear();
-	} else {
-	    body = utf8;
+    {
+	string decoded;
+	const string *bdp;
+	if (!decodeBody(att->m_contentTransferEncoding, body, decoded, &bdp)) {
+	    return false;
 	}
+	if (bdp != &body)
+	    body.swap(decoded);
     }
 
     // Special case for application/octet-stream: try to better
@@ -287,6 +283,14 @@ bool MimeHandlerMail::processAttach()
 	    m_metaData[cstr_dj_keymt] = mt;
     }
 
+    // Special case for text/plain content. Internfile should deal
+    // with this but it expects text/plain to be utf-8 already, so we
+    // handle the transcoding if needed
+    if (m_metaData[cstr_dj_keymt] == cstr_textplain) {
+	if (!txtdcode("MimeHandlerMail::processAttach"))
+	    body.clear();
+    }
+
     // Ipath
     char nbuf[20];
     sprintf(nbuf, "%d", m_idx);
@@ -316,32 +320,32 @@ bool MimeHandlerMail::processMsg(Binc::MimePart *doc, int depth)
     // Handle some headers. 
     string& text = m_metaData[cstr_dj_keycontent];
     Binc::HeaderItem hi;
-    string transcoded;
+    string decoded;
     if (doc->h.getFirstHeader("From", hi)) {
-	rfc2047_decode(hi.getValue(), transcoded);
+	rfc2047_decode(hi.getValue(), decoded);
 	if (preview())
 	    text += string("From: ");
-	text += transcoded + cstr_newline;
+	text += decoded + cstr_newline;
 	if (depth == 1) {
-	    m_metaData[cstr_dj_keyauthor] = transcoded;
+	    m_metaData[cstr_dj_keyauthor] = decoded;
 	}
     }
     if (doc->h.getFirstHeader("To", hi)) {
-	rfc2047_decode(hi.getValue(), transcoded);
+	rfc2047_decode(hi.getValue(), decoded);
 	if (preview())
 	    text += string("To: ");
-	text += transcoded + cstr_newline;
+	text += decoded + cstr_newline;
 	if (depth == 1) {
-	    m_metaData[cstr_dj_keyrecipient] = transcoded;
+	    m_metaData[cstr_dj_keyrecipient] = decoded;
 	}
     }
     if (doc->h.getFirstHeader("Cc", hi)) {
-	rfc2047_decode(hi.getValue(), transcoded);
+	rfc2047_decode(hi.getValue(), decoded);
 	if (preview())
 	    text += string("Cc: ");
-	text += transcoded + cstr_newline;
+	text += decoded + cstr_newline;
 	if (depth == 1) {
-	    m_metaData[cstr_dj_keyrecipient] += " " + transcoded;
+	    m_metaData[cstr_dj_keyrecipient] += " " + decoded;
 	}
     }
     if (doc->h.getFirstHeader("Message-Id", hi)) {
@@ -351,31 +355,31 @@ bool MimeHandlerMail::processMsg(Binc::MimePart *doc, int depth)
 	}
     }
     if (doc->h.getFirstHeader("Date", hi)) {
-	rfc2047_decode(hi.getValue(), transcoded);
+	rfc2047_decode(hi.getValue(), decoded);
 	if (depth == 1) {
-	    time_t t = rfc2822DateToUxTime(transcoded);
+	    time_t t = rfc2822DateToUxTime(decoded);
 	    if (t != (time_t)-1) {
 		char ascuxtime[100];
 		sprintf(ascuxtime, "%ld", (long)t);
 		m_metaData[cstr_dj_keymd] = ascuxtime;
 	    } else {
 		// Leave mtime field alone, ftime will be used instead.
-		LOGDEB(("rfc2822Date...: failed: [%s]\n", transcoded.c_str()));
+		LOGDEB(("rfc2822Date...: failed: [%s]\n", decoded.c_str()));
 	    }
 	}
 	if (preview())
 	    text += string("Date: ");
-	text += transcoded + cstr_newline;
+	text += decoded + cstr_newline;
     }
     if (doc->h.getFirstHeader("Subject", hi)) {
-	rfc2047_decode(hi.getValue(), transcoded);
+	rfc2047_decode(hi.getValue(), decoded);
 	if (depth == 1) {
-	    m_metaData[cstr_dj_keytitle] = transcoded;
-	    m_subject = transcoded;
+	    m_metaData[cstr_dj_keytitle] = decoded;
+	    m_subject = decoded;
 	}
 	if (preview())
 	    text += string("Subject: ");
-	text += transcoded + cstr_newline;
+	text += decoded + cstr_newline;
     }
 
     // Check for the presence of configured additional headers and possibly
@@ -503,7 +507,12 @@ void MimeHandlerMail::walkmime(Binc::MimePart* doc, int depth)
     it = content_disposition.params.find(string("filename"));
     if (it != content_disposition.params.end())
 	filename = it->second;
-
+    if (filename.empty()) {
+	it = content_type.params.find(string("name"));
+	if (it != content_type.params.end())
+	    filename = it->second;
+    }
+	
     if (doc->isMessageRFC822()) {
 	LOGDEB2(("walkmime: message/RFC822 part\n"));
 	
@@ -527,11 +536,13 @@ void MimeHandlerMail::walkmime(Binc::MimePart* doc, int depth)
 
     // "Simple" part. 
     LOGDEB2(("walkmime: simple  part\n"));
-    // Normally the default charset is us-ascii. But it happens that
-    // 8 bit chars exist in a message that is stated as us-ascii. Ie the 
-    // mailer used by yahoo support ('KANA') does this. We could convert 
-    // to iso-8859 only if the transfer-encoding is 8 bit, or test for
-    // actual 8 bit chars, but what the heck, le'ts use 8859-1 as default
+    // Normally the default charset is us-ascii. But it happens that 8
+    // bit chars exist in a message that is stated as us-ascii. Ie the
+    // mailer used by yahoo support ('KANA') does this. We could
+    // convert to iso-8859 only if the transfer-encoding is 8 bit, or
+    // test for actual 8 bit chars, but what the heck, le'ts use
+    // 8859-1 (actually CP1252 which is compatible, but with more
+    // useful chars) as default.
     string charset;
     it = content_type.params.find(cstr_mail_charset);
     if (it != content_type.params.end())
@@ -542,9 +553,8 @@ void MimeHandlerMail::walkmime(Binc::MimePart* doc, int depth)
 	!stringlowercmp("x-user-defined", charset) || 
 	!stringlowercmp("x-unknown", charset) || 
 	!stringlowercmp("unknown", charset) ) {
-        m_config->getConfParam("maildefcharset", charset);
-        if (charset.empty())
-            charset = "iso-8859-1";
+        if (!m_config->getConfParam("maildefcharset", charset))
+            charset = "CP1252";
     }
 
     // Content transfer encoding
@@ -590,40 +600,42 @@ void MimeHandlerMail::walkmime(Binc::MimePart* doc, int depth)
 	return;
     }
 
-    // We are dealing with an inline part of text/plain or text/html type
-
+    // We are dealing with an inline part of text/plain or text/html
+    // type. We can't just return a text or html subdoc and let the
+    // filter stack work: this would create another subdocument, but
+    // we want instead to decode a body part of this message document.
 
     LOGDEB2(("walkmime: final: body start offset %d, length %d\n", 
 	     doc->getBodyStartOffset(), doc->getBodyLength()));
     string body;
     doc->getBody(body, 0, doc->bodylength);
-
-    string decoded;
-    const string *bdp;
-    if (!decodeBody(cte, body, decoded, &bdp)) {
-	LOGERR(("MimeHandlerMail::walkmime: failed decoding body\n"));
+    {
+	string decoded;
+	const string *bdp;
+	if (!decodeBody(cte, body, decoded, &bdp)) {
+	    LOGERR(("MimeHandlerMail::walkmime: failed decoding body\n"));
+	}
+	if (bdp != &body)
+	    body.swap(decoded);
     }
-    if (bdp != &body)
-	body = decoded;
 
     // Handle html stripping and transcoding to utf8
-    string utf8;
-    const string *putf8 = 0;
     if (!stringlowercmp("text/html", content_type.value)) {
-	MimeHandlerHtml mh(m_config, "text/html");
+	MimeHandlerHtml mh(m_config, "1234");
 	mh.set_property(Dijon::Filter::OPERATING_MODE, 
 			m_forPreview ? "view" : "index");
 	mh.set_property(Dijon::Filter::DEFAULT_CHARSET, charset);
-	mh.set_document_string(body);
+	mh.set_document_string("text/html", body);
 	mh.next_document();
 	map<string, string>::const_iterator it = 
 	    mh.get_meta_data().find(cstr_dj_keycontent);
 	if (it != mh.get_meta_data().end())
 	    out += it->second;
     } else {
+	string utf8;
 	// Transcode to utf-8 
 	LOGDEB1(("walkmime: transcoding from %s to UTF-8\n", charset.c_str()));
-	if (!transcode(body, utf8, charset, "UTF-8")) {
+	if (!transcode(body, utf8, charset, cstr_utf8)) {
 	    LOGERR(("walkmime: transcode failed from cs '%s' to UTF-8\n",
 		    charset.c_str()));
 	    out += body;
