@@ -11,26 +11,10 @@
         <source>Any clause</source>
         <translation type="unfinished"></translation>
     </message>
-    <message>
-        <source>texts</source>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <source>spreadsheets</source>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <source>presentations</source>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>media</source>
         <translation type="unfinished"></translation>
     </message>
-    <message>
-        <source>messages</source>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>other</source>
         <translation type="unfinished"></translation>
@@ -39,6 +23,22 @@
         <source>Bad multiplier suffix in size filter</source>
         <translation type="unfinished"></translation>
     </message>
+    <message>
+        <source>text</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>spreadsheet</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>presentation</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>message</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>AdvSearchBase</name>
@@ -242,6 +242,56 @@ p, li { white-space: pre-wrap; }
         <translation type="unfinished"></translation>
     </message>
 </context>
+<context>
+    <name>EditTrans</name>
+    <message>
+        <source>Source path</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Local path</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Config error</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Original path</source>
+        <translation type="unfinished"></translation>
+    </message>
+</context>
+<context>
+    <name>EditTransBase</name>
+    <message>
+        <source>Path Translations</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Setting path translations for </source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Select one or several file types, then use the controls in the frame below to change how they are processed</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Add</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Delete</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Cancel</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Save</source>
+        <translation type="unfinished"></translation>
+    </message>
+</context>
 <context>
     <name>FirstIdxDialog</name>
     <message>
@@ -383,10 +433,6 @@ p, li { white-space: pre-wrap; }
         <source>Match &amp;Case</source>
         <translation type="unfinished"></translation>
     </message>
-    <message>
-        <source>Cannot create temporary directory: </source>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>Error while loading file</source>
         <translation type="unfinished"></translation>
@@ -490,7 +536,11 @@ p, li { white-space: pre-wrap; }
         <translation type="unfinished"></translation>
     </message>
     <message>
-        <source>Beagle web history</source>
+        <source>Search parameters</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Web history</source>
         <translation type="unfinished"></translation>
     </message>
 </context>
@@ -608,11 +658,6 @@ p, li { white-space: pre-wrap; }
         <source>This search is not active any more</source>
         <translation type="unfinished"></translation>
     </message>
-    <message>
-        <source>Bad viewer command line for %1: [%2]
-Please check the mimeconf file</source>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>Cannot extract document or create temporary file</source>
         <translation type="unfinished"></translation>
@@ -677,12 +722,6 @@ Please check the mimeconf file</source>
         <source>filtered</source>
         <translation type="unfinished"></translation>
     </message>
-    <message>
-        <source>External applications/commands needed and not found for indexing your file types:
-
-</source>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>No helpers found missing</source>
         <translation type="unfinished"></translation>
@@ -720,10 +759,6 @@ Do you want to start the  preferences dialog ?</source>
         <source>Result count (est.)</source>
         <translation type="unfinished"></translation>
     </message>
-    <message>
-        <source>Query details</source>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>Could not open external index. Db not open. Check external indexes list.</source>
         <translation type="unfinished"></translation>
@@ -784,10 +819,6 @@ Do you want to start the  preferences dialog ?</source>
         <source>Content has been indexed for these mime types:</source>
         <translation type="unfinished"></translation>
     </message>
-    <message>
-        <source>Index not up to date for this file. Refusing to risk showing the wrong entry. Click Ok to update the index for this file, then re-run the query when indexing is done. Else, Cancel.</source>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>Can&apos;t update index: indexer running</source>
         <translation type="unfinished"></translation>
@@ -796,6 +827,43 @@ Do you want to start the  preferences dialog ?</source>
         <source>Indexed MIME Types</source>
         <translation type="unfinished"></translation>
     </message>
+    <message>
+        <source>Bad viewer command line for %1: [%2]
+Please check the mimeview file</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Viewer command line for %1 specifies both file and parent file value: unsupported</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Cannot find parent document</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Indexing did not run yet</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>External applications/commands needed for your file types and not found, as stored by the last indexing pass in </source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Index not up to date for this file. Refusing to risk showing the wrong entry.</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Click Ok to update the index for this file, then re-run the query when indexing is done. Else, Cancel.</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Indexer running so things should improve when it&apos;s done</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Sub-documents and attachments</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>RclMainBase</name>
@@ -907,18 +975,10 @@ Do you want to start the  preferences dialog ?</source>
         <source>Previous page of results</source>
         <translation type="unfinished"></translation>
     </message>
-    <message>
-        <source>&amp;Query configuration</source>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>External index dialog</source>
         <translation type="unfinished"></translation>
     </message>
-    <message>
-        <source>&amp;Indexing configuration</source>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>All</source>
         <translation type="unfinished"></translation>
@@ -951,18 +1011,10 @@ Do you want to start the  preferences dialog ?</source>
         <source>&amp;Erase search history</source>
         <translation type="unfinished"></translation>
     </message>
-    <message>
-        <source>sortByDateAsc</source>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>Sort by dates from oldest to newest</source>
         <translation type="unfinished"></translation>
     </message>
-    <message>
-        <source>sortByDateDesc</source>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>Sort by dates from newest to oldest</source>
         <translation type="unfinished"></translation>
@@ -971,10 +1023,6 @@ Do you want to start the  preferences dialog ?</source>
         <source>Show Query Details</source>
         <translation type="unfinished"></translation>
     </message>
-    <message>
-        <source>Show results as table</source>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>&amp;Rebuild index</source>
         <translation type="unfinished"></translation>
@@ -995,6 +1043,54 @@ Do you want to start the  preferences dialog ?</source>
         <source>E&amp;xternal index dialog</source>
         <translation type="unfinished"></translation>
     </message>
+    <message>
+        <source>&amp;Index configuration</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>&amp;GUI configuration</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>&amp;Results</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Sort by date, oldest first</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Sort by date, newest first</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Show as table</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Show results in a spreadsheet-like table</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Save as CSV (spreadsheet) file</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Saves the result into a file which you can load in a spreadsheet</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Next Page</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Previous Page</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>First Page</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>RecollModel</name>
@@ -1077,22 +1173,6 @@ Do you want to start the  preferences dialog ?</source>
         <source>(show query)</source>
         <translation type="unfinished"></translation>
     </message>
-    <message>
-        <source>&amp;Preview</source>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <source>Copy &amp;File Name</source>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <source>Copy &amp;URL</source>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <source>Find &amp;similar documents</source>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>Document history</source>
         <translation type="unfinished"></translation>
@@ -1125,22 +1205,6 @@ Do you want to start the  preferences dialog ?</source>
         <source>&lt;p&gt;&lt;i&gt;Alternate spellings (accents suppressed): &lt;/i&gt;</source>
         <translation type="unfinished"></translation>
     </message>
-    <message>
-        <source>&amp;Write to File</source>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <source>Preview P&amp;arent document/folder</source>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <source>&amp;Open Parent document/folder</source>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <source>&amp;Open</source>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>Documents</source>
         <translation type="unfinished"></translation>
@@ -1153,6 +1217,26 @@ Do you want to start the  preferences dialog ?</source>
         <source>for</source>
         <translation type="unfinished"></translation>
     </message>
+    <message>
+        <source>&lt;p&gt;&lt;i&gt;Alternate spellings: &lt;/i&gt;</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Duplicate documents</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>These Urls ( | ipath) share the same content:</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Result count (est.)</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Query details</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>ResTable</name>
@@ -1172,38 +1256,6 @@ Do you want to start the  preferences dialog ?</source>
         <source>Can&apos;t open/create file: </source>
         <translation type="unfinished"></translation>
     </message>
-    <message>
-        <source>&amp;Preview</source>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <source>&amp;Open</source>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <source>Copy &amp;File Name</source>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <source>Copy &amp;URL</source>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <source>&amp;Write to File</source>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <source>Find &amp;similar documents</source>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <source>Preview P&amp;arent document/folder</source>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <source>&amp;Open Parent document/folder</source>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>&amp;Save as CSV</source>
         <translation type="unfinished"></translation>
@@ -1214,7 +1266,7 @@ Do you want to start the  preferences dialog ?</source>
     </message>
 </context>
 <context>
-    <name>ResTableDetailArea</name>
+    <name>ResultPopup</name>
     <message>
         <source>&amp;Preview</source>
         <translation type="unfinished"></translation>
@@ -1236,7 +1288,7 @@ Do you want to start the  preferences dialog ?</source>
         <translation type="unfinished"></translation>
     </message>
     <message>
-        <source>Find &amp;similar documents</source>
+        <source>Save selection to files</source>
         <translation type="unfinished"></translation>
     </message>
     <message>
@@ -1247,6 +1299,18 @@ Do you want to start the  preferences dialog ?</source>
         <source>&amp;Open Parent document/folder</source>
         <translation type="unfinished"></translation>
     </message>
+    <message>
+        <source>Find &amp;similar documents</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Open &amp;Snippets window</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Show subdocuments / attachments</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>SSearch</name>
@@ -1274,18 +1338,6 @@ Do you want to start the  preferences dialog ?</source>
         <source>Out of memory</source>
         <translation type="unfinished"></translation>
     </message>
-    <message>
-        <source>Too many completions</source>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <source>Completions</source>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <source>Select an item:</source>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>Enter query language expression. Cheat sheet:&lt;br&gt;
 &lt;i&gt;term1 term2&lt;/i&gt; : &apos;term1&apos; and &apos;term2&apos; in any field.&lt;br&gt;
@@ -1385,6 +1437,32 @@ Use &lt;b&gt;Show Query&lt;/b&gt; link when in doubt about result and see manual
         <translation type="unfinished"></translation>
     </message>
 </context>
+<context>
+    <name>Snippets</name>
+    <message>
+        <source>Snippets</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Find:</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Next</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Prev</source>
+        <translation type="unfinished"></translation>
+    </message>
+</context>
+<context>
+    <name>SnippetsW</name>
+    <message>
+        <source>Search</source>
+        <translation type="unfinished"></translation>
+    </message>
+</context>
 <context>
     <name>SpellBase</name>
     <message>
@@ -1411,6 +1489,18 @@ Use &lt;b&gt;Show Query&lt;/b&gt; link when in doubt about result and see manual
         <source>No db info.</source>
         <translation type="unfinished"></translation>
     </message>
+    <message>
+        <source>Match</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Case</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Accents</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>SpellW</name>
@@ -1455,7 +1545,55 @@ Use &lt;b&gt;Show Query&lt;/b&gt; link when in doubt about result and see manual
         <translation type="unfinished"></translation>
     </message>
     <message>
-        <source>Index: %1 documents, average length %2 terms</source>
+        <source>Index: %1 documents, average length %2 terms.%3 results</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>%1 results</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>List was truncated alphabetically, some frequent </source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>terms may be missing. Try using a longer root.</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Show index statistics</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Number of documents</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Average terms per document</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Smallest document length</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Longest document length</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Database directory size</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>MIME types:</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Item</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Value</source>
         <translation type="unfinished"></translation>
     </message>
 </context>
@@ -1478,20 +1616,36 @@ Use &lt;b&gt;Show Query&lt;/b&gt; link when in doubt about result and see manual
         <translation type="unfinished"></translation>
     </message>
     <message>
-        <source>Select xapian index directory (ie: /home/buddy/.recoll/xapiandb)</source>
+        <source>Choose</source>
         <translation type="unfinished"></translation>
     </message>
     <message>
-        <source>Choose</source>
+        <source>Result list paragraph format (erase all to reset to default)</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Result list header (default is empty)</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Select recoll config directory or xapian index directory (e.g.: /home/me/.recoll or /home/me/.recoll/xapiandb)</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>The selected directory looks like a Recoll configuration directory but the configuration could not be read</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>At most one index should be selected</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Cant add index with different case/diacritics stripping option</source>
         <translation type="unfinished"></translation>
     </message>
 </context>
 <context>
     <name>ViewAction</name>
-    <message>
-        <source>Changing actions with different current values</source>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>Command</source>
         <translation type="unfinished"></translation>
@@ -1500,6 +1654,14 @@ Use &lt;b&gt;Show Query&lt;/b&gt; link when in doubt about result and see manual
         <source>MIME type</source>
         <translation type="unfinished"></translation>
     </message>
+    <message>
+        <source>Desktop Default</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Changing entries with different current values</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>ViewActionBase</name>
@@ -1507,33 +1669,53 @@ Use &lt;b&gt;Show Query&lt;/b&gt; link when in doubt about result and see manual
         <source>Native Viewers</source>
         <translation type="unfinished"></translation>
     </message>
-    <message>
-        <source>Select one or several file types, then click Change Action to modify the program used to open them</source>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <source>Change Action</source>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>Close</source>
         <translation type="unfinished"></translation>
     </message>
     <message>
-        <source>Select one or several mime types then click &quot;Change Action&quot;&lt;br&gt;You can also close this dialog and check &quot;Use desktop preferences&quot;&lt;br&gt;in the main panel to ignore this list and use your desktop defaults.</source>
+        <source>Select one or several mime types then use the controls in the bottom frame to change how they are processed.</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Use Desktop preferences by default</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Select one or several file types, then use the controls in the frame below to change how they are processed</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Exception to Desktop preferences</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Action (empty -&gt; recoll default)</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Apply to current selection</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Recoll action:</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>current value</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Select same</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>&lt;b&gt;New Values:&lt;/b&gt;</source>
         <translation type="unfinished"></translation>
     </message>
 </context>
 <context>
     <name>confgui::ConfBeaglePanelW</name>
-    <message>
-        <source>Steal Beagle indexing queue</source>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <source>Beagle MUST NOT be running. Enables processing the beagle queue to index Firefox web history.&lt;br&gt;(you should also install the Firefox Beagle plugin)</source>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>Entries will be recycled once the size is reached</source>
         <translation type="unfinished"></translation>
@@ -1550,6 +1732,14 @@ Use &lt;b&gt;Show Query&lt;/b&gt; link when in doubt about result and see manual
         <source>Max. size for the web store (MB)</source>
         <translation type="unfinished"></translation>
     </message>
+    <message>
+        <source>Process the WEB history queue</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Enables indexing Firefox visited pages.&lt;br&gt;(you need also install the Firefox Recoll plugin)</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>confgui::ConfIndexW</name>
@@ -1576,6 +1766,41 @@ Use &lt;b&gt;Show Query&lt;/b&gt; link when in doubt about result and see manual
         <translation type="unfinished"></translation>
     </message>
 </context>
+<context>
+    <name>confgui::ConfSearchPanelW</name>
+    <message>
+        <source>Automatic diacritics sensitivity</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>&lt;p&gt;Automatically trigger diacritics sensitivity if the search term has accented characters (not in unac_except_trans). Else you need to use the query language and the &lt;i&gt;D&lt;/i&gt; modifier to specify diacritics sensitivity.</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Automatic character case sensitivity</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>&lt;p&gt;Automatically trigger character case sensitivity if the entry has upper-case characters in any but the first position. Else you need to use the query language and the &lt;i&gt;C&lt;/i&gt; modifier to specify character-case sensitivity.</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Maximum term expansion count</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>&lt;p&gt;Maximum expansion count for a single term (e.g.: when using wildcards). The default of 10 000 is reasonable and will avoid queries that appear frozen while the engine is walking the term list.</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Maximum Xapian clauses count</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>&lt;p&gt;Maximum number of elementary clauses we add to a single Xapian query. In some cases, the result of term expansion can be multiplicative, and we want to avoid using excessive memory. The default of 100 000 should be both high enough in most cases and compatible with current typical hardware configurations.</source>
+        <translation type="unfinished"></translation>
+    </message>
+</context>
 <context>
     <name>confgui::ConfSubPanelW</name>
     <message>
@@ -1613,7 +1838,7 @@ This will help searching very big text  files (ie: log files).</source>
         <translation type="unfinished"></translation>
     </message>
     <message>
-        <source>External filters working longer than this will be aborted. This is for the rare case (ie: postscript) where a document could cause a filter to loopSet to -1 for no limit.
+        <source>External filters working longer than this will be aborted. This is for the rare case (ie: postscript) where a document could cause a filter to loop. Set to -1 for no limit.
 </source>
         <translation type="unfinished"></translation>
     </message>
@@ -1684,30 +1909,30 @@ This will help searching very big text  files (ie: log files).</source>
         <source>Aspell language</source>
         <translation type="unfinished"></translation>
     </message>
-    <message>
-        <source>The language for the aspell dictionary. This should look like &apos;en&apos; or &apos;fr&apos; ...&lt;br&gt;If this value is not set, the NLS environment will be used to compute it, which usually works.To get an idea of what is installed on your system, type &apos;aspell config&apos; and look for .dat files inside the &apos;data-dir&apos; directory. </source>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>Database directory name</source>
         <translation type="unfinished"></translation>
     </message>
-    <message>
-        <source>The name for a directory where to store the index&lt;br&gt;A non-absolute path is taken relative to the  configuration directory. The default is &apos;xapiandb&apos;.</source>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <source>Use system&apos;s &apos;file&apos; command</source>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <source>Use the system&apos;s &apos;file&apos; command if internal&lt;br&gt;mime type identification fails.</source>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>Disables use of aspell to generate spelling approximation in the term explorer tool.&lt;br&gt; Useful if aspell is absent or does not work. </source>
         <translation type="unfinished"></translation>
     </message>
+    <message>
+        <source>The language for the aspell dictionary. This should look like &apos;en&apos; or &apos;fr&apos; ...&lt;br&gt;If this value is not set, the NLS environment will be used to compute it, which usually works. To get an idea of what is installed on your system, type &apos;aspell config&apos; and look for .dat files inside the &apos;data-dir&apos; directory. </source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>The name for a directory where to store the index&lt;br&gt;A non-absolute path is taken relative to the configuration directory. The default is &apos;xapiandb&apos;.</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Unac exceptions</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>&lt;p&gt;These are exceptions to the unac mechanism which, by default, removes all diacritics, and performs canonic decomposition. You can override unaccenting for some characters, depending on your language, and specify additional decompositions, e.g. for ligatures. In each space-separated entry, the first character is the source one, and the rest is the translation.</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>uiPrefsDialogBase</name>
@@ -1763,10 +1988,6 @@ This will help searching very big text  files (ie: log files).</source>
         <source>Maximum text size highlighted for preview (megabytes)</source>
         <translation type="unfinished"></translation>
     </message>
-    <message>
-        <source>Use desktop preferences to choose document editor.</source>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>Choose editor applications</source>
         <translation type="unfinished"></translation>
@@ -1869,10 +2090,6 @@ May be slow for big documents.</source>
         <source>Remove selected</source>
         <translation type="unfinished"></translation>
     </message>
-    <message>
-        <source>Click to add another index directory to the list</source>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>Add index</source>
         <translation type="unfinished"></translation>
@@ -1913,14 +2130,6 @@ May be slow for big documents.</source>
         <source>Resets the style sheet to default</source>
         <translation type="unfinished"></translation>
     </message>
-    <message>
-        <source>Lines in PRE text are not folded. Using BR loses some indentation.</source>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <source>Use &lt;PRE&gt; tags instead of &lt;BR&gt;to display plain text as html in preview.</source>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>Result List</source>
         <translation type="unfinished"></translation>
@@ -1948,5 +2157,53 @@ The default value is 2 (percent). </source>
         <source>Autophrase term frequency threshold percentage</source>
         <translation type="unfinished"></translation>
     </message>
+    <message>
+        <source>Plain text to HTML line style</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Lines in PRE text are not folded. Using BR loses some indentation. PRE + Wrap style may be what you want.</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>&lt;BR&gt;</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>&lt;PRE&gt;</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>&lt;PRE&gt; + wrap</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Disable Qt autocompletion in search entry.</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Search as you type.</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Paths translations</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Click to add another index directory to the list. You can select either a Recoll configuration directory or a Xapian index.</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Snippets window CSS file</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Opens a dialog to select the Snippets window CSS style sheet file</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Resets the Snippets window style</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 </TS>
