@@ -31,7 +31,6 @@
 #include <QTextDocument>
 #include <QPainter>
 #include <QSplitter>
-#include <QClipboard>
 #include <QFileDialog>
 #include <QMessageBox>
 
@@ -46,6 +45,12 @@
 #include "rclconfig.h"
 #include "plaintorich.h"
 #include "indexer.h"
+#include "respopup.h"
+#include "rclmain_w.h"
+#include "multisave.h"
+
+static const QKeySequence quitKeySeq("Ctrl+q");
+static const QKeySequence closeKeySeq("Ctrl+w");
 
 // Compensate for the default and somewhat bizarre vertical placement
 // of text in cells
@@ -86,8 +91,7 @@ private:
 
 bool ResTablePager::append(const string& data, int, const Rcl::Doc&)
 {
-    m_parent->m_detail->moveCursor(QTextCursor::End, 
-				      QTextCursor::MoveAnchor);
+    m_parent->m_detail->moveCursor(QTextCursor::End, QTextCursor::MoveAnchor);
     m_parent->m_detail->textCursor().insertBlock();
     m_parent->m_detail->insertHtml(QString::fromUtf8(data.c_str()));
     return true;
@@ -133,23 +137,14 @@ ResTableDetailArea::ResTableDetailArea(ResTable* parent)
 
 void ResTableDetailArea::createPopupMenu(const QPoint& pos)
 {
-    if (!m_table || m_table->m_detaildocnum < 0) {
-	LOGDEB(("ResTableDetailArea::createPopupMenu: no table/detaildoc\n"));
-	return;
+    if (m_table && m_table->m_model && m_table->m_detaildocnum >= 0) {
+	int opts = m_table->m_ismainres ? ResultPopup::showExpand : 0;
+	opts |= ResultPopup::showSaveOne;
+	QMenu *popup = ResultPopup::create(m_table, opts, 
+					   m_table->m_model->getDocSource(),
+					   m_table->m_detaildoc);
+	popup->popup(mapToGlobal(pos));
     }
-    QMenu *popup = new QMenu(this);
-    popup->addAction(tr("&Preview"), m_table, SLOT(menuPreview()));
-    popup->addAction(tr("&Open"), m_table, SLOT(menuEdit()));
-    popup->addAction(tr("Copy &File Name"), m_table, SLOT(menuCopyFN()));
-    popup->addAction(tr("Copy &URL"), m_table, SLOT(menuCopyURL()));
-    if (!m_table->m_detaildoc.ipath.empty())
-	popup->addAction(tr("&Write to File"), m_table, SLOT(menuSaveToFile()));
-    popup->addAction(tr("Find &similar documents"), m_table, SLOT(menuExpand()));
-    popup->addAction(tr("Preview P&arent document/folder"), 
-		      m_table, SLOT(menuPreviewParent()));
-    popup->addAction(tr("&Open Parent document/folder"), 
-		      m_table, SLOT(menuOpenParent()));
-    popup->popup(mapToGlobal(pos));
 }
 
 //////////////////////////////////////////////////////////////////////////////
@@ -184,7 +179,7 @@ static string dategetter(const string&, const Rcl::Doc& doc)
     datebuf[0] = 0;
     if (!doc.dmtime.empty() || !doc.fmtime.empty()) {
 	time_t mtime = doc.dmtime.empty() ?
-	    atol(doc.fmtime.c_str()) : atol(doc.dmtime.c_str());
+	    atoll(doc.fmtime.c_str()) : atoll(doc.dmtime.c_str());
 	struct tm *tm = localtime(&mtime);
 	strftime(datebuf, 99, "%Y-%m-%d", tm);
     }
@@ -197,7 +192,7 @@ static string datetimegetter(const string&, const Rcl::Doc& doc)
     datebuf[0] = 0;
     if (!doc.dmtime.empty() || !doc.fmtime.empty()) {
 	time_t mtime = doc.dmtime.empty() ?
-	    atol(doc.fmtime.c_str()) : atol(doc.dmtime.c_str());
+	    atoll(doc.fmtime.c_str()) : atoll(doc.dmtime.c_str());
 	struct tm *tm = localtime(&mtime);
 	strftime(datebuf, 99, "%Y-%m-%d %H:%M:%S %z", tm);
     }
@@ -299,8 +294,8 @@ void RecollModel::setDocSource(RefCntr<DocSequence> nsource)
 	m_source = RefCntr<DocSequence>();
     } else {
 	m_source = RefCntr<DocSequence>(new DocSource(theconfig, nsource));
-	m_hdata.reset();
-	m_source->getTerms(m_hdata.terms, m_hdata.groups, m_hdata.gslks);
+	m_hdata.clear();
+	m_source->getTerms(m_hdata);
     }
 }
 
@@ -487,7 +482,6 @@ void ResTable::init()
     tableView->setModel(m_model);
     tableView->setMouseTracking(true);
     tableView->setSelectionBehavior(QAbstractItemView::SelectRows);
-    tableView->setSelectionMode(QAbstractItemView::SingleSelection);
     tableView->setItemDelegate(new ResTableDelegate(this));
     tableView->setContextMenuPolicy(Qt::CustomContextMenu);
     connect(tableView, SIGNAL(customContextMenuRequested(const QPoint&)),
@@ -503,6 +497,7 @@ void ResTable::init()
 	header->setSortIndicatorShown(true);
 	header->setSortIndicator(-1, Qt::AscendingOrder);
 	header->setContextMenuPolicy(Qt::CustomContextMenu);
+	header->setStretchLastSection(1);
 	connect(header, SIGNAL(sectionResized(int,int,int)),
 		this, SLOT(saveColState()));
 	connect(header, SIGNAL(customContextMenuRequested(const QPoint&)),
@@ -520,7 +515,8 @@ void ResTable::init()
     QShortcut *sc = new QShortcut(seq, this);
     connect(sc, SIGNAL (activated()), 
 	    tableView->selectionModel(), SLOT (clear()));
-    connect(tableView->selectionModel(), SIGNAL(currentChanged(const QModelIndex&, const QModelIndex &)),
+    connect(tableView->selectionModel(), 
+	    SIGNAL(currentChanged(const QModelIndex&, const QModelIndex &)),
 	    this, SLOT(onTableView_currentChanged(const QModelIndex&)));
 
     m_pager = new ResTablePager(this);
@@ -547,6 +543,33 @@ void ResTable::init()
     splitter->setOrientation(Qt::Vertical);
 }
 
+void ResTable::setRclMain(RclMain *m, bool ismain) 
+{
+    m_rclmain = m;
+    m_ismainres = ismain;
+
+    // We allow single selection only in the main table because this
+    // may have a mix of file-level docs and subdocs and multisave
+    // only works for subdocs
+    if (m_ismainres)
+	tableView->setSelectionMode(QAbstractItemView::SingleSelection);
+    else
+	tableView->setSelectionMode(QAbstractItemView::ExtendedSelection);
+
+    if (!m_ismainres) {
+	connect(new QShortcut(closeKeySeq, this), SIGNAL (activated()), 
+		this, SLOT (close()));
+	connect(new QShortcut(quitKeySeq, this), SIGNAL (activated()), 
+		m_rclmain, SLOT (fileExit()));
+	connect(this, SIGNAL(previewRequested(Rcl::Doc)), 
+		m_rclmain, SLOT(startPreview(Rcl::Doc)));
+	connect(this, SIGNAL(docSaveToFileClicked(Rcl::Doc)), 
+		m_rclmain, SLOT(saveDocToFile(Rcl::Doc)));
+	connect(this, SIGNAL(editRequested(Rcl::Doc)), 
+		m_rclmain, SLOT(startNativeViewer(Rcl::Doc)));
+    }
+}
+
 int ResTable::getDetailDocNumOrTopRow()
 {
     if (m_detaildocnum >= 0)
@@ -568,6 +591,8 @@ void ResTable::makeRowVisible(int row)
 // This is called by rclmain_w prior to exiting
 void ResTable::saveColState()
 {
+    if (!m_ismainres)
+	return;
     QSettings settings;
     settings.setValue("resTableSplitterSizes", splitter->saveState());
 
@@ -607,7 +632,8 @@ void ResTable::onTableView_currentChanged(const QModelIndex& index)
 	m_detail->clear();
 	m_detaildocnum = index.row();
 	m_detaildoc = doc;
-	m_pager->displayDoc(theconfig, index.row(), doc, m_model->m_hdata);
+	m_pager->displayDoc(theconfig, index.row(), m_detaildoc, 
+			    m_model->m_hdata);
     } else {
 	m_detaildocnum = -1;
     }
@@ -649,6 +675,8 @@ void ResTable::saveAsCSV()
 				     tr("Save table to CSV file"),
 				     QString::fromLocal8Bit(path_home().c_str())
 	    );
+    if (s.isEmpty())
+	return;
     const char *tofile = s.toLocal8Bit();
     FILE *fp = fopen(tofile, "w");
     if (fp == 0) {
@@ -726,7 +754,7 @@ void ResTable::linkWasClicked(const QUrl &url)
 	if (what == 'P')
 	    emit docPreviewClicked(i, m_detaildoc, 0);
 	else
-	    emit docEditClicked(m_detaildoc);
+	    emit editRequested(m_detaildoc);
     }
     break;
     default: 
@@ -738,120 +766,117 @@ void ResTable::linkWasClicked(const QUrl &url)
 void ResTable::createPopupMenu(const QPoint& pos)
 {
     LOGDEB(("ResTable::createPopupMenu: m_detaildocnum %d\n", m_detaildocnum));
-    if (m_detaildocnum < 0)
-	return;
-    QMenu *popup = new QMenu(this);
-    popup->addAction(tr("&Preview"), this, SLOT(menuPreview()));
-    popup->addAction(tr("&Open"), this, SLOT(menuEdit()));
-    popup->addAction(tr("Copy &File Name"), this, SLOT(menuCopyFN()));
-    popup->addAction(tr("Copy &URL"), this, SLOT(menuCopyURL()));
-    if (m_detaildoc.ipath.empty())
-	popup->addAction(tr("&Write to File"), this, SLOT(menuSaveToFile()));
-    popup->addAction(tr("Find &similar documents"), this, SLOT(menuExpand()));
-    popup->addAction(tr("Preview P&arent document/folder"), 
-		      this, SLOT(menuPreviewParent()));
-    popup->addAction(tr("&Open Parent document/folder"), 
-		      this, SLOT(menuOpenParent()));
-    popup->popup(mapToGlobal(pos));
+    if (m_detaildocnum >= 0 && m_model) {
+	int opts = m_ismainres? ResultPopup::isMain : 0;
+    
+	int selsz = tableView->selectionModel()->selectedRows().size();
+
+	if (selsz == 1) {
+	    opts |= ResultPopup::showSaveOne;
+	} else if (selsz > 1 && !m_ismainres) {
+	    // We don't show save multiple for the main list because not all 
+	    // docs are necessary subdocs and multisave only works with those.
+	    opts |= ResultPopup::showSaveSel;
+	}
+	QMenu *popup = ResultPopup::create(this, opts, m_model->getDocSource(),
+					   m_detaildoc);
+	popup->popup(mapToGlobal(pos));
+    }
 }
 
 void ResTable::menuPreview()
 {
-    if (m_detaildocnum < 0) 
-	return;
-    emit docPreviewClicked(m_detaildocnum, m_detaildoc, 0);
+    if (m_detaildocnum >= 0) {
+	if (m_ismainres) {
+	    emit docPreviewClicked(m_detaildocnum, m_detaildoc, 0);
+	} else {
+	    emit previewRequested(m_detaildoc);
+	}
+    }
 }
 
 void ResTable::menuSaveToFile()
 {
-    if (m_detaildocnum < 0) 
+    if (m_detaildocnum >= 0) 
+	emit docSaveToFileClicked(m_detaildoc);
+}
+
+void ResTable::menuSaveSelection()
+{
+    if (m_model == 0 || m_model->getDocSource().isNull())
 	return;
-    emit docSaveToFileClicked(m_detaildoc);
+
+    QModelIndexList indexl = tableView->selectionModel()->selectedRows();
+    vector<Rcl::Doc> v;
+    for (int i = 0; i < indexl.size(); i++) {
+	Rcl::Doc doc;
+	if (m_model->getDocSource()->getDoc(indexl[i].row(), doc))
+	    v.push_back(doc);
+    }
+    if (v.size() == 0) {
+	return;
+    } else if (v.size() == 1) {
+	emit docSaveToFileClicked(v[0]);
+    } else {
+	multiSave(this, v);
+    }
 }
 
 void ResTable::menuPreviewParent()
 {
-    if (!m_model || m_detaildocnum < 0) 
-	return;
-    RefCntr<DocSequence> source = m_model->getDocSource();
-    if (source.isNull())
-	return;
-    Rcl::Doc& doc = m_detaildoc;
-    Rcl::Doc pdoc;
-    if (source->getEnclosing(doc, pdoc)) {
-	emit previewRequested(pdoc);
-    } else {
-	// No parent doc: show enclosing folder with app configured for
-	// directories
-	pdoc.url = path_getfather(doc.url);
-	pdoc.mimetype = "application/x-fsdirectory";
-	emit editRequested(pdoc);
+    if (m_detaildocnum >= 0 && m_model &&  
+	m_model->getDocSource().isNotNull()) {
+	Rcl::Doc pdoc = ResultPopup::getParent(m_model->getDocSource(), 
+					      m_detaildoc);
+	if (pdoc.mimetype == "inode/directory") {
+	    emit editRequested(pdoc);
+	} else {
+	    emit previewRequested(pdoc);
+	}
     }
 }
 
 void ResTable::menuOpenParent()
 {
-    if (!m_model || m_detaildocnum < 0) 
-	return;
-    RefCntr<DocSequence> source = m_model->getDocSource();
-    if (source.isNull())
-	return;
-    Rcl::Doc& doc = m_detaildoc;
-    Rcl::Doc pdoc;
-    if (source->getEnclosing(doc, pdoc)) {
-	emit editRequested(pdoc);
-    } else {
-	// No parent doc: show enclosing folder with app configured for
-	// directories
-	pdoc.url = path_getfather(doc.url);
-	pdoc.mimetype = "application/x-fsdirectory";
-	emit editRequested(pdoc);
-    }
+    if (m_detaildocnum >= 0 && m_model && m_model->getDocSource().isNotNull())
+	emit editRequested(
+	    ResultPopup::getParent(m_model->getDocSource(), m_detaildoc));
 }
 
 void ResTable::menuEdit()
 {
-    if (m_detaildocnum < 0) 
-	return;
-    emit docEditClicked(m_detaildoc);
+    if (m_detaildocnum >= 0) 
+	emit editRequested(m_detaildoc);
 }
 
 void ResTable::menuCopyFN()
 {
-    if (m_detaildocnum < 0) 
-	return;
-    Rcl::Doc &doc = m_detaildoc;
-
-    // Our urls currently always begin with "file://" 
-    //
-    // Problem: setText expects a QString. Passing a (const char*)
-    // as we used to do causes an implicit conversion from
-    // latin1. File are binary and the right approach would be no
-    // conversion, but it's probably better (less worse...) to
-    // make a "best effort" tentative and try to convert from the
-    // locale's charset than accept the default conversion.
-    QString qfn = QString::fromLocal8Bit(doc.url.c_str()+7);
-    QApplication::clipboard()->setText(qfn, QClipboard::Selection);
-    QApplication::clipboard()->setText(qfn, QClipboard::Clipboard);
+    if (m_detaildocnum >= 0) 
+	ResultPopup::copyFN(m_detaildoc);
 }
 
 void ResTable::menuCopyURL()
 {
-    if (m_detaildocnum < 0) 
-	return;
-    Rcl::Doc &doc = m_detaildoc;
-    string url =  url_encode(doc.url, 7);
-    QApplication::clipboard()->setText(url.c_str(), 
-				       QClipboard::Selection);
-    QApplication::clipboard()->setText(url.c_str(), 
-				       QClipboard::Clipboard);
+    if (m_detaildocnum >= 0) 
+	ResultPopup::copyURL(m_detaildoc);
 }
 
 void ResTable::menuExpand()
 {
-    if (m_detaildocnum < 0) 
-	return;
-    emit docExpand(m_detaildoc);
+    if (m_detaildocnum >= 0) 
+	emit docExpand(m_detaildoc);
+}
+
+void ResTable::menuShowSnippets()
+{
+    if (m_detaildocnum >= 0)
+	emit showSnippets(m_detaildoc);
+}
+
+void ResTable::menuShowSubDocs()
+{
+    if (m_detaildocnum >= 0)
+	emit showSubDocs(m_detaildoc);
 }
 
 void ResTable::createHeaderPopupMenu(const QPoint& pos)
@@ -902,10 +927,7 @@ void ResTable::addColumn()
     if (!m_model)
 	return;
     QAction *action = (QAction *)sender();
-    LOGDEB(("addColumn: text %s, data %s\n", 
-	    (const char *)action->text().toUtf8(),
-	    (const char *)action->data().toString().toUtf8()
-	       ));
-    string field((const char *)action->data().toString().toUtf8());
-    m_model->addColumn(m_popcolumn, field);
+    LOGDEB(("addColumn: text %s, data %s\n", qs2utf8s(action->text()).c_str(), 
+	    qs2utf8s(action->data().toString()).c_str()));
+    m_model->addColumn(m_popcolumn, qs2utf8s(action->data().toString()));
 }
