@@ -17,8 +17,11 @@
 #ifndef _SSEARCH_W_H_INCLUDED_
 #define _SSEARCH_W_H_INCLUDED_
 
-#include <qvariant.h>
-#include <qwidget.h>
+#include <QVariant>
+#include <QWidget>
+
+class QTimer;
+
 #include "recoll.h"
 #include "searchdata.h"
 #include "refcntr.h"
@@ -38,13 +41,13 @@ public:
 	setupUi(this);
 	init();
     }
-    ~SSearch(){}
 
     virtual void init();
     virtual void setAnyTermMode();
     virtual void completion();
     virtual bool eventFilter(QObject *target, QEvent *event);
     virtual bool hasSearchString();
+    virtual void setPrefs();				  
 public slots:
     virtual void searchTextChanged(const QString & text);
     virtual void searchTypeChanged(int);
@@ -52,11 +55,31 @@ public slots:
     virtual void startSimpleSearch();
     virtual void addTerm(QString);
     virtual void onWordReplace(const QString&, const QString&);
+    virtual void completionTermChosen(const QString& text);
+    virtual void wrapupCompletion();
+    virtual void timerDone();
+    virtual void takeFocus();
+
 signals:
-    void startSearch(RefCntr<Rcl::SearchData>);
+    void startSearch(RefCntr<Rcl::SearchData>, bool);
     void clearSearch();
  private:
     bool m_escape;
+
+    bool m_displayingCompletions;
+    QString m_chosenCompletion;
+    QString m_savedEditText;
+    unsigned int m_completedWordStart;
+
+    bool m_disableAutosearch;
+    QTimer *m_stroketimeout;
+    bool m_keystroke;
+    QString m_tstartqs;
+    QAbstractItemModel *m_savedModel;
+
+    int partialWord(string& s);
+    int completionList(string s, QStringList& lst, int max = 100);
+    bool startSimpleSearch(const string& q, int maxexp = -1);
 };
 
 
