@@ -17,10 +17,12 @@
 #include <math.h>
 #include <time.h>
 
+#include <list>
+using std::list;
+
 #include "docseqdb.h"
 #include "rcldb.h"
 #include "debuglog.h"
-#include "internfile.h"
 #include "wasatorcl.h"
 
 DocSequenceDb::DocSequenceDb(RefCntr<Rcl::Query> q, const string &t, 
@@ -31,7 +33,8 @@ DocSequenceDb::DocSequenceDb(RefCntr<Rcl::Query> q, const string &t,
       m_queryReplaceAbstract(false),
       m_isFiltered(false),
       m_isSorted(false),
-      m_needSetQuery(false)
+      m_needSetQuery(false),
+      m_lastSQStatus(true)
 {
 }
 
@@ -39,16 +42,9 @@ DocSequenceDb::~DocSequenceDb()
 {
 }
 
-bool DocSequenceDb::getTerms(vector<string>& terms, 
-			     vector<vector<string> >& groups, 
-			     vector<int>& gslks)
+void DocSequenceDb::getTerms(HighlightData& hld)
 {
-    return m_fsdata->getTerms(terms, groups, gslks);
-}
-
-void DocSequenceDb::getUTerms(vector<string>& terms)
-{
-    m_sdata->getUTerms(terms);
+    m_fsdata->getTerms(hld);
 }
 
 string DocSequenceDb::getDescription() 
@@ -58,47 +54,91 @@ string DocSequenceDb::getDescription()
 
 bool DocSequenceDb::getDoc(int num, Rcl::Doc &doc, string *sh)
 {
-    setQuery();
+    if (!setQuery())
+	return false;
     if (sh) sh->erase();
     return m_q->getDoc(num, doc);
 }
 
 int DocSequenceDb::getResCnt()
 {
-    setQuery();
+    if (!setQuery())
+	return false;
     if (m_rescnt < 0) {
 	m_rescnt= m_q->getResCnt();
     }
     return m_rescnt;
 }
 
-bool DocSequenceDb::getAbstract(Rcl::Doc &doc, vector<string>& vabs)
+static const string cstr_mre("[...]");
+
+// This one only gets called to fill-up the snippets window
+// We ignore most abstract/snippets preferences.
+bool DocSequenceDb::getAbstract(Rcl::Doc &doc, vector<Rcl::Snippet>& vpabs)
 {
-    setQuery();
-    if (m_q->whatDb() &&
-	m_queryBuildAbstract && (doc.syntabs || m_queryReplaceAbstract)) {
-	m_q->whatDb()->makeDocAbstract(doc, m_q.getptr(), vabs);
+    LOGDEB(("DocSequenceDb::getAbstract/pair\n"));
+    if (!setQuery())
+	return false;
+
+    // Have to put the limit somewhere. 
+    int maxoccs = 1000;
+    int ret = Rcl::ABSRES_ERROR;
+    if (m_q->whatDb()) {
+	ret = m_q->makeDocAbstract(doc, vpabs, maxoccs, 
+				   m_q->whatDb()->getAbsCtxLen()+ 2);
     } 
-    if (vabs.empty())
-	vabs.push_back(doc.meta[Rcl::Doc::keyabs]);
+    LOGDEB(("DocSequenceDb::getAbstract: got ret %d vpabs len %u\n", ret,
+	    vpabs.size()));
+    if (vpabs.empty()) {
+	return true;
+    }
+
+    // If the list was probably truncated, indicate it.
+    if (ret & Rcl::ABSRES_TRUNC) {
+	vpabs.push_back(Rcl::Snippet(-1, cstr_mre));
+    } 
+    if (ret & Rcl::ABSRES_TERMMISS) {
+	vpabs.insert(vpabs.begin(), 
+		     Rcl::Snippet(-1, "(Words missing in snippets)"));
+    }
 
     return true;
 }
 
-bool DocSequenceDb::getEnclosing(Rcl::Doc& doc, Rcl::Doc& pdoc) 
+bool DocSequenceDb::getAbstract(Rcl::Doc &doc, vector<string>& vabs)
 {
-    setQuery();
-    string udi;
-    if (!FileInterner::getEnclosing(doc.url, doc.ipath, pdoc.url, pdoc.ipath,
-                                    udi))
-        return false;
-    return m_q->whatDb()->getDoc(udi, pdoc);
+    if (!setQuery())
+	return false;
+    if (m_q->whatDb() &&
+	m_queryBuildAbstract && (doc.syntabs || m_queryReplaceAbstract)) {
+	m_q->makeDocAbstract(doc, vabs);
+    } 
+    if (vabs.empty())
+	vabs.push_back(doc.meta[Rcl::Doc::keyabs]);
+    return true;
+}
+
+int DocSequenceDb::getFirstMatchPage(Rcl::Doc &doc, string& term)
+{
+    if (!setQuery())
+	return false;
+    if (m_q->whatDb()) {
+	return m_q->getFirstMatchPage(doc, term);
+    }
+    return -1;
+}
+
+Rcl::Db *DocSequenceDb::getDb()
+{
+    return m_q.isNotNull() ? m_q->whatDb() : 0;
 }
 
 list<string> DocSequenceDb::expand(Rcl::Doc &doc)
 {
-    setQuery();
-    return m_q->expand(doc);
+    if (!setQuery())
+	return list<string>();
+    vector<string> v = m_q->expand(doc);
+    return list<string>(v.begin(), v.end());
 }
 
 string DocSequenceDb::title()
@@ -118,9 +158,10 @@ bool DocSequenceDb::setFiltSpec(const DocSeqFiltSpec &fs)
     LOGDEB(("DocSequenceDb::setFiltSpec\n"));
     if (fs.isNotNull()) {
 	// We build a search spec by adding a filtering layer to the base one.
-	m_fsdata = RefCntr<Rcl::SearchData>(new Rcl::SearchData(Rcl::SCLT_AND));
+	m_fsdata = RefCntr<Rcl::SearchData>(
+	    new Rcl::SearchData(Rcl::SCLT_AND, m_sdata->getStemLang()));
 	Rcl::SearchDataClauseSub *cl = 
-	    new Rcl::SearchDataClauseSub(Rcl::SCLT_SUB, m_sdata);
+	    new Rcl::SearchDataClauseSub(m_sdata);
 	m_fsdata->addClause(cl);
     
 	for (unsigned int i = 0; i < fs.crits.size(); i++) {
@@ -136,11 +177,12 @@ bool DocSequenceDb::setFiltSpec(const DocSeqFiltSpec &fs)
 		string reason;
 		Rcl::SearchData *sd = 
 		    wasaStringToRcl(m_q->whatDb()->getConf(), 
+				    m_sdata->getStemLang(),
 				    fs.values[i], reason);
 		if (sd)  {
 		    Rcl::SearchDataClauseSub *cl1 = 
-			new Rcl::SearchDataClauseSub(Rcl::SCLT_SUB, 
-						     RefCntr<Rcl::SearchData>(sd));
+			new Rcl::SearchDataClauseSub(
+			    RefCntr<Rcl::SearchData>(sd));
 		    m_fsdata->addClause(cl1);
 		}
 	    }
@@ -177,7 +219,23 @@ bool DocSequenceDb::setQuery()
 {
     if (!m_needSetQuery)
 	return true;
+
+    m_needSetQuery = false;
     m_rescnt = -1;
-    m_needSetQuery = !m_q->setQuery(m_fsdata);
-    return !m_needSetQuery;
+    m_lastSQStatus = m_q->setQuery(m_fsdata);
+    if (!m_lastSQStatus) {
+	m_reason = m_q->getReason();
+	LOGERR(("DocSequenceDb::setQuery: rclquery::setQuery failed: %s\n",
+		m_reason.c_str()));
+    }
+    return m_lastSQStatus;
+}
+
+bool DocSequenceDb::docDups(const Rcl::Doc& doc, std::vector<Rcl::Doc>& dups)
+{
+    if (m_q->whatDb()) {
+	return m_q->whatDb()->docDups(doc, dups);
+    } else {
+	return false;
+    }
 }
