@@ -19,8 +19,9 @@
 #include <string.h>
 #include <stdio.h>
 
-#include <list>
 #include <vector>
+#include <sstream>
+using namespace std;
 
 #include "xapian.h"
 
@@ -36,10 +37,11 @@
 #include "searchdata.h"
 #include "unacpp.h"
 
-#ifndef NO_NAMESPACES
 namespace Rcl {
-#endif
-
+// This is used as a marker inside the abstract frag lists, but
+// normally doesn't remain in final output (which is built with a
+// custom sep. by our caller).
+static const string cstr_ellipsis("...");
 
 // Field names inside the index data record may differ from the rcldoc ones
 // (esp.: caption / title)
@@ -58,14 +60,21 @@ static const string& docfToDatf(const string& df)
 // special processing for special fields like dates and sizes. User
 // custom field data will have to be processed before insertion to
 // achieve equivalent results.
+#if XAPIAN_MAJOR_VERSION == 1 && XAPIAN_MINOR_VERSION < 2
 class QSorter : public Xapian::Sorter {
+#else
+class QSorter : public Xapian::KeyMaker {
+#endif
 public:
     QSorter(const string& f) 
 	: m_fld(docfToDatf(f) + "=") 
     {
 	m_ismtime = !m_fld.compare("dmtime=");
-	m_issize = !m_fld.compare("fbytes=") || !m_fld.compare("dbytes=") ||
-	    !m_fld.compare("pcbytes=");
+	if (m_ismtime)
+	    m_issize = false;
+	else 
+	    m_issize = !m_fld.compare("fbytes=") || !m_fld.compare("dbytes=") ||
+		!m_fld.compare("pcbytes=");
     }
 
     virtual std::string operator()(const Xapian::Document& xdoc) const 
@@ -111,7 +120,7 @@ public:
 	// approach to collating...)
 	string sortterm;
 	// We're not even sure the term is utf8 here (ie: url)
-	if (!unacmaybefold(term, sortterm, "UTF-8", true)) {
+	if (!unacmaybefold(term, sortterm, "UTF-8", UNACOP_UNACFOLD)) {
 	    sortterm = term;
 	}
 	// Also remove some common uninteresting starting characters
@@ -132,8 +141,10 @@ private:
 
 Query::Query(Db *db)
     : m_nq(new Native(this)), m_db(db), m_sorter(0), m_sortAscending(true),
-      m_collapseDuplicates(false), m_resCnt(-1)
+      m_collapseDuplicates(false), m_resCnt(-1), m_snipMaxPosWalk(1000000)
 {
+    if (db)
+	db->getConf()->getConfParam("snippetMaxPosWalk", &m_snipMaxPosWalk);
 }
 
 Query::~Query()
@@ -145,16 +156,6 @@ Query::~Query()
     }
 }
 
-string Query::getReason() const
-{
-    return m_reason;
-}
-
-Db *Query::whatDb() 
-{
-    return m_db;
-}
-
 void Query::setSortBy(const string& fld, bool ascending) {
     if (fld.empty()) {
 	m_sortField.erase();
@@ -182,7 +183,8 @@ bool Query::setQuery(RefCntr<SearchData> sdata)
     m_reason.erase();
 
     m_nq->clear();
-
+    m_sd = sdata;
+    
     Xapian::Query xq;
     if (!sdata->toNativeQuery(*m_db, &xq)) {
 	m_reason += sdata->getReason();
@@ -201,16 +203,17 @@ bool Query::setQuery(RefCntr<SearchData> sdata)
                 m_nq->xenquire->set_collapse_key(Xapian::BAD_VALUENO);
             }
 	    m_nq->xenquire->set_docid_order(Xapian::Enquire::DONT_CARE);
-            if (!m_sortField.empty()) {
+            if (!m_sortField.empty() && 
+		stringlowercmp("relevancyrating", m_sortField)) {
                 if (m_sorter) {
                     delete (QSorter*)m_sorter;
                     m_sorter = 0;
                 }
-                m_sorter = new QSorter(m_sortField);
-                // It really seems there is a xapian bug about sort order, we 
-                // invert here.
-                m_nq->xenquire->set_sort_by_key((QSorter*)m_sorter, 
-                                                !m_sortAscending);
+		m_sorter = new QSorter(m_sortField);
+		// It really seems there is a xapian bug about sort order, we 
+		// invert here.
+		m_nq->xenquire->set_sort_by_key((QSorter*)m_sorter, 
+						!m_sortAscending);
             }
             m_nq->xenquire->set_query(m_nq->xquery);
             m_nq->xmset = Xapian::MSet();
@@ -235,12 +238,12 @@ bool Query::setQuery(RefCntr<SearchData> sdata)
 	d.erase(0, strlen("Xapian::Query"));
 
     sdata->setDescription(d);
+    m_sd = sdata;
     LOGDEB(("Query::SetQuery: Q: %s\n", sdata->getDescription().c_str()));
     return true;
 }
 
-
-bool Query::getQueryTerms(list<string>& terms)
+bool Query::getQueryTerms(vector<string>& terms)
 {
     if (ISNULL(m_nq))
 	return false;
@@ -261,32 +264,69 @@ bool Query::getQueryTerms(list<string>& terms)
     return true;
 }
 
-bool Query::getMatchTerms(const Doc& doc, list<string>& terms)
+int Query::makeDocAbstract(const Doc &doc,
+			   vector<Snippet>& abstract, 
+			   int maxoccs, int ctxwords)
 {
-    return getMatchTerms(doc.xdocid, terms);
-}
-bool Query::getMatchTerms(unsigned long xdocid, list<string>& terms)
-{
-    if (ISNULL(m_nq) || !m_nq->xenquire) {
-	LOGERR(("Query::getMatchTerms: no query opened\n"));
-	return -1;
+    LOGDEB(("makeDocAbstract: maxoccs %d ctxwords %d\n", maxoccs, ctxwords));
+    if (!m_db || !m_db->m_ndb || !m_db->m_ndb->m_isopen || !m_nq) {
+	LOGERR(("Query::makeDocAbstract: no db or no nq\n"));
+	return ABSRES_ERROR;
     }
-
-    terms.clear();
-    Xapian::TermIterator it;
-    Xapian::docid id = Xapian::docid(xdocid);
-
-    XAPTRY(terms.insert(terms.begin(),
-                        m_nq->xenquire->get_matching_terms_begin(id),
-                        m_nq->xenquire->get_matching_terms_end(id)),
+    int ret = ABSRES_ERROR;
+    XAPTRY(ret = m_nq->makeAbstract(doc.xdocid, abstract, maxoccs, ctxwords),
            m_db->m_ndb->xrdb, m_reason);
-
     if (!m_reason.empty()) {
-	LOGERR(("getQueryTerms: xapian error: %s\n", m_reason.c_str()));
+	LOGDEB(("makeDocAbstract: makeAbstract error, reason: %s\n", 
+		m_reason.c_str()));
+	return ABSRES_ERROR;
+    }
+    return ret;
+}
+
+bool Query::makeDocAbstract(const Doc &doc, vector<string>& abstract)
+{
+    vector<Snippet> vpabs;
+    if (!makeDocAbstract(doc, vpabs))
+	return false;
+    for (vector<Snippet>::const_iterator it = vpabs.begin();
+	 it != vpabs.end(); it++) {
+	string chunk;
+	if (it->page > 0) {
+	    ostringstream ss;
+	    ss << it->page;
+	    chunk += string(" [p ") + ss.str() + "] ";
+	}
+	chunk += it->snippet;
+	abstract.push_back(chunk);
+    }
+    return true;
+}
+
+bool Query::makeDocAbstract(const Doc &doc, string& abstract)
+{
+    vector<Snippet> vpabs;
+    if (!makeDocAbstract(doc, vpabs))
+	return false;
+    for (vector<Snippet>::const_iterator it = vpabs.begin(); 
+	 it != vpabs.end(); it++) {
+	abstract.append(it->snippet);
+	abstract.append(cstr_ellipsis);
+    }
+    return m_reason.empty() ? true : false;
+}
+
+int Query::getFirstMatchPage(const Doc &doc, string& term)
+{
+    LOGDEB1(("Db::getFirstMatchPage\n"));;
+    if (!m_nq) {
+	LOGERR(("Query::getFirstMatchPage: no nq\n"));
 	return false;
     }
-
-    return true;
+    int pagenum = -1;
+    XAPTRY(pagenum = m_nq->getFirstMatchPage(Xapian::docid(doc.xdocid), term),
+	   m_db->m_ndb->xrdb, m_reason);
+    return m_reason.empty() ? pagenum : -1;
 }
 
 
@@ -380,13 +420,7 @@ bool Query::getDoc(int xapi, Doc &doc)
             data = xdoc.get_data();
             m_reason.erase();
             Chrono chron;
-            Xapian::TermIterator it = xdoc.termlist_begin();
-            it.skip_to("Q");
-            if (it != xdoc.termlist_end()) {
-                udi = *it;
-                if (!udi.empty())
-                    udi = udi.substr(1);
-            }
+	    m_db->m_ndb->xdocToUdi(xdoc, udi);
             LOGDEB2(("Query::getDoc: %d ms for udi [%s], collapse count %d\n", 
 		     chron.millis(), udi.c_str(), collapsecount));
             break;
@@ -406,25 +440,26 @@ bool Query::getDoc(int xapi, Doc &doc)
 
     doc.pc = pc;
     char buf[200];
-    if (collapsecount>0) {
-	sprintf(buf,"%3d%% (%d)", pc, collapsecount+1);
+    if (collapsecount > 0) {
+	sprintf(buf,"%3d%% (%d)", pc, collapsecount + 1);
     } else {
 	sprintf(buf,"%3d%%", pc);
     }
     doc.meta[Doc::keyrr] = buf;
 
-    sprintf(buf, "%d", collapsecount);
-    doc.meta[Rcl::Doc::keycc] = buf;
-
+    if (collapsecount > 0) {
+	sprintf(buf, "%d", collapsecount);
+	doc.meta[Rcl::Doc::keycc] = buf;
+    }
 
     // Parse xapian document's data and populate doc fields
     return m_db->m_ndb->dbDataToRclDoc(docid, data, doc);
 }
 
-list<string> Query::expand(const Doc &doc)
+vector<string> Query::expand(const Doc &doc)
 {
     LOGDEB(("Rcl::Query::expand()\n"));
-    list<string> res;
+    vector<string> res;
     if (ISNULL(m_nq) || !m_nq->xenquire) {
 	LOGERR(("Query::expand: no query opened\n"));
 	return res;
@@ -441,7 +476,7 @@ list<string> Query::expand(const Doc &doc)
 	    for (Xapian::ESetIterator it = eset.begin(); 
 		 it != eset.end(); it++) {
 		LOGDEB((" [%s]\n", (*it).c_str()));
-		if ((*it).empty() || ((*it).at(0)>='A' && (*it).at(0)<='Z'))
+		if ((*it).empty() || has_prefix(*it))
 		    continue;
 		res.push_back(*it);
 		if (res.size() >= 10)
