@@ -21,6 +21,7 @@
 #include <stdio.h>
 #include <unistd.h>
 #include <fcntl.h>
+#include <dirent.h>
 #include <sys/param.h>
 #include <pwd.h>
 #include <math.h>
@@ -28,6 +29,7 @@
 #include <sys/types.h>
 #include <sys/file.h>
 #include <sys/stat.h>
+#include <glob.h>
 
 // Let's include all files where statfs can be defined and hope for no
 // conflict...
@@ -47,13 +49,10 @@
 #include <cstdlib>
 #include <cstring>
 #include <iostream>
-#include <list>
+#include <sstream>
 #include <stack>
-#ifndef NO_NAMESPACES
-using std::string;
-using std::list;
-using std::stack;
-#endif /* NO_NAMESPACES */
+#include <set>
+using namespace std;
 
 #include "pathut.h"
 #include "transcode.h"
@@ -96,14 +95,18 @@ bool fsocc(const string &path, int *pc, long *blocks)
     return true;
 }
 
-static const char *tmplocation()
+const string& tmplocation()
 {
-    const char *tmpdir = getenv("RECOLL_TMPDIR");
-    if (!tmpdir)
-	tmpdir = getenv("TMPDIR");
-    if (!tmpdir)
-	tmpdir = "/tmp";
-    return tmpdir;
+    static string stmpdir;
+    if (stmpdir.empty()) {
+        const char *tmpdir = getenv("RECOLL_TMPDIR");
+        if (tmpdir == 0) 
+            tmpdir = getenv("TMPDIR");
+        if (tmpdir == 0)
+            tmpdir = "/tmp";
+        stmpdir = string(tmpdir);
+    }
+    return stmpdir;
 }
 
 bool maketmpdir(string& tdir, string& reason)
@@ -145,6 +148,7 @@ bool maketmpdir(string& tdir, string& reason)
 }
 
 TempFileInternal::TempFileInternal(const string& suffix)
+    : m_noremove(false)
 {
     string filename = path_cat(tmplocation(), "rcltmpfXXXXXX");
     char *cp = strdup(filename.c_str());
@@ -176,7 +180,7 @@ TempFileInternal::TempFileInternal(const string& suffix)
 
 TempFileInternal::~TempFileInternal()
 {
-    if (!m_filename.empty())
+    if (!m_filename.empty() && !m_noremove)
 	unlink(m_filename.c_str());
 }
 
@@ -270,6 +274,14 @@ string path_basename(const string &s, const string &suff)
     return simple;
 }
 
+string path_suffix(const string& s)
+{
+    string::size_type dotp = s.rfind('.');
+    if (dotp == string::npos)
+	return string();
+    return s.substr(dotp+1);
+}
+
 string path_home()
 {
     uid_t uid = getuid();
@@ -323,17 +335,22 @@ extern string path_absolute(const string &is)
 }
 
 #include <smallut.h>
-extern string path_canon(const string &is)
+extern string path_canon(const string &is, const string* cwd)
 {
     if (is.length() == 0)
 	return is;
     string s = is;
     if (s[0] != '/') {
 	char buf[MAXPATHLEN];
-	if (!getcwd(buf, MAXPATHLEN)) {
-	    return string();
+	const char *cwdp = buf;
+	if (cwd) {
+	    cwdp = cwd->c_str();
+	} else {
+	    if (!getcwd(buf, MAXPATHLEN)) {
+		return string();
+	    }
 	}
-	s = path_cat(string(buf), s); 
+	s = path_cat(string(cwdp), s); 
     }
     vector<string> elems;
     stringToTokens(s, elems, "/");
@@ -361,12 +378,30 @@ extern string path_canon(const string &is)
     return ret;
 }
 
-#include <glob.h>
-#include <sys/stat.h>
-list<string> path_dirglob(const string &dir, 
-				    const string pattern)
+bool makepath(const string& ipath)
 {
-    list<string> res;
+    string path = path_canon(ipath);
+    vector<string> elems;
+    stringToTokens(path, elems, "/");
+    path = "/";
+    for (vector<string>::const_iterator it = elems.begin(); 
+	 it != elems.end(); it++){
+	path += *it;
+	// Not using path_isdir() here, because this cant grok symlinks
+	// If we hit an existing file, no worry, mkdir will just fail.
+	if (access(path.c_str(), 0) != 0) {
+	    if (mkdir(path.c_str(), 0700) != 0)  {
+		return false;
+	    }
+	}
+	path += "/";
+    }
+    return true;
+}
+
+vector<string> path_dirglob(const string &dir, const string pattern)
+{
+    vector<string> res;
     glob_t mglob;
     string mypat=path_cat(dir, pattern);
     if (glob(mypat.c_str(), 0, 0, &mglob)) {
@@ -440,7 +475,7 @@ string url_encode(const string& url, string::size_type offs)
     string out = url.substr(0, offs);
     const char *cp = url.c_str();
     for (string::size_type i = offs; i < url.size(); i++) {
-	int c;
+	unsigned int c;
 	const char *h = "0123456789ABCDEF";
 	c = cp[i];
 	if (c <= 0x20 || 
@@ -505,6 +540,10 @@ string fileurltolocalpath(string url)
     }
     return url;
 }
+bool urlisfileurl(const string& url)
+{
+    return url.find("file://") == 0;
+}
 
 // Printable url: this is used to transcode from the system charset
 // into either utf-8 if transcoding succeeds, or url-encoded
@@ -517,7 +556,53 @@ bool printableUrl(const string &fcharset, const string &in, string &out)
     return true;
 }
 
+bool readdir(const string& dir, string& reason, set<string>& entries)
+{
+    struct stat st;
+    int statret;
+    ostringstream msg;
+    DIR *d = 0;
+    statret = lstat(dir.c_str(), &st);
+    if (statret == -1) {
+	msg << "readdir: cant stat " << dir << " errno " <<  errno;
+	goto out;
+    }
+    if (!S_ISDIR(st.st_mode)) {
+	msg << "readdir: " << dir <<  " not a directory";
+	goto out;
+    }
+    if (access(dir.c_str(), R_OK) < 0) {
+	msg << "readdir: no read access to " << dir;
+	goto out;
+    }
 
+    d = opendir(dir.c_str());
+    if (d == 0) {
+	msg << "readdir: cant opendir " << dir << ", errno " << errno;
+	goto out;
+    }
+
+    struct dirent *ent;
+    while ((ent = readdir(d)) != 0) {
+	if (!strcmp(ent->d_name, ".") || !strcmp(ent->d_name, "..")) 
+	    continue;
+	entries.insert(ent->d_name);
+    }
+
+out:
+    if (d)
+	closedir(d);
+    reason = msg.str();
+    if (reason.empty())
+	return true;
+    return false;
+}
+
+// We do not want to mess with the pidfile content in the destructor:
+// the lock might still be in use in a child process. In fact as much
+// as we'd like to reset the pid inside the file when we're done, it
+// would be very difficult to do it right and it's probably best left
+// alone.
 Pidfile::~Pidfile()
 {
     if (m_fd >= 0)
@@ -550,7 +635,7 @@ int Pidfile::flopen()
 {
     const char *path = m_path.c_str();
     if ((m_fd = ::open(path, O_RDWR|O_CREAT, 0644)) == -1) {
-	m_reason = "Open failed";
+	m_reason = "Open failed: [" + m_path + "]: " + strerror(errno);
 	return -1;
     }
 
@@ -624,12 +709,36 @@ int Pidfile::remove()
     return unlink(m_path.c_str());
 }
 
-// Freedesktop standard paths for thumbnails
+
+// Freedesktop standard paths for cache directory (thumbnails are now in there)
+static const string& xdgcachedir()
+{
+    static string xdgcache;
+    if (xdgcache.empty()) {
+	const char *cp = getenv("XDG_CACHE_HOME");
+	if (cp == 0) 
+	    xdgcache = path_cat(path_home(), ".cache");
+	else
+	    xdgcache = string(cp);
+    }
+    return xdgcache;
+}
+static const string& thumbnailsdir()
+{
+    static string thumbnailsd;
+    if (thumbnailsd.empty()) {
+	thumbnailsd = path_cat(xdgcachedir(), "thumbnails");
+	if (access(thumbnailsd.c_str(), 0) != 0) {
+	    thumbnailsd = path_cat(path_home(), ".thumbnails");
+	}
+    }
+    return thumbnailsd;
+}
 
 // Place for 256x256 files
-static const string thmbdirlarge = ".thumbnails/large";
+static const string thmbdirlarge = "large";
 // 128x128
-static const string thmbdirnormal = ".thumbnails/normal";
+static const string thmbdirnormal = "normal";
 
 static void thumbname(const string& url, string& name)
 {
@@ -645,13 +754,13 @@ bool thumbPathForUrl(const string& url, int size, string& path)
     string name;
     thumbname(url, name);
     if (size <= 128) {
-	path = path_cat(path_home(), thmbdirnormal);
+	path = path_cat(thumbnailsdir(), thmbdirnormal);
 	path = path_cat(path, name);
 	if (access(path.c_str(), R_OK) == 0) {
 	    return true;
 	}
     } 
-    path = path_cat(path_home(), thmbdirlarge);
+    path = path_cat(thumbnailsdir(), thmbdirlarge);
     path = path_cat(path, name);
     if (access(path.c_str(), R_OK) == 0) {
 	return true;
@@ -666,6 +775,15 @@ bool thumbPathForUrl(const string& url, int size, string& path)
     return false;
 }
 
+// Call funcs that need static init (not initially reentrant)
+void pathut_init_mt()
+{
+    path_home();
+    tmplocation();
+    thumbnailsdir();
+}
+
+
 #else // TEST_PATHUT
 #include <stdlib.h>
 #include <iostream>
@@ -690,7 +808,7 @@ int main(int argc, const char **argv)
     thisprog = *argv++;argc--;
 
     string s;
-    list<string>::const_iterator it;
+    vector<string>::const_iterator it;
 #if 0
     for (unsigned int i = 0;i < sizeof(tstvec) / sizeof(char *); i++) {
 	cout << tstvec[i] << " Father " << path_getfather(tstvec[i]) << endl;
@@ -730,7 +848,7 @@ int main(int argc, const char **argv)
     }
     string dir = *argv++;argc--;
     string pattern =  *argv++;argc--;
-    list<string> matched = path_dirglob(dir, pattern);
+    vector<string> matched = path_dirglob(dir, pattern);
     for (it = matched.begin(); it != matched.end();it++) {
 	cout << *it << endl;
     }
@@ -766,7 +884,7 @@ int main(int argc, const char **argv)
   pidfile.remove();
 #endif
 
-#if 1
+#if 0
   if (argc != 2) {
       fprintf(stderr, "Usage: thumbpath <filepath> <size>\n");
       exit(1);
@@ -795,7 +913,16 @@ int main(int argc, const char **argv)
   exit(0);
 #endif
 
+#if 1
+    if (argc != 1) {
+	cerr << "Usage: trpathut <filename>" << endl;
+	exit(1);
+    }
+    string fn = *argv++;argc--;
+    string ext = path_suffix(fn);
+    cout << "Suffix: [" << ext << "]" << endl;
     return 0;
+#endif
 }
 
 #endif // TEST_PATHUT
