@@ -25,22 +25,23 @@
 #include <ctype.h>
 #include <unistd.h>
 #include <errno.h>
-#include <sys/time.h>
+#include <time.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <string.h>
+#include <math.h>
 
 #include <string>
 #include <iostream>
+#include <list>
+#include <tr1/unordered_map>
+using std::tr1::unordered_map;
+using namespace std;
 
 #include "smallut.h"
 #include "utf8iter.h"
-
-#ifndef NO_NAMESPACES
-using namespace std;
-#endif /* NO_NAMESPACES */
-
-#define MIN(A,B) ((A)<(B)?(A):(B))
+#include "hldata.h"
+#include "cstr.h"
 
 int stringicmp(const string & s1, const string& s2) 
 {
@@ -319,6 +320,15 @@ template <class T> void stringsToString(const T &tokens, string &s)
 template void stringsToString<list<string> >(const list<string> &, string &);
 template void stringsToString<vector<string> >(const vector<string> &,string &);
 template void stringsToString<set<string> >(const set<string> &, string &);
+template <class T> string stringsToString(const T &tokens)
+{
+    string out;
+    stringsToString<T>(tokens, out);
+    return out;
+}
+template string stringsToString<list<string> >(const list<string> &);
+template string stringsToString<vector<string> >(const vector<string> &);
+template string stringsToString<set<string> >(const set<string> &);
 
 template <class T> void stringsToCSV(const T &tokens, string &s, 
 				     char sep)
@@ -593,20 +603,22 @@ string displayableBytes(off_t size)
 {
     char sizebuf[50];
     const char *unit;
-
+    
+    double roundable = 0;
     if (size < 1000) {
 	unit = " B ";
+	roundable = double(size);
     } else if (size < 1E6) {
 	unit = " KB ";
-	size /= 1000;
+	roundable = double(size) / 1E3;
     } else if (size < 1E9) {
 	unit = " MB ";
-	size /= (1E6);
+	roundable = double(size) / 1E6;
     } else {
 	unit = " GB ";
-	size /= (1E9);
+	roundable = double(size) / 1E9;
     }
-
+    size = round(roundable);
     sprintf(sizebuf, OFFTPC "%s", size, unit);
     return string(sizebuf);
 }
@@ -647,39 +659,57 @@ string breakIntoLines(const string& in, unsigned int ll,
 }
 
 ////////////////////
+
+#if 0
 // Internal redefinition of system time interface to help with dependancies
 struct m_timespec {
   time_t tv_sec;
   long   tv_nsec;
 };
+#endif
 
 #ifndef CLOCK_REALTIME
+typedef int clockid_t;
 #define CLOCK_REALTIME 1
 #endif
 
-#define MILLIS(TV) ( (long)(((TV).tv_sec - m_secs) * 1000 + \
+#define MILLIS(TV) ( (long)(((TV).tv_sec - m_secs) * 1000L + \
   ((TV).tv_nsec - m_nsecs) / 1000000))
-
-#define MICROS(TV) ( (long)(((TV).tv_sec - m_secs) * 1000000 + \
+#define MICROS(TV) ( (long)(((TV).tv_sec - m_secs) * 1000000L + \
   ((TV).tv_nsec - m_nsecs) / 1000))
+#define NANOS(TV) ( (long long)(((TV).tv_sec - m_secs) * 1000000000LL + \
+  ((TV).tv_nsec - m_nsecs)))
 
+// Using clock_gettime() is nice because it gives us ns res and it helps with
+// computing threads work times, but it's also a pita because it forces linking
+// with -lrt. So keep it optional. And not on the mac anyway
+// #define USE_CLOCK_GETTIME
 
-// We use gettimeofday instead of clock_gettime for now and get only
-// uS resolution, because clock_gettime is more configuration trouble
-// than it's worth
-static void gettime(int, struct m_timespec *ts)
+#ifdef __APPLE__
+#undef USE_CLOCK_GETTIME
+#endif
+
+#ifndef USE_CLOCK_GETTIME
+#include <sys/time.h>
+#endif
+
+static void gettime(clockid_t clk_id, struct timespec *ts)
 {
-  struct timeval tv;
-  gettimeofday(&tv, 0);
-  ts->tv_sec = tv.tv_sec;
-  ts->tv_nsec = tv.tv_usec * 1000;
+#ifndef USE_CLOCK_GETTIME
+    struct timeval tv;
+    gettimeofday(&tv, 0);
+    ts->tv_sec = tv.tv_sec;
+    ts->tv_nsec = tv.tv_usec * 1000;
+#else
+    clock_gettime(clk_id, ts);
+#endif
 }
 ///// End system interface
 
 // Note: this not protected against multithread access and not reentrant, but
 // this is mostly debug code, and it won't crash, just show bad results. Also 
 // the frozen thing is not used that much
-static m_timespec frozen_tv;
+static timespec frozen_tv;
 void Chrono::refnow()
 {
   gettime(CLOCK_REALTIME, &frozen_tv);
@@ -693,7 +723,7 @@ Chrono::Chrono()
 // Reset and return value before rest in milliseconds
 long Chrono::restart()
 {
-  struct m_timespec tv;
+  struct timespec tv;
   gettime(CLOCK_REALTIME, &tv);
   long ret = MILLIS(tv);
   m_secs = tv.tv_sec;
@@ -704,30 +734,29 @@ long Chrono::restart()
 // Get current timer value, milliseconds
 long Chrono::millis(int frozen)
 {
-  if (frozen) {
-    return MILLIS(frozen_tv);
-  } else {
-    struct m_timespec tv;
-    gettime(CLOCK_REALTIME, &tv);
-    return MILLIS(tv);
-  }
+    return nanos() / 1000000;
 }
 
 //
 long Chrono::micros(int frozen)
+{
+    return nanos() / 1000;
+}
+
+long long Chrono::nanos(int frozen)
 {
   if (frozen) {
-    return MICROS(frozen_tv);
+    return NANOS(frozen_tv);
   } else {
-    struct m_timespec tv;
+    struct timespec tv;
     gettime(CLOCK_REALTIME, &tv);
-    return MICROS(tv);
+    return NANOS(tv);
   }
 }
 
 float Chrono::secs(int frozen)
 {
-  struct m_timespec tv;
+  struct timespec tv;
   gettime(CLOCK_REALTIME, &tv);
   float secs = (float)(frozen?frozen_tv.tv_sec:tv.tv_sec - m_secs);
   float nsecs = (float)(frozen?frozen_tv.tv_nsec:tv.tv_nsec - m_nsecs); 
@@ -1037,8 +1066,126 @@ void catstrerror(string *reason, const char *what, int _errno)
 #endif
 }
 
+void HighlightData::toString(std::string& out)
+{
+    out.append("\nUser terms (orthograph): ");
+    for (std::set<std::string>::const_iterator it = uterms.begin();
+	 it != uterms.end(); it++) {
+	out.append(" [").append(*it).append("]");
+    }
+    out.append("\nUser terms to Query terms:");
+    for (map<string, string>::const_iterator it = terms.begin();
+	 it != terms.end(); it++) {
+	out.append("[").append(it->first).append("]->[");
+	out.append(it->second).append("] ");
+    }
+    out.append("\nGroups: ");
+    char cbuf[200];
+    sprintf(cbuf, "Groups size %d grpsugidx size %d ugroups size %d",
+	    int(groups.size()), int(grpsugidx.size()), int(ugroups.size()));
+    out.append(cbuf);
 
-#else
+    unsigned int ugidx = (unsigned int)-1;
+    for (unsigned int i = 0; i < groups.size(); i++) {
+	if (ugidx != grpsugidx[i]) {
+	    ugidx = grpsugidx[i];
+	    out.append("\n(");
+	    for (unsigned int j = 0; j < ugroups[ugidx].size(); j++) {
+		out.append("[").append(ugroups[ugidx][j]).append("] ");
+	    }
+	    out.append(") ->");
+	}
+	out.append(" {");
+	for (unsigned int j = 0; j < groups[i].size(); j++) {
+	    out.append("[").append(groups[i][j]).append("]");
+	}
+	sprintf(cbuf, "%d", slacks[i]);
+	out.append("}").append(cbuf);
+    }
+    out.append("\n");
+}
+
+void HighlightData::append(const HighlightData& hl)
+{
+    uterms.insert(hl.uterms.begin(), hl.uterms.end());
+    terms.insert(hl.terms.begin(), hl.terms.end());
+    size_t ugsz0 = ugroups.size();
+    ugroups.insert(ugroups.end(), hl.ugroups.begin(), hl.ugroups.end());
+
+    groups.insert(groups.end(), hl.groups.begin(), hl.groups.end());
+    slacks.insert(slacks.end(), hl.slacks.begin(), hl.slacks.end());
+    for (std::vector<unsigned int>::const_iterator it = hl.grpsugidx.begin(); 
+	 it != hl.grpsugidx.end(); it++) {
+	grpsugidx.push_back(*it + ugsz0);
+    }
+}
+
+static const char *vlang_to_code[] = {
+    "be", "cp1251",
+    "bg", "cp1251",
+    "cs", "iso-8859-2",
+    "el", "iso-8859-7",
+    "he", "iso-8859-8",
+    "hr", "iso-8859-2",
+    "hu", "iso-8859-2",
+    "ja", "eucjp",
+    "kk", "pt154",
+    "ko", "euckr",
+    "lt", "iso-8859-13",
+    "lv", "iso-8859-13",
+    "pl", "iso-8859-2",
+    "rs", "iso-8859-2",
+    "ro", "iso-8859-2",
+    "ru", "koi8-r",
+    "sk", "iso-8859-2",
+    "sl", "iso-8859-2",
+    "sr", "iso-8859-2",
+    "th", "iso-8859-11",
+    "tr", "iso-8859-9",
+    "uk", "koi8-u",
+};
+
+string langtocode(const string& lang)
+{
+    static unordered_map<string, string> lang_to_code;
+    if (lang_to_code.empty()) {
+	for (unsigned int i = 0; 
+	     i < sizeof(vlang_to_code) / sizeof(char *); i += 2) {
+	    lang_to_code[vlang_to_code[i]] = vlang_to_code[i+1];
+	}
+    }
+    unordered_map<string,string>::const_iterator it = 
+	lang_to_code.find(lang);
+
+    // Use cp1252 by default...
+    if (it == lang_to_code.end())
+	return cstr_cp1252;
+
+    return it->second;
+}
+
+string localelang()
+{
+    const char *lang = getenv("LANG");
+
+    if (lang == 0 || *lang == 0 || !strcmp(lang, "C") || !strcmp(lang, "POSIX"))
+	return "en";
+    string locale(lang);
+    string::size_type under = locale.find_first_of("_");
+    if (under == string::npos)
+	return locale;
+    return locale.substr(0, under);
+}
+
+// Initialization for static stuff to be called from main thread before going 
+// multiple
+void smallut_init_mt()
+{
+    // Init langtocode() static table
+    langtocode("");
+}
+
+#else // TEST_SMALLUT
 
 #include <string>
 using namespace std;
