@@ -20,14 +20,12 @@
 #include <stdlib.h>
 
 #include <string>
-#include <list>
 #include <vector>
 #include <map>
 #include <set>
 
 #ifndef NO_NAMESPACES
 using std::string;
-using std::list;
 using std::vector;
 using std::map;
 using std::set;
@@ -55,6 +53,11 @@ extern string stringtolower(const string& io);
 // Is one string the end part of the other ?
 extern int stringisuffcmp(const string& s1, const string& s2);
 
+// Divine language from locale
+extern std::string localelang();
+// Divine 8bit charset from language
+extern std::string langtocode(const string& lang);
+
 // Compare charset names, removing the more common spelling variations
 extern bool samecharset(const string &cs1, const string &cs2);
 
@@ -92,6 +95,7 @@ template <class T> bool stringToStrings(const string& s, T &tokens,
  * Inverse operation:
  */
 template <class T> void stringsToString(const T &tokens, string &s);
+template <class T> std::string stringsToString(const T &tokens);
 
 /**
  * Strings to CSV string. tokens containing the separator are quoted (")
@@ -163,6 +167,7 @@ class Chrono {
   long millis(int frozen = 0);
   long ms() {return millis();}
   long micros(int frozen = 0);
+  long long nanos(int frozen = 0);
   float secs(int frozen = 0);
  private:
   long	m_secs;
@@ -194,13 +199,9 @@ inline void leftzeropad(string& s, unsigned len)
 	s = s.insert(0, len - s.length(), '0');
 }
 
-#ifndef deleteZ
-#define deleteZ(X) {delete X;X = 0;}
-#endif
-
 // Code for static initialization of an stl map. Somewhat like Boost.assign. 
 // Ref: http://stackoverflow.com/questions/138600/initializing-a-static-stdmapint-int-in-c
-// Example use: map<int, int> m = map_list_of (1,2) (3,4) (5,6) (7,8);
+// Example use: map<int, int> m = create_map<int, int> (1,2) (3,4) (5,6) (7,8);
 
 template <typename T, typename U>
 class create_map
@@ -224,5 +225,39 @@ public:
         return m_map;
     }
 };
+template <typename T>
+class create_vector
+{
+private:
+    std::vector<T> m_vector;
+public:
+    create_vector(const T& val)
+    {
+        m_vector.push_back(val);
+    }
+
+    create_vector<T>& operator()(const T& val)
+    {
+        m_vector.push_back(val);
+        return *this;
+    }
+
+    operator std::vector<T>()
+    {
+        return m_vector;
+    }
+};
+
+#ifndef MIN
+#define MIN(A,B) (((A)<(B)) ? (A) : (B))
+#endif
+#ifndef MAX
+#define MAX(A,B) (((A)>(B)) ? (A) : (B))
+#endif
+#ifndef deleteZ
+#define deleteZ(X) {delete X;X = 0;}
+#endif
+
+void smallut_init_mt();
 
 #endif /* _SMALLUT_H_INCLUDED_ */
